@@ -174,7 +174,9 @@ returned."
 (defun mew-addrbook-alias-add (addr)
   ;; Let's downcase user-name for .mew-alias since it is automatically
   ;; maintained.
-  (when (and (stringp addr) (string-match "@" addr))
+  (when (and (stringp addr) (string-match "@" addr)
+	     (or (null mew-addrbook-alias-not-learn-list)
+		 (not (mew-member-match addr mew-addrbook-alias-not-learn-list))))
     (let* ((user (downcase (mew-addrstr-extract-user addr)))
 	   (match-auto (mew-autoent-by-shortname user))
 	   (match-adbk-orig (mew-adbkorigent-by-shortname user)))
@@ -340,7 +342,7 @@ it will add an alias."
 	from shortname addrs name)
     (save-excursion
       (set-buffer buf)
-      (unless (mew-header-p)
+      (unless (mew-msghder-p)
 	(setq buf (mew-cache-hit fld msg 'must-hit))
 	(set-buffer buf))
       (setq from (mew-header-get-value mew-from:))
@@ -447,7 +449,7 @@ The keys that are defined for this mode are:
        ((and name (string-match "^\"[^\"]*[^\000-\177]" name))
 	(setq errmsg "Remove quote around non-ASCII Name")))))
     (if errmsg
-	(message errmsg)
+	(message "%s" errmsg)
       (save-excursion
 	(setq buf (find-file-noselect
 		   (expand-file-name mew-addrbook-file mew-conf-path)))
