@@ -55,10 +55,10 @@ attachments region.
   (let (ret max)
     (save-excursion
       (beginning-of-line)
-      (setq ret (mew-count-lines (1+ (mew-attach-begin)) (point)))
+      (setq ret (count-lines (1+ (mew-attach-begin)) (point)))
       (goto-char (point-max))
       (beginning-of-line)
-      (setq max (mew-count-lines (1+ (mew-attach-begin)) (point)))
+      (setq max (count-lines (1+ (mew-attach-begin)) (point)))
       (if (or (= ret max) (= ret (1- max))) -1 ret))))
 
 
@@ -67,7 +67,7 @@ attachments region.
     (save-excursion
       (goto-char (point-max))
       (beginning-of-line)
-      (= (mew-count-lines here (point)) 2))))
+      (= (count-lines here (point)) 2))))
 
 (defun mew-attach-move-onto-file ()
   (end-of-line)
@@ -338,7 +338,7 @@ attachments region.
 	     (mew-encode-syntax-single tofile (list ct) nil nil nil 'cdp)))
       (mew-encode-syntax-print mew-encode-syntax)
       (if mew-attach-move-next-after-copy (mew-attach-next))
-      (if msg (message msg)))))
+      (if msg (message "%s" msg)))))
 
 (defun mew-attach-link (&optional from to)
   "Link a file with a symbolic link on '.' in attachments.
@@ -441,6 +441,7 @@ on '.' in attachments."
 		(if (not (y-or-n-p "Are you ready? "))
 		    (message "Nothing is recoded")
 		  (message
+		   "%s"
 		   (substitute-command-keys
 		    "Type '\\[keyboard-quit]' to finish recording..."))
 		  (mew-plet
@@ -450,6 +451,7 @@ on '.' in attachments."
 			  (nth 1 mew-prog-audio2)))))
 	    (quit
 	     (message
+	      "%s"
 	      (substitute-command-keys
 	       "Type '\\[keyboard-quit]' to finish recording...done"))
 	     (setq recorded t)
@@ -574,7 +576,7 @@ on '.' in attachments."
 	   (file (read-string "Filename: " oldfile)))
       (if (string= file "")
 	  (mew-syntax-set-cdp syntax nil)
-	(mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file)))
+	(mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file nil 'force-file)))
       (mew-encode-syntax-print mew-encode-syntax))))
 
 (defun mew-attach-base64 ()
