@@ -218,7 +218,7 @@
 
 (defun mew-header-encode-string (str &optional key-len)
   (let* ((ecsdb (or (mew-ecsdb-guess-string str)
-		    (mew-charset-to-ecsdb mew-charset-m17n)))
+		    (mew-charset-to-ecsdb (mew-charset-m17n))))
 	 (hcs (mew-ecsdb-hcs ecsdb))
 	 charset b-or-q fun)
     (if (not (mew-coding-system-p hcs))
@@ -334,10 +334,10 @@
 	      (throw 'quote nil))
 	     ((> c 127)
 	      (mew-encode-error
-	       "Only ASCII is allowed in quoted-string in the header. ")))
+	       "Only ASCII is allowed in quoted-string in the header")))
 	    (setq I (1+ I))))
 	(if open
-	    (mew-encode-error "Quote string must be closed in the header. "))
+	    (mew-encode-error "Quote string must be closed in the header"))
 	(mew-header-encode-cond ?a)
 	(setq i I))
        ;; end of quote
@@ -364,9 +364,9 @@
 	      (setq i (1+ i))))
 	  (if (and qp (eq status 'non-ascii))
 	      (mew-encode-error
-	       "Only ASCII is allowed in quoted-string in the header. ")))
+	       "Only ASCII is allowed in quoted-string in the header")))
 	(if open
-	    (mew-encode-error "Comment must be closed in the header. "))
+	    (mew-encode-error "Comment must be closed in the header"))
 	(mew-header-encode-cond2 'comment)
 	(if (= i len)
 	    ()
@@ -393,19 +393,19 @@
 	      (setq open nil)
 	      (throw 'route nil))
 	     ((> c 127)
-	      (mew-encode-error "<> must contain ASCII only. "))
+	      (mew-encode-error "<> must contain ASCII only"))
 	     (t
 	      (insert c)))
 	    (setq i (1+ i))))
 	(if open
-	    (mew-encode-error "<> must be closed in the header. "))
+	    (mew-encode-error "<> must be closed in the header"))
 	(if (= i len)
 	    ()
 	  (insert ">")
 	  (setq bound (1+ i)))
 	(setq status 'space))
        ((char-equal c ?>)
-	(mew-encode-error "Unbalanced <> in the header. "))
+	(mew-encode-error "Unbalanced <> in the header"))
        ;; end of <>
 
        ;; group
@@ -424,7 +424,7 @@
 	(insert ":")
 	(setq i (1+ i))
 	(unless (char-equal (aref str i) ?\;)
-	  (mew-encode-error ": must be followed by ; in the header. "))
+	  (mew-encode-error ": must be followed by ; in the header"))
 	(insert ";")
 	(setq bound (1+ i))
 	(setq status 'space))
@@ -620,6 +620,9 @@
     (beginning-of-line)
     (char-equal (following-char) ?\")))
 
+(defsubst mew-header-decode-in-addr-spec ()
+  (eq (get-text-property (match-beginning 0) 'mew-addr-spec) t))
+
 (defun mew-decode-warning-body (err msg)
   (mew-xinfo-set-warning
    (cons 
@@ -668,7 +671,7 @@
 
 (defun mew-header-decode-region (key rbeg rend &optional draftp)
   "RFC 2047 decoding. This is liberal on the one point from RFC 2047.
-That is, each line may be more than 75. "
+That is, each line may be more than 75."
   (setq key (capitalize key))
   (let* ((type (mew-field-type-for-decoding key)) (nl "\0"))
     (save-restriction
@@ -707,8 +710,8 @@ That is, each line may be more than 75. "
 	  ;; encoded-word in quoted-string should not be decoded
 	  ;; according to RFC 2047. However, if users wish
 	  ;; (ie mew-decode-broken is *non-nil*), decode it.
-	  ;; Regular expression cannot express quoted string. So,
-	  ;; canonicalize as follows:
+	  ;; Regular expression cannot express a quoted string in 
+	  ;; general case. So, canonicalize as follows:
 	  ;;     Not quoted line (may be null line)
           ;;     "quoted line"
 	  ;;     Not quoted line
@@ -718,6 +721,12 @@ That is, each line may be more than 75. "
 	  (goto-char (point-min))
 	  (while (re-search-forward "\n" nil t)
 	    (replace-match nl nil t))
+	  ;;
+	  (goto-char (point-min))
+	  (while (re-search-forward mew-regex-id nil t)
+	    (put-text-property (match-beginning 0) (match-end 0)
+			       'mew-addr-spec t))
+	  ;;
 	  (goto-char (point-min))
 	  ;; Perfect regular expression for quoted strings.
 	  ;; See "Marstering Regular Expressions", O'REILLY
@@ -730,22 +739,27 @@ That is, each line may be more than 75. "
 	  (goto-char (point-min)) ;; anyway
 	  ;;
 	  (while (re-search-forward mew-header-decode-regex nil t)
-	    (if (mew-header-decode-in-quotedp)
-		(when mew-decode-broken
-		  (mew-decode-warning-fields key 'quoted)
-		  (mew-header-decode-string key)
-		  (while (looking-at mew-header-decode-regex2)
-		    (mew-header-decode-string key)))
+	    (cond
+	     ((mew-header-decode-in-addr-spec)
+	      ) ;; do nothing
+	     ((mew-header-decode-in-quotedp)
+	      (when mew-decode-broken
+		(mew-decode-warning-fields key 'quoted)
+		(mew-header-decode-string key)
+		(while (looking-at mew-header-decode-regex2)
+		  (mew-header-decode-string key))))
+	     (t
 	      (mew-header-decode-string key)
 	      (while (looking-at mew-header-decode-regex2)
-		(mew-header-decode-string key))))
+		(mew-header-decode-string key)))))
 	  ;; Getting back to the original
 	  (goto-char (point-min))
 	  (while (re-search-forward "\n" nil t)
 	    (replace-match "" nil t))
 	  (goto-char (point-min))
 	  (while (re-search-forward nl nil t)
-	    (replace-match "\n" nil t)))))
+	    (replace-match "\n" nil t))
+	  (remove-text-properties (point-min) (point-max) '(mew-addr-spec t)))))
       ;;
       (if (and mew-decode-broken (member key mew-decode-ws-fields))
 	  (let (ws)
@@ -932,7 +946,7 @@ That is, each line may be more than 75. "
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
