@@ -67,6 +67,7 @@
 (defconst mew-face:        "Face:")
 (defconst mew-x-mew:       "X-Mew:")
 (defconst mew-x-mew-uidl:  "X-Mew-Uidl:")
+(defconst mew-x-mew-ref:   "X-Mew-Ref:")
 
 (defconst mew-keywords: "Keywords:")
 (defconst mew-body:	"Body:")
@@ -199,22 +200,45 @@
 (defconst mew-regex-message-files2 "^[1-9][0-9]*")
 (defconst mew-regex-message-files3 "^[0-9][0-9]*$")
 (defconst mew-regex-message-files4 "^0[1-9][0-9]*$")
-(defconst mew-regex-mark "\\([^ 0-9]\\)")
 
-(defconst mew-regex-msg         "^ *\\([0-9]+\\)")
-(defconst mew-regex-msg-valid   "^ *\\([1-9][0-9]*\\)")
-(defconst mew-regex-msg-invalid "^ *\\(0[1-9][0-9]*\\)")
+;;;
+;;; Sumsyn
+;;;
+
+(defconst mew-regex-mark "^\\([^ \n]\\)")
+(defsubst mew-sumsyn-mark () (string-to-char (mew-match-string 1)))
+
+;; fld msg 
+(defconst mew-regex-sumsyn-short   "\r \\([^ \n]+\\) \\([0-9]+\\)")
+
+;; fld msg my-id par-id uid siz
+(defconst mew-regex-sumsyn-long    "\r \\([^ \n]+\\) \\([0-9]+\\) <\\([^>\n]*\\)> <\\([^>\n]*\\)> \\([^ \n]*\\) \\([0-9]*\\)")
+
+;; fld msg 
+(defsubst mew-regex-sumsyn-msg (msg)
+                           (format "\r \\([^ \n]\\)+ \\(%s\\) " msg))
+
+(defconst mew-regex-sumsyn-valid   "\r \\([^ \n]+\\) \\([1-9][0-9]*\\)")
+(defconst mew-regex-sumsyn-invalid "\r [^ \n]+ 0[1-9][0-9]*")
+
+(defsubst mew-msg-validp (msg)
+  (string-match mew-regex-message-files msg))
+
+;; fld msg my-id par-id uid siz
+(defsubst mew-regex-sumsyn-my-id  (my-id)
+  (format "\r [^ \n]+ [0-9]+ <%s> <[^>\n]*>" (regexp-quote my-id)))
+(defsubst mew-regex-sumsyn-par-id (par-id)
+  (format "\r [^ \n]+ [0-9]+ <[^>\n]*> <%s>" (regexp-quote par-id)))
+
+;;;
+;;;
+;;;
 
 (defconst mew-regex-msg-icon  "^\\([0-9]+\\) ")
-(defconst mew-regex-part      "^.....\\([.0-9]+\\) +")
+(defconst mew-regex-part      "^...  \\([.0-9]+\\) ") ;; yyy
 (defconst mew-regex-part-icon "<\\([.0-9]+\\)>")
 
-(defconst mew-regex-summary
-  "[^\r\n]+\r <\\([^>]*\\)> <\\([^>]*\\)>")
-(defconst mew-regex-summary2
-  "[^\r\n]+\r <\\([^>]*\\)> <\\([^>]*\\)> \\([^ ]+\\) \\([0-9]*\\)")
-(defconst mew-regex-virtual
-  "[^\006\n]+\006 \\([^ ]*\\) \\([0-9]+\\)$")
+(defsubst mew-regex-jmp-part (part) (format "^...  \\(%s\\) " part)) ;; yyy
 
 ;; [\033\200-\377] does not work due to the multibyte
 (defconst mew-regex-esc-or-nonascii "[^\000-\032\034-\177]") ;; \033 = esc
@@ -233,9 +257,6 @@
 ;; 	<19981005133317N.kazu@iijlab.net>
 (defconst mew-regex-id "<\\([^>\n\t]+\\)>")
 
-(defsubst mew-regex-jmp-msg (msg)
-  (concat  "^[ ]*" msg "[^0-9]"))
-
 (defconst mew-regex-case "^\\([^-+$%*,:][^,:]+\\):\\(.*\\)")
 (defconst mew-regex-case2 "\\([^-+$%*,:][^,:]+\\):")
 
