@@ -90,8 +90,7 @@
        (goto-char (point-max))
        (insert "\n"))
      (when (and (integerp mew-header-max-length)
-		(> (mew-count-lines (point-min) (point))
-		   mew-header-max-length))
+		(> (count-lines (point-min) (point)) mew-header-max-length))
        (mew-xinfo-set-action
 	(substitute-command-keys
 	 "Too long header. To see the message, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again]'"))
@@ -325,9 +324,7 @@ Return a part syntax after moving the beginning of the content body."
 	     (setq value (mew-buffer-substring med (1- (point)))))
 	    ((eq act 'id)
 	     (setq value (mew-buffer-substring med (1- (point))))
-	     (if (string-match mew-regex-id value)
-		 (setq value (match-string 1 value))
-	       (setq value nil))))
+	     (setq value (mew-idstr-get-first-id value 'no-angle))))
 	   (aset vec n value)))))
     (if (eobp)
 	(insert "\n")
@@ -435,7 +432,7 @@ Return a part syntax after moving the beginning of the content body."
 	    (forward-line)
 	  (mew-decode-error "No end-of-header(null line) in RFC822 message"))
 	;; the beginning of the BODY(i.e. the physical content body)
-	(if (eq (mew-dinfo-get-decode-text) t)
+	(if (eq (mew-dinfo-get-decode-text) t) ;; not 'no-cs-conv
 	    (mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
 	(mew-syntax-set-key syntax 'message)
 	(mew-syntax-set-end syntax (1- (point)))
@@ -924,7 +921,7 @@ Return a part syntax after moving the beginning of the content body."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
