@@ -43,7 +43,12 @@
   (make-local-variable 'comment-start)
   (setq comment-start ">")
   (make-local-hook 'after-change-functions)
-  (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local))
+  (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local)
+  (add-hook 'local-write-file-hooks 'mew-encode-make-backup)
+  (make-local-variable 'after-save-hook)
+  (when mew-require-final-newline
+    (make-local-variable 'require-final-newline)
+    (setq require-final-newline t)))
 
 (defun mew-draft-mode (&optional encrypted)
   "\\<mew-draft-mode-map>
@@ -171,24 +176,20 @@ Moreover, '~/.mew-fib' supports aliases like;
 
 "
   (interactive)
-  (mew-draft-set-local-variables)
-  (add-hook 'local-write-file-hooks 'mew-encode-make-backup)
   (setq major-mode 'mew-draft-mode)
+  (setq mode-line-buffer-identification mew-mode-line-id)
+  (mew-draft-mode-name)
+  (mew-draft-set-local-variables)
   (use-local-map mew-draft-mode-map)
   (set-syntax-table mew-draft-mode-syntax-table)
-  (mew-ainfo-set-icon (file-name-nondirectory (buffer-file-name)))
   (cd (expand-file-name mew-home))
-  (when mew-require-final-newline
-    (make-local-variable 'require-final-newline)
-    (setq require-final-newline t))
   (mew-draft-setup-decoration)
+  (mew-ainfo-set-icon (file-name-nondirectory (buffer-file-name)))
   (mew-tinfo-set-encrypted-p encrypted)
   (mew-tinfo-set-privacy-err nil)
   (mew-tinfo-set-privacy-type nil)
-  (setq mode-line-buffer-identification mew-mode-line-id)
-  (mew-draft-mode-name)
   (run-hooks 'text-mode-hook 'mew-draft-mode-hook)
-  ;; must be here for auto-fill
+  ;; auto-fill-function is set by mew-draft-mode-hook
   (when auto-fill-function
     (make-local-variable 'auto-fill-function)
     (setq auto-fill-function 'mew-draft-auto-fill))
@@ -269,8 +270,7 @@ Moreover, '~/.mew-fib' supports aliases like;
     (when mew-use-other-frame-for-draft
       (mew-tinfo-set-other-frame t)
       ;; to ensure to cite a message from summary frame.
-      (if (get-buffer (mew-buffer-message))
-	  (kill-buffer (mew-buffer-message))))
+      (mew-remove-buffer (mew-buffer-message)))
     (rename-buffer draftname)))
 
 (defun mew-draft-to-attach (draft)
@@ -300,8 +300,7 @@ the Body: field."
   (let ((cnt 0) (beg (point)) med adr)
     (while adrs
       (setq adr (car adrs) adrs (cdr adrs))
-      (if (mew-is-my-address del adr)
-	  ()
+      (unless (mew-is-my-address del adr)
 	(if (= cnt 0)
 	    (insert adr)
 	  (insert ", " adr))
@@ -339,53 +338,53 @@ the Body: field."
 ;; cc -- string or list
 ;; nl -- one empty line under "----", which is necessary if
 ;;      attachment is prepared
-  (let ((del mew-regex-my-address-list) ;; deleting list for Cc:
+  (let ((del (unless fromme mew-regex-my-address-list)) ;; deleting list
 	case body)
     (goto-char (point-min))
-    ;; Insert To: first.
-    ;; All addresses inserted on To: are appended to del.
-    (cond
-     ((null to) (insert mew-to: " \n"))
-     ((stringp to) ;; To: inputed from the mini-buffer.
-      ;; do not check to is mine. Cc: is also string
-      ;; We believe that user never specifies the same address of To: to Cc:.
-      (insert mew-to: " " to "\n"))
-     ;; To: collected by reply
-     ((listp to)
-      (if fromme
-	  (mew-insert-address-list2 mew-to: to)
-	(setq del (mew-insert-address-list mew-to: to del t)))))
-    (cond
-     ((null cc) ()) ;; do nothing 
-     ((stringp cc) ;; Cc: inputed from the mini-buffer.
-      (insert mew-cc: " " cc "\n"))
-     ((listp cc) ;; Cc: collected by reply.
-      (if fromme
-	  (mew-insert-address-list2 mew-cc: cc)
+    (if newsgroups
+	(cond
+	 ((stringp newsgroups)
+	  (mew-draft-header-insert mew-newsgroups: newsgroups))
+	 ((listp newsgroups)
+	  (mew-insert-address-list2 mew-newsgroups: newsgroups)))
+      ;; Insert To: first.
+      ;; All addresses inserted on To: are appended to del.
+      (cond
+       ((null to) (mew-draft-header-insert mew-to: ""))
+       ((stringp to) ;; To: inputed from the mini-buffer.
+	;; do not check to is mine. Cc: is also string
+	;; We believe that user never specifies the same address of To: to Cc:.
+	(mew-draft-header-insert mew-to: to))
+       ;; To: collected by reply
+       ((listp to)
+	(setq del (mew-insert-address-list mew-to: to del t))))
+      (cond
+       ((null cc) ()) ;; do nothing 
+       ((stringp cc) ;; Cc: inputed from the mini-buffer.
+	(mew-draft-header-insert mew-cc: cc))
+       ((listp cc) ;; Cc: collected by reply.
 	(mew-insert-address-list mew-cc: cc del nil))))
-    (mew-draft-header-insert mew-newsgroups:  newsgroups)
     (if mew-case-guess-when-prepared
 	(mew-draft-set-case-by-guess))
     (setq case (mew-tinfo-get-case))
-    (mew-draft-header-insert mew-cc:          (mew-cc case))
-    (mew-draft-header-insert mew-subj:        (or subject ""))
-    (mew-draft-header-insert mew-from:        (mew-from case))
-    (mew-draft-header-insert mew-fcc:         (mew-fcc case))
-    (mew-draft-header-insert mew-dcc:         (mew-dcc case))
-    (mew-draft-header-insert mew-reply-to:    (mew-reply-to case))
-    (mew-draft-header-insert mew-in-reply-to: in-reply-to)
-    (mew-draft-header-insert mew-references:  references)
+    (unless newsgroups
+      (mew-draft-header-insert mew-cc: (mew-cc case)))
+    (mew-draft-header-insert mew-subj: (or subject ""))
+    (mew-draft-header-insert mew-from: (mew-from case))
+    (mew-draft-header-insert mew-fcc: (mew-fcc case))
+    (unless newsgroups
+      (mew-draft-header-insert mew-dcc: (mew-dcc case)))
+    (mew-draft-header-insert mew-reply-to: (mew-reply-to case))
+    (unless newsgroups
+      (mew-draft-header-insert mew-in-reply-to: in-reply-to))
+    (mew-draft-header-insert mew-references: references)
     (if (and mew-x-face-file
 	     (file-exists-p (expand-file-name mew-x-face-file)))
-	(let ((buf (generate-new-buffer mew-buffer-prefix))
-	      xface)
-	  (save-excursion
-	    (set-buffer buf)
-	    (mew-erase-buffer)
+	(let (xface)
+	  (with-temp-buffer
 	    (insert-file-contents (expand-file-name mew-x-face-file))
 	    (setq xface (mew-buffer-substring (point-min)
 					      (max (buffer-size) 1))))
-	  (mew-remove-buffer buf)
 	  (mew-draft-header-insert mew-x-face: xface)))
     (mew-draft-header-insert mew-organization: (mew-organization case))
     (setq body (mew-draft-header-insert-alist other-headers))
@@ -395,13 +394,14 @@ the Body: field."
     (mew-draft-header-insert-alist (mew-header-alist case))
     ;; X-Mailer: must be the last
     (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
+    ;; (mew-header-set "\n") is enough. But highlighting delayed.
     (mew-header-prepared)
     ;; on the body
     (if nl (insert "\n"))
     (if body (save-excursion (insert body)))
     ;; move the cursor after "To: "
     (goto-char (point-min))
-    (forward-char 4))) ;; do not use (end-of-line) since the value may exist.
+    (search-forward ": " nil t)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -448,7 +448,7 @@ citation prefix and label.
 	      (setq beg (point-min) end (point-max)))
 	     ((mew-mark) 
 	      (setq beg (region-beginning) end (region-end)))
-	     ((mew-header-p)
+	     ((mew-msghder-p)
 	      ;; header exists in Message mode
 	      (mew-header-goto-body)
 	      (setq beg (point) end (point-max)))
@@ -504,7 +504,7 @@ citation prefix and label.
 	     ((and (mew-mark) (or (not mew-cite-ignore-mouse-region)
 				  (not (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
-	     ((mew-header-p)
+	     ((mew-msghder-p)
 	      ;; header exists in Message mode. Skip the header
 	      ;; because we will concatenate it to cite later.
 	      (mew-header-goto-body)
@@ -517,7 +517,7 @@ citation prefix and label.
 	(setq tbuf (or nonmewbuf
 		       (save-excursion
 			 (when (get-buffer msg-buf)
-			   (if (mew-header-p) (current-buffer))))
+			   (if (mew-msghder-p) (current-buffer))))
 		       ;; header exists only in cache if multipart
 		       (mew-cache-hit fld msg)))
 	(if (get-buffer tbuf)
@@ -528,28 +528,25 @@ citation prefix and label.
 	  (mew-header-goto-end)
 	  (setq cite (concat (mew-buffer-substring (point-min) (point)) 
 			     "\n" cite))
-          (setq irt-msgid (mew-header-get-value mew-message-id:))))
+          (setq irt-msgid (mew-idstr-get-first-id
+			   (mew-header-get-value mew-message-id:)))))
       ;; 
       ;; Draft mode, insert the header and the body.
       ;;
 
       ;; Append message-id to In-Reply-To:
-      (if (and irt-msgid (mew-header-p))
+      (if (and irt-msgid (mew-msghder-p))
           (save-excursion
-            (let ((irt (mew-header-get-value mew-in-reply-to:))
-                  (irtl nil) rb)
-              (when irt
-                (setq rb irt)
-                (while (string-match "<[^>]+>" rb)
-                  (setq irtl (cons (match-string 0 rb) irtl)) 
-                  (setq rb (substring rb (match-end 0))))
-                (if (member irt-msgid irtl)
-                    (setq irt-msgid nil)))
-              (when irt-msgid
-                (setq irt (concat irt (if irt "\n\t") irt-msgid))
-                (mew-header-delete-lines (list mew-in-reply-to:))
-                (unless irtl (goto-char (mew-header-end)))
-                (mew-draft-header-insert mew-in-reply-to: irt)))))
+            (let* ((mew-references-max-count nil)
+		   (irt (mew-header-get-value mew-in-reply-to:))
+		   (irtl (mew-idstr-to-id-list irt nil 'rev))
+		   irtstr)
+	      (mew-addq irtl irt-msgid)
+	      (setq irtl (nreverse irtl))
+	      (setq irtstr (mew-id-list-to-idstr irtl))
+	      (mew-header-delete-lines (list mew-in-reply-to:))
+	      (unless irt (goto-char (mew-header-end)))
+	      (mew-draft-header-insert mew-in-reply-to: irtstr))))
       (save-restriction
 	;; this gets complicated due to supercite, please do not care
 	(narrow-to-region (point)(point)) ;; for (goto-char (point-min))
@@ -612,14 +609,11 @@ citation prefix and label.
 	   (label (apply 'format mew-cite-format vals))
 	   (ellipses (if (stringp mew-draft-cite-ellipses)
 			 mew-draft-cite-ellipses ""))
-	   buf beg eol str)
+	   beg eol)
       (if (not (or (eq mew-draft-cite-fill-mode 'truncate)
 		   (eq mew-draft-cite-fill-mode 'wrap)))
 	  label
-	(setq buf (generate-new-buffer mew-buffer-prefix))
-	(save-excursion
-	  (set-buffer buf)
-	  (mew-erase-buffer)
+	(with-temp-buffer
 	  (let ((fill-column
 		 (or mew-draft-cite-label-fill-column fill-column)))
 	    (insert label)
@@ -644,9 +638,7 @@ citation prefix and label.
 		  (if (= (current-column) 0) ;; for XEmacs
 		      (delete-backward-char 1)))))
 	      (forward-line)))
-	  (setq str (buffer-string)))
-	(mew-remove-buffer buf)
-	str))))
+	  (buffer-string))))))
 
 (defun mew-cite-prefix-username ()
   "A good candidate for mew-cite-prefix-function.
@@ -666,6 +658,16 @@ The citation style is 'from_address> ', e.g. 'kazu> '"
 ;;; Misc
 ;;;
 
+(defun mew-draft-save-buffer ()
+  (interactive)
+  (save-excursion
+    (mew-header-clear)
+    (insert-before-markers "\n") ;; for mew-summary-reply
+    (save-buffer)
+    (delete-region (1- (point)) (point))
+    (mew-header-prepared)
+    (set-buffer-modified-p nil)))
+
 (defun mew-draft-kill ()
   "Kill this draft."
   (interactive)
@@ -796,6 +798,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
   (mew-draft-process-message 'send))
 
 (defun mew-draft-process-message (action &optional privacy signer)
+  (run-hooks 'mew-make-message-hook)
   (let* ((case (or (mew-tinfo-get-case) mew-case-default))
 	 (old-case case)
 	 guessed-case)
@@ -859,12 +862,12 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 		    (mew-smtp-queue case "from Draft mode"))
 		   ((eq action 'send)
 		    (mew-smtp-send-message case))))
-	      (setq err t)))))
-      (if (not err)
-	  (run-hooks 'mew-real-send-hook)
-	(mew-current-set-window-config)
-	(switch-to-buffer buf)
-	(delete-other-windows)))))
+	      (setq err t))))
+	(if (not err)
+	    (run-hooks 'mew-real-send-hook)
+	  (mew-current-set-window-config)
+	  (switch-to-buffer buf)
+	  (delete-other-windows))))))
 
 (defun mew-draft-nntp-process-message (case action &optional privacy signer)
   (run-hooks 'mew-post-hook)
@@ -899,12 +902,12 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 		    (mew-nntp2-queue case "from Draft mode"))
 		   ((eq action 'send)
 		    (mew-nntp2-send-message case))))
-	      (setq err t)))))
-      (if (not err)
-	  (run-hooks 'mew-real-post-hook)
-	(mew-current-set-window-config)
-	(switch-to-buffer buf)
-	(delete-other-windows)))))
+	      (setq err t))))
+	(if (not err)
+	    (run-hooks 'mew-real-post-hook)
+	  (mew-current-set-window-config)
+	  (switch-to-buffer buf)
+	  (delete-other-windows))))))
 
 (defun mew-draft-remove-illegal-fields ()
   (when (mew-header-end)
@@ -942,7 +945,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
