@@ -40,7 +40,11 @@ Execute mew-dinfo-set before calling this."
   (mew-header-setup-decoration)
   (mew-draft-mode-name 'header)
   (force-mode-line-update)
-  (run-hooks 'mew-header-mode-hook))
+  (run-hooks 'mew-header-mode-hook)
+  ;; auto-fill-function is set by mew-header-mode-hook
+  (when auto-fill-function
+    (make-local-variable 'auto-fill-function)
+    (setq auto-fill-function 'mew-draft-auto-fill)))
 
 (defmacro mew-summary-header-mode (fields &rest body)
   `(mew-summary-msg-or-part
@@ -54,12 +58,13 @@ Execute mew-dinfo-set before calling this."
        (mew-draft-find-and-switch draft)
        (mew-delete-directory-recursively attachdir)
        ,@body
-       (mew-header-prepared 'header)
+       (mew-header-mode)
+       ;; (mew-header-set "\n") is enough. But highlighting delayed.
+       (mew-header-prepared)
        (goto-char (point-min))
        (end-of-line)
        (mew-tinfo-set-hdr-file (mew-expand-folder fld msg))
-       (mew-tinfo-set-field-del ,fields)
-       (mew-header-mode)))))
+       (mew-tinfo-set-field-del ,fields)))))
 
 (defun mew-summary-send-to-others ()
   (interactive)
@@ -262,6 +267,7 @@ a text object to be saved can be specified."
        (setq ctl (mew-syntax-get-ct syntax))
        ;;
        (if (and mew-use-samba-encoding
+		file
 		(string-match mew-regex-nonascii file))
 	   (setq file (mew-samba-encoding file)))
        (setq file (mew-summary-input-file-name nil file))
@@ -300,10 +306,10 @@ a text object to be saved can be specified."
 	       ;; When saving an entire message or its body, 
 	       ;; its charset is unknown since it is not analyzed.
 	       ;; So, we make use of mew-cs-autoconv.
-	       (if (not (mew-coding-system-p fromcs))
-		   (error "Unknown coding system %s" (symbol-name fromcs)))
-	       (if (not (mew-coding-system-p tocs))
-		   (error "Unknown coding system %s" (symbol-name tocs)))
+	       (unless (mew-coding-system-p fromcs)
+		 (error "Unknown coding system %s" (symbol-name fromcs)))
+	       (unless (mew-coding-system-p tocs)
+		 (error "Unknown coding system %s" (symbol-name tocs)))
 	       (mew-cs-decode-region
 		(point-min) (point-max) (or fromcs mew-cs-autoconv))
 	       (mew-cs-encode-region (point-min) (point-max) tocs))
@@ -437,7 +443,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	    (message "Nothing was processed")
 	  (mew-touch-folder folder)
 	  (setq ret (list mstr (int-to-string (1- m)))))))
-      (if errmsg (error errmsg))
+      (if errmsg (error "%s" errmsg))
       ret)))
 
 (defun mew-summary-burst-part (part folder newmsg &optional num)
@@ -628,7 +634,7 @@ See also mew-summary-edit-again."
   ;; X-Mailer: must be the last
   (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
   (mew-header-clear) ;; erase the old header separator
-  (mew-header-prepared))
+  (mew-header-set "\n"))
 
 (defun mew-summary-edit-header-for-draft ()
   (mew-elet
