@@ -316,7 +316,7 @@
    (addsep ;; reedit
     ;; To:
     ;; Content-*
-    ;; ---
+    ;; ----
     (mew-header-clear) ;; mew-header-p returns nil
     ;; To:
     ;; Content-*
@@ -354,7 +354,7 @@
 	  (mew-smtp-encode-message pnm case resentp fcc privacy signer headerp)
 	(error
 	 (mew-encode-undo)
-	 (message (mew-tinfo-get-encode-err))
+	 (message "%s" (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
@@ -362,7 +362,6 @@
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (run-hooks 'mew-make-message-hook)
   (let (multip recipients msgid-logtime)
     (goto-char (point-min))
     (mew-encode-remove-illegal-fields)
@@ -426,7 +425,7 @@
 	  (mew-nntp2-encode-message pnm case fcc privacy signer headerp)
 	(error
 	 (mew-encode-undo)
-	 (message (mew-tinfo-get-encode-err))
+	 (message "%s" (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
@@ -434,7 +433,6 @@
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (run-hooks 'mew-make-message-hook)
   (let (multip newsgroups msgid-logtime)
     (goto-char (point-min))
     (mew-encode-remove-illegal-fields)
@@ -688,7 +686,7 @@
 
 (defun mew-encode-make-multi ()
   ;; Just after the header
-  (let (beg path multip buffered privacy first ct)
+  (let (beg path multip buffered privacy decrypters first ct)
     (if (mew-encode-syntax-have-one-part)
 	;; A user may want to do any MIME stuff to the body
 	(progn
@@ -722,8 +720,10 @@
 	    (setq path (expand-file-name
 			(mew-syntax-get-file mew-encode-syntax) path))
 	    (setq privacy (mew-syntax-get-privacy mew-encode-syntax))
+	    (setq decrypters (mew-syntax-get-decrypters mew-encode-syntax))
 	    (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax))
-	    (mew-syntax-set-privacy mew-encode-syntax privacy)))))
+	    (mew-syntax-set-privacy mew-encode-syntax privacy)
+	    (mew-syntax-set-decrypters mew-encode-syntax decrypters)))))
     (if multip
 	(mew-encode-multipart mew-encode-syntax path 0 buffered)
       (mew-encode-singlepart mew-encode-syntax path nil nil 'cover))))
@@ -1050,7 +1050,8 @@
 		  (mew-case-equal cte mew-8bit)
 		  (mew-case-equal cte mew-bin))
 	      (setq cte mew-b64))
-	  (mew-convert-mime-body ctbody-beg (point-max) cte 'linebasep))
+	  (unless (mew-case-equal cte mew-7bit)
+	    (mew-convert-mime-body ctbody-beg (point-max) cte 'linebasep)))
 	 (t
 	  ;; rare case
 	  (setq cte (mew-ctdb-cte (mew-ctdb-by-ct ct)))
@@ -1058,8 +1059,9 @@
 		  (mew-case-equal cte mew-8bit)
 		  (mew-case-equal cte mew-bin))
 	      (setq cte mew-b64))
-	  (mew-convert-mime-body
-	   ctbody-beg (point-max) cte (mew-ct-linebasep ct)))
+	  (unless (mew-case-equal cte mew-7bit)
+	    (mew-convert-mime-body
+	     ctbody-beg (point-max) cte (mew-ct-linebasep ct))))
 	 (goto-char (point-min)))
 	(mew-header-delete-lines (list mew-cte:))
 	(goto-char cthd-end)
@@ -1152,7 +1154,7 @@
      (if (not (mew-encode-insert-backup))
 	 (message "Cannot undo")
        (mew-header-clear) ;; erase the old header separator
-       (mew-header-prepared (mew-tinfo-get-hdr-file))
+       (mew-header-prepared)
        (if mew-encode-syntax (mew-draft-prepare-attachments))
        (mew-draft-toolbar-update)
        (setq buffer-undo-list nil)))))
