@@ -17,12 +17,13 @@ PIPE is usually recommended for speed but some OSes such as Linux
 requires PTY.")
 
 (defvar mew-connection-type2 t
-  "Connection type for processes that requires a password. ")
+  "Connection type for processes that requires a password.")
 
 (defvar mew-connection-type-for-scan t
-  "Connection type for 'mewls' whose output speed is very fast. 
+  "Connection type for 'mewl' whose output speed is very fast. 
 To slow down, PTY should be used.")
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Must be here for key setup
 ;;;
@@ -34,6 +35,7 @@ To slow down, PTY should be used.")
   (defalias 'mew-set-keymap-parent 'set-keymap-parent)))
 
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Emacsen
 ;;;
@@ -44,6 +46,7 @@ To slow down, PTY should be used.")
 (defvar mew-mule-p   nil)
 (defvar mew-icon-p   nil)
 (defvar mew-mule-ver 0)
+(defvar mew-internal-utf-8p nil)
 
 (if (featurep 'mule) (setq mew-mule-p t))
 
@@ -74,10 +77,13 @@ To slow down, PTY should be used.")
     (require 'image) ;; to prepend the loading messages
     (require 'jit-lock) ;; to prepend the loading messages
     (require 'regexp-opt) ;; to prepend the loading messages
-    (require 'mew-gemacs))
+    (require 'mew-gemacs)
+    (if (fboundp 'utf-translate-cjk-mode) ;; Emacs 21.3.50 or later
+	(setq mew-internal-utf-8p t)))
   (setq mew-mule-ver 3)
   (require 'mew-mule3)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Emacs vs XEmacs
 ;;;
@@ -108,6 +114,11 @@ To slow down, PTY should be used.")
     (defun mew-md5 (str)
       (md5 str nil nil 'binary)))))
 
+(if mew-xemacs-p
+    (defalias 'mew-window-edges 'window-pixel-edges)
+  (defalias 'mew-window-edges 'window-edges))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Misc
 ;;;
@@ -122,15 +133,21 @@ To slow down, PTY should be used.")
     (defalias 'mew-mouse-region-p 'mouse-region-match)
   (defsubst mew-mouse-region-p () nil))
 
-(if (boundp 'automatic-hscrolling)
-    (defsubst mew-hscroll ()
-      (set (make-local-variable 'automatic-hscrolling) t))
-  (defsubst mew-hscroll () (auto-show-mode 1)))
+(cond
+ ((boundp 'auto-hscroll-mode) ;; Emacs 21.3 or later
+  (defsubst mew-hscroll ()
+    (set (make-local-variable 'auto-hscroll-mode) t)))
+ ((boundp 'automatic-hscrolling) ;; Emacs 21.2 or earlier
+  (defsubst mew-hscroll ()
+    (set (make-local-variable 'automatic-hscrolling) t)))
+ (t
+  (defsubst mew-hscroll () (auto-show-mode 1))))
 
 (if (fboundp 'minibuffer-prompt-end)
     (defalias 'mew-minibuf-point-min 'minibuffer-prompt-end)
   (defalias 'mew-minibuf-point-min 'point-min))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Sticky
 ;;;
@@ -170,6 +187,7 @@ To slow down, PTY should be used.")
 (defsubst mew-rear-nonsticky (beg end)
   (put-text-property beg end mew-rn mew-rnv))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Overlay
 ;;;
@@ -192,6 +210,7 @@ To slow down, PTY should be used.")
     (widen)
     (mew-overlay-delete-region (point-min) (point-max))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; File operations
 ;;;
@@ -204,6 +223,7 @@ To slow down, PTY should be used.")
 on all frames."
     (custom-set-faces `(,face ,spec))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; File operations
 ;;;
@@ -231,6 +251,7 @@ on all frames."
 	(error "Cannot make a copy of directory")
       (copy-file filename newname OK-IF-ALREADY-EXISTS 'keepdate)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Property
 ;;;
@@ -244,6 +265,7 @@ on all frames."
 (defsubst mew-insert-buffer-substring (buf beg end)
   (insert (save-excursion (set-buffer buf) (mew-buffer-substring beg end))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Base64
 ;;;
@@ -337,11 +359,25 @@ on all frames."
 	    (aset ret (1+ (1+ j)) (logand c 255)))
 	ret))))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Language
+;;;
+
+;; to load the thread separator
+(cond
+ ((and (boundp 'current-language-environment)
+       (string= current-language-environment "Japanese"))
+  (require 'mew-lang-jp))
+ ((and (boundp 'current-language-environment)
+       (string= current-language-environment "Korean"))
+  (require 'mew-lang-kr)))
+
 (provide 'mew-env)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
