@@ -13,7 +13,9 @@
 ;;;
 
 (defun mew-mark-kill-refile (src msg)
-  (not (member src (mew-refile-get msg))))
+  (let ((myself (member src (mew-refile-get msg))))
+    (if myself (mew-summary-refile-remove-body))
+    (not myself))) ;; return value
 
 (defun mew-mark-exec-refile (src msgs)
   "Refile MSGs from the SRC folder."
@@ -60,7 +62,8 @@ DELS represents the messages to be deleted."
 	    (progn
 	      (goto-char (point-min))
 	      (while (re-search-forward mew-regex-msg-delete nil t)
-		(mew-summary-undo-one)))
+		(mew-summary-undo-one)
+		(forward-line)))
 	  (mew-summary-totrash-msgs src dels))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -81,7 +84,7 @@ DELS represents the messages to be deleted."
 	(mew-addq uniq-dsts dst)))
     (mew-addq uniq-dsts mew-trash-folder)
     (while uniq-dsts
-      (setq udst (mew-folder-case-folder mew-inherit-case (car uniq-dsts)))
+      (setq udst (mew-folder-case-folder mew-inherit-exec-case (car uniq-dsts)))
       (setq uniq-dsts (cdr uniq-dsts))
       (setq dir (mew-expand-folder udst))
       (if (file-exists-p dir)
@@ -97,13 +100,23 @@ DELS represents the messages to be deleted."
 ;;; Unlinking, moving, to-trash
 ;;;
 
+(defun mew-folder-suffix (folder)
+  (if (or (mew-folder-queuep folder) (mew-folder-postqp folder))
+      mew-queue-info-suffix
+    (if (mew-folder-draftp folder)
+	mew-draft-info-suffix)))
+
 (defun mew-summary-unlink-msgs (src dels)
-  (let (file)
+  (let ((suffix (mew-folder-suffix src))
+	file infofile)
     (while dels
       (setq file (mew-expand-folder src (car dels)))
       (setq dels (cdr dels))
       ;; if file does not exist, the marked line will be deleted anyway.
-      (mew-delete-file file))))
+      (mew-delete-file file)
+      (when suffix
+	(setq infofile (concat file suffix))
+	(mew-delete-file infofile)))))
 
 (defun mew-i2s (num cache)
   (if cache
@@ -116,7 +129,8 @@ DELS represents the messages to be deleted."
 	 (srcx (mew-folder-folder src))
 	 (myselfp (member srcx dsts))
 	 (cache mew-inherit-offline)
-	 msgs- msg srcfile dstfile dst num strnum)
+	 (suffix (mew-folder-suffix srcx))
+	 msgs- msg srcfile dstfile dst num strnum infofile)
     (if myselfp
 	;; msg stays in the src folder with the same number
 	(progn
@@ -128,7 +142,7 @@ DELS represents the messages to be deleted."
 		(setq msgs- (cons msg msgs-))))
 	  (setq msgs- (nreverse msgs-))
 	  (setq msgs msgs-))
-      (setq dst (mew-folder-case-folder mew-inherit-case (car dsts)))
+      (setq dst (mew-folder-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
       (setq num (string-to-int (mew-folder-new-message dst 'num-only cache)))
       (while msgs
@@ -139,12 +153,15 @@ DELS represents the messages to be deleted."
 	  (setq msgs- (cons strnum msgs-))
 	  (setq dstfile (mew-expand-folder dst strnum))
 	  (setq num (1+ num))
-	  (rename-file srcfile dstfile 'override)))
+	  (rename-file srcfile dstfile 'override)
+	  (when suffix
+	    (setq infofile (concat srcfile suffix))
+	    (mew-delete-file infofile))))
       (mew-touch-folder dst)
       (setq msgs- (nreverse msgs-))
       (setq src dst)) ;; myselfp
     (while dsts
-      (setq dst (mew-folder-case-folder mew-inherit-case (car dsts)))
+      (setq dst (mew-folder-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
       (setq num (string-to-int (mew-folder-new-message dst 'num-only cache)))
       (setq msgs msgs-)
@@ -158,14 +175,16 @@ DELS represents the messages to be deleted."
       (mew-touch-folder dst))))
 
 (defun mew-summary-totrash-msgs (src dels)
-  (let* ((trash (mew-folder-case-folder mew-inherit-case mew-trash-folder))
+  (let* ((trash (mew-folder-case-folder mew-inherit-exec-case mew-trash-folder))
 	 (trashdir (mew-expand-folder trash))
-	 (queue-p (or (mew-folder-queuep src) (mew-folder-postqp src)))
 	 (cache mew-inherit-offline)
-	 num srcfile dstfile mqifile)
+	 (suffix (mew-folder-suffix src))
+	 num srcfile dstfile infofile)
     (unless (file-directory-p trashdir) (make-directory trashdir))
     (setq num (string-to-int (mew-folder-new-message trash 'num-only cache)))
     ;; must be here after ensuring that +trash exists.
+
+
     (while dels
       (setq srcfile (mew-expand-folder src (car dels))) ;; xxx
       (setq dels (cdr dels))
@@ -174,9 +193,9 @@ DELS represents the messages to be deleted."
       (if (file-regular-p srcfile)
 	  ;; if not, the marked line will be deleted anyway.
 	  (rename-file srcfile dstfile 'override))
-      (when queue-p
-	(setq mqifile (concat srcfile mew-queue-info-suffix))
-	(mew-delete-file mqifile)))
+      (when suffix
+	(setq infofile (concat srcfile suffix))
+	(mew-delete-file infofile)))
     (mew-touch-folder trash)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -202,7 +221,7 @@ region. To cancel the '*' mark, use '\\[mew-summary-undo]' or
     (mew-summary-exec-region beg end)))
 
 (defun mew-summary-exec-one (&optional arg)
-  "Process the current marked messages.
+  "Process the current marked message.
 When called with '\\[universal-argument]', process 
 all marked messages before the current message."
   (interactive "P")
@@ -233,19 +252,37 @@ all marked messages before the current message."
 	 (mew-mark-spec (list ent)))
     (mew-summary-exec-region (point-min) (point-max))))
 
-(defun mew-summary-exec-refile ()
-  "Process messages marked with 'o'."
-  (interactive)
+(defun mew-summary-exec-refile (&optional arg)
+  "Process messages marked with 'o'.
+If called with '\\[universal-argument]', only messages whose
+destination is the same as that of the current message are processed."
+  (interactive "P")
   (let* ((ent (assoc mew-mark-refile mew-mark-spec))
 	 (mew-mark-spec (list ent)))
-    (mew-summary-exec-region (point-min) (point-max))))
+    (if (not arg)
+	(mew-summary-exec-region (point-min) (point-max))
+      (let* ((msg (mew-summary-message-number))
+	     (dst (cdr (mew-refile-get msg)))
+	     func)
+	(if (not dst)
+	    (message "No message to be refiled here")
+	  (setq func (lambda (num) (equal dst (cdr (mew-refile-get num)))))
+	  (mew-summary-exec-region (point-min) (point-max) func))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Exec region
 ;;;
 
-(defun mew-summary-exec-region (beg end)
+(defun mew-summary-move-and-display (msg)
+  (let ((here (point)))
+    (goto-char (point-min))
+    (if (not (re-search-forward (mew-regex-sumsyn-msg msg) nil t))
+	(goto-char here)
+      (beginning-of-line)
+      (mew-summary-display nil))))
+
+(defun mew-summary-exec-region (beg end &optional func)
   (let ((folder (mew-sinfo-get-folder)))
     (cond
      ((mew-folder-nntpp folder)
@@ -264,7 +301,7 @@ all marked messages before the current message."
 	      (message "No physical folder")
 	    (mew-summary-visit-folder fld nil 'no-ls)
 	    (if msg
-		(mew-summary-jump-message msg)
+		(mew-summary-move-and-display msg)
 	      (goto-char (point-max)))
 	    (message "Now in %s. Type '%s' again"
 		     fld
@@ -273,7 +310,23 @@ all marked messages before the current message."
      (t
       (message "Refiling and deleting...")
       (sit-for 0)
-      (mew-summary-exec-local beg end)))))
+      (mew-summary-exec-local beg end func)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Refile log
+;;;
+
+(defun mew-exec-refile-log (buf)
+  (when (get-buffer buf)
+    (save-excursion
+      (set-buffer buf)
+      (mew-frwlet
+       mew-cs-dummy mew-cs-m17n
+       (write-region (point-min) (point-max)
+		     (expand-file-name mew-refile-log-file mew-conf-path)
+		     'append 'no-msg)))
+    (mew-remove-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -287,16 +340,17 @@ all marked messages before the current message."
     (forward-line)
     (mew-elet (delete-region start (point)))))
 
-(defun mew-summary-exec-local (beg end)
+(defun mew-summary-exec-local (beg end &optional func-decide)
   (when (mew-summary-exclusive-p)
     (save-excursion
       (condition-case nil
-	  (let ((marks (mew-mark-get-all-marks))
-		(src (mew-summary-folder-name 'ext))
-		(m (make-marker))
-		(cnt 0)
-		(case-fold-search nil)
-		msg msgs err-folders mark func-exec func-sanity killp regex)
+	  (let* ((marks (mew-mark-get-all-marks))
+		 (src (mew-summary-folder-name 'ext))
+		 (lbuf (mew-refile-log-buffer src))
+		 (m (make-marker))
+		 (cnt 0)
+		 (case-fold-search nil)
+		 msg msgs err-folders mark func-exec func-sanity killp regex)
 	    (set-marker m end)
 	    (while marks
 	      (setq mark (car marks))
@@ -309,22 +363,21 @@ all marked messages before the current message."
 		(goto-char beg)
 		(while (re-search-forward regex m t)
 		  (setq msg (mew-summary-message-number))
-		  (setq msgs (cons msg msgs))
-		  (cond
-		   ((fboundp killp)
-		    (if (funcall killp src msg)
-			(mew-mark-kill-line)
-		      (mew-mark-delete-here t)))
-		   ((eq killp t)
-		    (mew-mark-kill-line))))
+		  (when (or (not func-decide) (funcall func-decide msg))
+		    (setq msgs (cons msg msgs))
+		    (cond
+		     ((fboundp killp)
+		      (if (funcall killp src msg)
+			  (mew-mark-kill-line)
+			(mew-mark-remove)))
+		     ((eq killp t)
+		      (mew-mark-kill-line)))))
 		(setq msgs (nreverse msgs))
 		(when msgs
 		  (when (= cnt 0)
 		    ;; opening...
 		    (mew-summary-lock t "Executing")
-		    (mew-window-configure 'summary)
-		    (mew-current-set nil nil nil)
-		    (mew-decode-syntax-delete))
+		    (mew-summary-reset))
 		  (setq cnt (1+ cnt))
 		  ;; refiling and deleting...
 		  (setq func-sanity (mew-markdb-func-sanity mark))
@@ -339,7 +392,6 @@ all marked messages before the current message."
 		(message "No marks")
 	      ;; ending...
 	      (mew-cache-clean-up)
-	      (mew-summary-reset-mode-line)
 	      (mew-summary-folder-cache-save)
 	      (set-buffer-modified-p nil)
 	      (mew-summary-unlock)
@@ -348,6 +400,7 @@ all marked messages before the current message."
 		;; This ensures that some messages are displayed in Summary.
 		(recenter))
 	      (run-hooks 'mew-summary-exec-hook)
+	      (mew-exec-refile-log lbuf)
 	      (message "Refiling and deleting...done")))
 	(quit
 	 (set-buffer-modified-p nil)
@@ -397,7 +450,8 @@ all marked messages before the current message."
 	(if modify
 	    (if del
 		(mew-mark-invisible-line)
-	      (mew-mark-delete-here t))))
+	      (mew-summary-refile-remove-body)
+	      (mew-mark-remove))))
       (setq refs (nreverse refs))
       (if modify (mew-sinfo-set-refile-back refile-back))
       ;;
@@ -431,16 +485,19 @@ all marked messages before the current message."
       (let* ((case (mew-sinfo-get-case))
 	     (folder (mew-sinfo-get-folder))
 	     (bnm (mew-summary-folder-name 'ext))
+	     (lbuf (mew-refile-log-buffer bnm))
 	     (info (mew-summary-exec-remote-get-info beg end 'modify))
 	     (refs (nth 0 info))
 	     (rmvs (nth 1 info))
 	     (kils (nth 2 info)))
 	(if (or refs rmvs)
-	    (cond
-	     ((mew-folder-popp folder)
-	      (mew-pop-retrieve case 'exec bnm refs rmvs kils))
-	     ((mew-folder-imapp folder)
-	      (mew-imap-retrieve case 'exec bnm refs rmvs kils)))
+	    (progn
+	      (cond
+	       ((mew-folder-popp folder)
+		(mew-pop-retrieve case 'exec bnm refs rmvs kils))
+	       ((mew-folder-imapp folder)
+		(mew-imap-retrieve case 'exec bnm refs rmvs kils)))
+	      (mew-exec-refile-log lbuf))
 	  (message "No messages to be processed"))))))
 
 (defun mew-mark-kill-invisible ()
@@ -482,6 +539,7 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
       (mew-summary-exec-offline-region beg end))))
 
 (defun mew-summary-exec-offline-region (beg end)
+  "Process the current marked message offline."
   (interactive)
   (let* ((info (mew-summary-exec-remote-get-info beg end nil))
 	 (refs (nth 0 info))
@@ -523,24 +581,44 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 	    (mew-frwlet
 	     mew-cs-dummy nil
 	     (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)))
+	  (mew-touch-folder imapq)
 	  (mew-remove-buffer buf)
+	  ;; refile log is not necessary since mew-summary-exec-local does it.
 	  (if (mew-imap-trash-folder case)
-	      (let* ((mew-inherit-case case)
+	      (let* ((mew-inherit-exec-case case)
 		     (mew-msg-rm-policy 'trashonly)
 		     (mew-trash-folder (mew-imap-trash-folder case))
 		     (mew-inherit-offline t))
 		(mew-summary-exec-local beg end))
-	    (let ((mew-inherit-case case)
+	    (let ((mew-inherit-exec-case case)
 		  (mew-msg-rm-policy 'always)
 		  (mew-inherit-offline t))
 	      (mew-summary-exec-local beg end))))
       (message "No messages to be processed"))))
 
+(defun mew-summary-exec-offline-one (&optional arg)
+  (interactive "P")
+  (if (not (mew-folder-imapp (mew-sinfo-get-folder)))
+      (message "This command cannot be used in this folder")
+    (mew-summary-goto-message)
+    (mew-decode-syntax-delete)
+    (save-window-excursion
+      (let (beg end)
+        (save-excursion
+          (beginning-of-line)
+          (setq beg (if arg (point-min) (point)))
+          (end-of-line)
+          (setq end (1+ (point))))
+        (if (> end (point-max))
+            (message "No message")
+          (sit-for 0)
+          (mew-summary-exec-offline-region beg end))))))
+
 (provide 'mew-exec)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
