@@ -24,6 +24,9 @@
 (defsubst mew-draft-p ()
   (eq major-mode 'mew-draft-mode))
 
+(defsubst mew-header-p ()
+  (eq major-mode 'mew-header-mode))
+
 (defsubst mew-draft-or-header-p ()
   (memq major-mode '(mew-draft-mode mew-header-mode)))
 
@@ -698,13 +701,11 @@ also ignored."
 	maxpath))))
 
 (defun mew-touch-folder (fld)
-  (let (file)
-    (when (and mew-touch-folder-p
-	       (stringp mew-summary-touch-file)
-	       (setq file (mew-expand-folder fld mew-summary-touch-file))
-	       (file-writable-p file))
-      (write-region "touched by Mew." nil file nil 'no-msg)
-      (mew-set-file-modes file))))
+  (when (stringp mew-summary-touch-file)
+    (let ((file (mew-expand-folder fld mew-summary-touch-file)))
+      (when (file-writable-p file)
+	(write-region "touched by Mew." nil file nil 'no-msg)
+	(mew-set-file-modes file)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -975,20 +976,39 @@ Does not examine containing directories for links."
 	(string-bytes (mew-buffer-substring beg end)))
     (- end beg)))
 
-(defun mew-count-lines (beg end &optional skip-regex)
+(defun mew-count-lines (beg end)
   "Return number of lines between BEG and END."
   (save-excursion
+    (goto-char beg)
+    (beginning-of-line)
+    (setq beg (point))
+    (goto-char end)
+    (beginning-of-line)
+    (setq end (point))
+    (if (or (not (mew-decode-syntax-p))
+	    (not (equal (mew-decode-syntax-buffer) (current-buffer))))
+	(mew-count-lines1 beg end)
+      (let ((mbeg (mew-decode-syntax-begin))
+	    (mend (mew-decode-syntax-end)))
+	(if (or (<= end mbeg) (>= beg mend))
+	    (mew-count-lines1 beg end)
+	  (+ (mew-count-lines1 beg mbeg) (mew-count-lines1 mend end)))))))
+
+(defun mew-count-lines1 (beg end)
+  (if (>= beg end) 
+      0
     (save-restriction
       (narrow-to-region beg end)
       (goto-char (point-min))
-      (if skip-regex
-	  (let ((lines 0))
-	    (while (not (eobp))
-	      (forward-line)
-	      (unless (looking-at skip-regex)
-		(setq lines (1+ lines))))
-	    lines)
-	(- (buffer-size) (forward-line (buffer-size)))))))
+      (if (not (and (mew-thread-p) mew-use-thread-separator))
+	  (- (buffer-size) (forward-line (buffer-size)))
+	(let ((regex mew-regex-thread-separator)
+	      (lines 0))
+	  (while (not (eobp))
+	    (unless (looking-at regex)
+	      (setq lines (1+ lines)))
+	    (forward-line))
+	  lines)))))
 
 (defun mew-buffer-list (regex &optional listp mode)
   (let ((bufs (mapcar 'buffer-name (buffer-list)))
