@@ -7,7 +7,7 @@
 
 (require 'mew)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cursor line
 ;;;
@@ -40,7 +40,7 @@
   (if (overlayp (mew-sinfo-get-cursor-line))
       (move-overlay (mew-sinfo-get-cursor-line) 1 1)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Marks
 ;;;
@@ -60,7 +60,7 @@
    (save-excursion (end-of-line) (point))
    '(face nil)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header
 ;;;
@@ -86,7 +86,7 @@
 	   (while (not (eobp))
 	     (if (not (looking-at mew-keyval))
 		 (forward-line)
-	       (setq key (mew-match 1))
+	       (setq key (mew-match-string 1))
 	       (setq beg (match-beginning 0))
 	       (setq med (match-end 0))
 	       (forward-line)
@@ -110,7 +110,7 @@
     (widen)
     (mew-highlight-header-region (point-min) (mew-header-end))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Body
 ;;;
@@ -139,7 +139,7 @@
 	     (setq beg (match-beginning 0))
 	     (setq end (match-end 0))
 	     (if (match-beginning 1)
-		 (setq fancy-prefix (mew-match 1))
+		 (setq fancy-prefix (mew-match-string 1))
 	       (setq fancy-prefix nil))
 	     (when (and fancy-prefix
 			(< (string-width fancy-prefix)
@@ -167,62 +167,50 @@
 	     (put-text-property
 	      beg end 'mouse-face mew-highlight-url-mouse-face))))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; X-Face:
+;;; X Face
 ;;;
 
-;; uncompface: ftp://ftp.cs.indiana.edu/pub/faces/xfaces/xfaces-<ver>.tar.Z
-;; icontopbm: ftp://ftp.sourceforge.net/pub/sourceforge/netpbm/netpbm-<ver>.tgz
-
 (defun mew-highlight-x-face (beg end)
-  "A function to display X-Face."
-  (if (and mew-use-highlight-x-face mew-use-highlight-x-face-function)
-      (funcall mew-use-highlight-x-face-function beg end)))
+  (when (and mew-use-highlight-x-face window-system)
+    (save-excursion
+      (goto-char beg)
+      (mew-elet
+       (let ((regex1 "^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n")
+	     (buf (current-buffer))
+	     overlay xface beg0 end0 xbmp)
+	 (while (re-search-forward regex1 end t)
+	   (setq beg0 (match-beginning 0))
+	   (setq end0 (match-end 0))
+	   (with-temp-buffer
+	     (mew-insert-buffer-substring
+	      buf (match-beginning 1) (match-end 1))
+	     (setq xbmp (mew-x-face-compface-to-xbm))
+	     (if xbmp (setq xface (mew-x-face-create))))
+	   (when xface
+	     (setq overlay (mew-overlay-make beg0 end0))
+	     (overlay-put overlay 'invisible t)
+	     (save-restriction
+	       (narrow-to-region beg end)
+	       (mew-x-face-display xface )))))))))
 
-(cond
- (mew-xemacs-p
-  ;;(autoload 'highlight-headers-x-face-to-pixmap "highlight-headers")
-  (if (mew-which-el "highlight-headers")
-      (require 'highlight-headers)) ;; due to the timing problem.
-  ;; now this is in the "mail-lib" package.
-  (defvar mew-use-highlight-x-face-function
-    (function
-     (lambda (beg end)
-       (when (and window-system
-		  (mew-which-exec mew-prog-uncompface)
-		  mew-use-highlight-x-face)
-	 (save-excursion
-	   (goto-char beg)
-	   (mew-elet
-	    (let ((regex1 "^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n")
-		  (regex2 (concat "^\\(" mew-from: "\\).*"))
-		  overlay xface)
-	      (while (re-search-forward regex1 end t)
-		(setq overlay (mew-overlay-make (match-beginning 0)
-						(match-end 0)))
-		(overlay-put overlay 'invisible t)
-		(mew-plet
-		 (setq xface (highlight-headers-x-face-to-pixmap
-			      (match-beginning 1)
-			      (match-end 1))))
-		(save-excursion
-		  (goto-char beg)
-		  (when (re-search-forward regex2 end t)
-		    (setq overlay (mew-overlay-make
-				   (match-end 1) (match-end 1)))
-		    (set-extent-begin-glyph overlay xface))))))))))))
- (t
-  (defvar mew-use-highlight-x-face-function nil
-    "*On Text Emacs, this function is called if mew-use-highlight-x-face
-is *non-nil*. This is a temporary solution.")
-  (if (and mew-use-highlight-x-face
-	   (mew-which-exec mew-prog-uncompface)
-	   (mew-which-el "bitmap")
-	   (mew-which-el "mew-xface-mule"))
-      (require 'mew-xface-mule))))
+(defun mew-x-face-compface-to-xbm ()
+  (when (and (mew-which-exec mew-prog-uncompface)
+	     (mew-which-exec mew-prog-icontopbm)
+	     (mew-which-exec mew-prog-pbmtoxbm))
+    (mew-set-buffer-multibyte nil)
+    (mew-flet
+     (call-process-region (point-min) (point-max) mew-prog-uncompface t t nil)
+     (goto-char (point-min))
+     (insert "/* Format_version=1, Width=48, Height=48, Depth=1, Valid_bits_per_item=16 */\n")
+     (call-process-region (point-min) (point-max) mew-prog-icontopbm t t nil)
+     (if mew-use-highlight-x-face-inversion
+	 (call-process-region (point-min) (point-max) mew-prog-pbminvert t t nil))
+     (call-process-region (point-min) (point-max) mew-prog-pbmtoxbm t t nil))
+    t)) ;; return value
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cooking
 ;;;
@@ -239,7 +227,8 @@ is *non-nil*. This is a temporary solution.")
 ;; See also mew-scan-insert-line
 (defun mew-summary-cook-region (beg end &optional interrupt)
   (when (and (mew-summary-or-virtual-p) mew-summary-buffer-raw)
-    (let ((inhibit-point-motion-hooks t) ret regex mark face start med)
+    (let ((inhibit-point-motion-hooks t)
+	  ret mark face start med)
       (catch 'loop
 	(save-excursion
 	  (mew-elet
@@ -271,12 +260,11 @@ is *non-nil*. This is a temporary solution.")
 	     (setq start (point)))
 	   (when mew-use-highlight-mark
 	     (goto-char beg)
-	     (setq regex (concat mew-regex-msg "\\([^ ]\\)"))
 	     (while (and (< (point) end) ;; we cannot trust end
-			 (re-search-forward regex end t))
+			 (re-search-forward mew-regex-mark end t))
 	       (if (and interrupt (input-pending-p))
 		   (throw 'loop (setq ret t)))
-	       (setq mark (string-to-char (mew-match 2)))
+	       (setq mark (mew-sumsyn-mark))
 	       (setq face (mew-highlight-mark-get-face mark))
 	       (beginning-of-line)
 	       (setq start (point))
@@ -313,7 +301,7 @@ is *non-nil*. This is a temporary solution.")
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
