@@ -90,7 +90,7 @@
 	  authl auth func)
       (save-excursion
 	(goto-char (point-min))
-	(while (re-search-forward "AUTH=\\([^\n\r ]+\\)" nil t)
+	(while (re-search-forward "AUTH=\\([^ \r\n]+\\)" nil t)
 	  (setq authl (cons (mew-match-string 1) authl))))
       (if (and authl
 	       (setq auth (mew-auth-select2 authl auth-list))
@@ -433,7 +433,7 @@
 	 (siz (nth 1 rtr))
 	 (fld-msg (nth 3 rtr)) ;; 'exec, 'scan, ...
 	 (truncated (mew-imap-get-truncated pnm))
-	 fld msg vec file msg-file lmsg)
+	 fld msg vec file msg-file lmsg mark)
     (cond
      ((null fld-msg)
       (setq fld (mew-imap-get-bnm pnm)))
@@ -467,9 +467,14 @@
     (delete-region (point) (point-max))
     ;; line delimiters
     (mew-eol-fix-for-read)
-    (setq msg-file (mew-net-get-new-message
-		    pnm fld msg 'mew-imap-get-msgdb 'mew-imap-set-msgdb))
-    (setq msg (car msg-file) file (cdr msg-file))
+    (cond
+     ((eq directive 'scan)
+      (setq msg uid) 
+      (setq file (mew-expand-folder fld msg)))
+     (t
+      (setq msg-file (mew-net-get-new-message
+		      pnm fld msg 'mew-imap-get-msgdb 'mew-imap-set-msgdb))
+      (setq msg (car msg-file) file (cdr msg-file))))
     (goto-char (point-min))
     (cond
      (truncated
@@ -491,11 +496,15 @@
       (mew-imap-set-rcnt pnm (1+ (mew-imap-get-rcnt pnm)))
       ;; xxx
       (mew-set-buffer-multibyte t)
-      (setq vec (mew-imap-scan-header))
+      (setq vec (mew-scan-header))
       (mew-scan-set-folder vec fld)
       (mew-scan-set-message vec msg)
+      (if (and (eq directive 'scan)
+	       (string= (mew-folder-folder fld) mew-imap-inbox-folder))
+	  (setq mark (mew-scan-inbox-action vec)))
+      (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg nil)
+      (mew-scan-insert-line fld vec width lmsg mark)
       (mew-imap-set-rtrs pnm (cdr rtrs))
       (mew-imap-set-status pnm "fetch")
       (mew-imap-command-fetch pro pnm))))
@@ -522,7 +531,7 @@
 	(unless regex
 	  (setq sep (mew-match-string 1))
 	  (unless (string= sep "NIL")
-	    (setq regex (concat "\\([^" sep "]+\\)$"))
+	    (setq regex (format "\\([^%s\n]+\\)$" sep))
 	    (setq mailboxes (cons (mew-folder-func mew-imap-inbox-folder sep) mailboxes))))
 	(setq mbx (mew-match-string 2))
 	(let ((case-fold-search nil))
@@ -663,6 +672,7 @@
 (defun mew-imap-process-send-string (pro pnm &rest args)
   (let ((str (apply 'format args))
 	(tag (mew-imap-tag)))
+    (mew-imap-debug "=SEND=" str)
     (mew-imap-set-tag pnm tag)
     (process-send-string pro (concat tag " " str mew-cs-eol))))
 
@@ -670,12 +680,6 @@
   (let ((str (apply 'format args)))
     (process-send-string pro (concat str mew-cs-eol))))
 
-(defun mew-imap-scan-header ()
-  (goto-char (point-min))
-  (unless (re-search-forward mew-eoh nil t)
-    (goto-char (point-max)))
-  (mew-scan-header))
-
 (defsubst mew-imap-message (pnm &rest args)
   (or (mew-imap-get-no-msg pnm) (apply 'message args)))
 
@@ -780,7 +784,7 @@
 	     (nconc (mew-sinfo-get-refile) (mew-sinfo-get-refile-back)))
 	    (mew-sinfo-set-refile-back nil))
 	(mew-summary-lock process "IMAPing" (or sshpro sslpro))
-	(mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-imap-set-no-msg pnm no-msg) ;; must come here
@@ -945,11 +949,11 @@
 	   (mew-imap-message pnm "No new messages"))
 	  ((= rttl 1)
 	   (mew-imap-message pnm "1 message retrieved")
-	   (mew-lisp-save file msgid)
+	   (mew-lisp-save file msgid nil 'unlimit)
 	   (mew-summary-folder-cache-save))
 	  (t
 	   (mew-imap-message pnm "%s messages retrieved" rttl)
-	   (mew-lisp-save file msgid)
+	   (mew-lisp-save file msgid nil 'unlimit)
 	   (mew-summary-folder-cache-save))))
 	((eq directive 'exec)
 	 (when kils
@@ -1089,7 +1093,7 @@
 
 (defun mew-imap-folder-insert (case folder)
   (let* ((sep (mew-imap-separator case))
-	 (regex (concat "\\([^" sep mew-folder-imap "]+\\)$"))
+	 (regex (format "\\([^%s%s\n]+\\)$" sep mew-folder-imap))
 	 (mailboxes (mew-imap-folder-alist case))
 	 (friends (mew-imap-friend-folder-list case))
 	 (friendp (concat "^" (regexp-quote (mew-imap-friend-folder case))))
