@@ -54,7 +54,7 @@
   (define-key mew-summary-mode-map "-"    'mew-summary-scroll-down)
   (define-key mew-summary-mode-map "\e\r" 'mew-summary-scroll-down)
   (define-key mew-summary-mode-map "g"    'mew-summary-goto-folder)
-  (define-key mew-summary-mode-map "j"    'mew-summary-jump-message)
+  (define-key mew-summary-mode-map "j"    'mew-summary-goto-line)
   (define-key mew-summary-mode-map "i"    'mew-summary-retrieve)
   (define-key mew-summary-mode-map "I"    'mew-summary-retrieve-message)
   (define-key mew-summary-mode-map "a"    'mew-summary-reply)
@@ -73,7 +73,10 @@
   (define-key mew-summary-mode-map "U"    'mew-summary-undo-all)
   (define-key mew-summary-mode-map "n"    'mew-summary-display-down)
   (define-key mew-summary-mode-map "p"    'mew-summary-display-up)
-  (define-key mew-summary-mode-map "M"    'mew-summary-mark-duplicated)
+  (define-key mew-summary-mode-map "\C-n" 'mew-summary-next-line)
+  (define-key mew-summary-mode-map "\C-p" 'mew-summary-previous-line)
+  (define-key mew-summary-mode-map "M"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "Md"   'mew-summary-mark-duplicated)
   (define-key mew-summary-mode-map "N"    'mew-summary-display-review-down)
   (define-key mew-summary-mode-map "P"    'mew-summary-display-review-up)
   (define-key mew-summary-mode-map "\en"  'mew-summary-find-keyword-down)
@@ -134,8 +137,9 @@
   (define-key mew-summary-mode-map "mt"   'mew-summary-mark-make-thread)
   (define-key mew-summary-mode-map "t"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "tt"   'mew-summary-make-thread)
-  (define-key mew-summary-mode-map "tn"   'mew-summary-thread-down)
-  (define-key mew-summary-mode-map "tp"   'mew-summary-thread-up)
+  (define-key mew-summary-mode-map "tn"   'mew-thread-down)
+  (define-key mew-summary-mode-map "tp"   'mew-thread-up)
+  (define-key mew-summary-mode-map "t "   'mew-thread-toggle)
   (define-key mew-summary-mode-map "t*"   'mew-thread-mark-review)
   (define-key mew-summary-mode-map "t@"   'mew-thread-mark-multi)
   (define-key mew-summary-mode-map "td"   'mew-thread-mark-delete)
@@ -143,6 +147,7 @@
   (define-key mew-summary-mode-map "tc"   'mew-thread-mark-copy)
   (define-key mew-summary-mode-map "t\ed" 'mew-thread-mark-unlink)
   (define-key mew-summary-mode-map "tu"   'mew-thread-unmark)
+  (define-key mew-summary-mode-map "tg"   'mew-thread-glue)
   (define-key mew-summary-mode-map "z"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "z8"   'mew-summary-toggle-8bit)
   (define-key mew-summary-mode-map "zc"   'mew-summary-cache-clean-up)
@@ -151,6 +156,7 @@
   (define-key mew-summary-mode-map "zw"   'mew-summary-toggle-warning)
   (define-key mew-summary-mode-map "l"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "lc"   'mew-summary-local-copy)
+  (define-key mew-summary-mode-map "="    'mew-summary-info)
   ;;
   ;; not provided in Virtual mode
   ;;
@@ -168,12 +174,12 @@
   (define-key mew-summary-mode-map "md"   'mew-summary-mark-delete)
   (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
   (define-key mew-summary-mode-map "m\ed" 'mew-summary-mark-unlink)
-  (define-key mew-summary-mode-map "mxd"  'mew-summary-exec-delete)
+  (define-key mew-summary-mode-map "mxd"   'mew-summary-exec-delete)
   (define-key mew-summary-mode-map "mx\ed" 'mew-summary-exec-unlink)
-  (define-key mew-summary-mode-map "mxo"  'mew-summary-exec-refile)
+  (define-key mew-summary-mode-map "mxo"   'mew-summary-exec-refile)
 
-  (define-key mew-summary-mode-map "O"    'mew-summary-pack)
   (define-key mew-summary-mode-map "s"    'mew-summary-ls)
+  (define-key mew-summary-mode-map "O"    'mew-summary-pack)
   (define-key mew-summary-mode-map "S"    'mew-summary-sort)
   (define-key mew-summary-mode-map "\C-c\C-b" 'mew-summary-exchange-point)
   (define-key mew-summary-mode-map "\eo"  'mew-summary-auto-refile)
@@ -189,7 +195,6 @@
     ["Previous" mew-summary-display-up   t]
     ["Top"      mew-summary-jump-top     t]
     ["Bottom"   mew-summary-jump-bottom  t]
-    ["Jump"     mew-summary-jump-message t]
     "----"
     ["Delete"       mew-summary-delete       (mew-summary-p)]
     ["Refile"       mew-summary-refile       (mew-summary-case1)]
@@ -205,7 +210,6 @@
     ("Manipulate folder"
      ["Retrieve"     mew-summary-retrieve    t]
      ["List"         mew-summary-ls          (mew-summary-p)]
-     ["Pack"         mew-summary-pack        (mew-summary-case2)]
      ["Sort"         mew-summary-sort        (mew-summary-case2)]
      ["Burst"        mew-summary-burst       t]
      ["Go to folder" mew-summary-goto-folder t]
@@ -307,6 +311,7 @@
 (unless mew-draft-body-map
   (setq mew-draft-body-map (make-sparse-keymap))
   (mew-set-keymap-parent mew-draft-body-map text-mode-map)
+  (define-key mew-draft-body-map "\C-x\C-s"     'mew-draft-save-buffer)
   (define-key mew-draft-body-map "\C-c\t"       'mew-draft-insert-signature)
   (define-key mew-draft-body-map "\C-c\C-y"     'mew-draft-cite)
   (define-key mew-draft-body-map "\C-c\C-t"     'mew-draft-yank)
@@ -414,6 +419,7 @@
 
 (unless mew-header-mode-map
   (setq mew-header-mode-map (make-sparse-keymap))
+  (define-key mew-header-mode-map "\C-x\C-s" 'mew-draft-save-buffer)
   (define-key mew-header-mode-map "\t"       'mew-draft-header-comp)
   (define-key mew-header-mode-map "\C-c\t"   'mew-draft-circular-comp)
   (define-key mew-header-mode-map "\e\t"     'mew-draft-expand)
