@@ -82,7 +82,7 @@ each candidate is a directory in 'while' loop."
 (defun mew-local-folder-entry (ent1 ent2 make-list)
   (if make-list
       ent1
-    (list ent1 ent2)))
+    (mew-folder-func ent1 ent2)))
 
 (defun mew-local-folder-make-alist (dirs prefix &optional make-list)
   (let (fldpfx dir ret ent)
@@ -178,8 +178,8 @@ each candidate is a directory in 'while' loop."
 	 ;; foo/bar -> bar 
 	 ;; foo -> foo
     (if (string-match mew-regex-ignore-folders folder)
-	(list folder nil)
-      (list folder subdir))))
+	(mew-folder-func folder)
+      (mew-folder-func folder subdir))))
 
 (defun mew-local-folder-insert (folder)
   "Insert FOLDER to 'mew-local-folder-alist'.
@@ -213,11 +213,17 @@ Binary search is used for speed reasons."
       (setq parent (file-name-directory (directory-file-name folder)))
       (if parent (mew-local-folder-insert1 parent)))))
 
-(defun mew-local-folder-check (folder)
+(defun mew-local-folder-check (folder &optional ask)
   (let ((absdir (mew-expand-folder folder)))
-    (unless (file-directory-p absdir)
-      (mew-make-directory absdir)
-      (mew-local-folder-insert folder))))
+    (if (file-directory-p absdir)
+	t
+      (if (or (not ask)
+	      (y-or-n-p (format "%s does not exist. Create it? " folder)))
+	  (progn
+	    (mew-make-directory absdir)
+	    (mew-local-folder-insert folder)
+	    folder)
+	nil))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -225,7 +231,7 @@ Binary search is used for speed reasons."
 ;;;
 
 (defvar mew-local-info-list
-  '("bnm" "mkdb" "flush" "rcnt" "rttl" "first"))
+  '("directive" "bnm" "mkdb" "flush" "rcnt" "rttl" "first"))
 
 (mew-info-defun "mew-local-" mew-local-info-list)
 
@@ -249,7 +255,7 @@ Binary search is used for speed reasons."
 ;;; Scan
 ;;;
 
-(defun mew-scan-mewls-src (folder &optional range)
+(defun mew-scan-mewl-src (folder &optional range)
   (setq folder (mew-expand-folder2 folder))
   (if range
       (list "-s" (format "%s %s" folder range))
@@ -258,36 +264,37 @@ Binary search is used for speed reasons."
 (defun mew-local-retrieve (directive &rest args)
   ;; 'inc  folder opts
   ;; 'scan folder range
-  ;; 'vir  opts count func
-  (mew-summary-with-mewls
+  ;; 'vir  opts func lra
+  (mew-summary-with-mewl
    (let* ((process-connection-type mew-connection-type-for-scan)
 	  (bnm (mew-summary-folder-name 'ext))
 	  (pnm (mew-local-info-name bnm))
 	  (buf (get-buffer-create (mew-local-buffer-name bnm)))
 	  range pro opts)
      (message "Scanning %s..." bnm)
-     (mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+     (mew-sinfo-set-summary-form (mew-get-summary-form bnm))
      (mew-sinfo-set-scan-id nil)
      (mew-info-clean-up pnm)
+     (mew-local-set-directive pnm directive)
      (mew-local-set-bnm pnm bnm)
      (cond
       ((eq directive 'inc)
-       (mew-local-set-flush pnm t)
-       (setq opts (append (nth 1 args) (mew-scan-mewls-src (nth 0 args)))))
+       (mew-local-set-flush pnm (nth 1 args))
+       (setq opts (append (nth 2 args) (mew-scan-mewl-src (nth 0 args)))))
       ((eq directive 'scan)
        (setq range (nth 1 args))
        (when (nth 2 args)
 	 ;; erasing
 	 (mew-local-set-mkdb pnm (mew-summary-mark-collect4))
 	 (mew-erase-buffer)
+	 (setq mew-summary-buffer-raw nil)
 	 (mew-summary-folder-cache-save))
-       (setq opts (mew-scan-mewls-src (nth 0 args) range)))
+       (setq opts (mew-scan-mewl-src (nth 0 args) range)))
       ((eq directive 'vir)
        (setq opts (nth 0 args))
-       (mew-vinfo-set-count (nth 1 args))
-       (mew-vinfo-set-func  (nth 2 args))
-       (mew-vinfo-set-lra   (nth 3 args))
-       (unless (nth 4 args) (mew-erase-buffer))))
+       (mew-vinfo-set-func  (nth 1 args))
+       (mew-vinfo-set-lra   (nth 2 args))
+       (unless (nth 3 args) (mew-erase-buffer))))
      (mew-sinfo-set-start-point (point)) ;; after erase-buffer
      (mew-local-set-rcnt pnm 1)
      (save-excursion
@@ -295,14 +302,15 @@ Binary search is used for speed reasons."
        (mew-erase-buffer))
      (setq opts (append (list "-b" mew-mail-path
 			      "-l" (int-to-string mew-scan-max-field-length)
-			      "-w" (int-to-string mew-scan-wait-for)
+			      "-c" (int-to-string mew-scan-max-body-length)
+			      "-w"
 			      "-f" (mapconcat 'identity
 					      (nthcdr 2 mew-scan-fields)
 					      ","))
 			opts))
-     (setq pro (apply 'start-process pnm buf mew-prog-mewls opts))
+     (setq pro (apply 'start-process pnm buf mew-prog-mewl opts))
      (mew-summary-lock pro "Scanning")
-     (mew-set-process-cs pro mew-cs-text-for-read mew-cs-dummy)
+     (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-write)
      ;; text may be broken, so undecided is very dangerous!
      (set-process-filter   pro 'mew-local-filter)
      (set-process-sentinel pro 'mew-local-sentinel)
@@ -313,31 +321,36 @@ Binary search is used for speed reasons."
 	 (pnm (process-name process))
 	 (bnm (mew-local-get-bnm pnm))
 	 (first (mew-local-get-first pnm))
-	 (virtualp (mew-folder-virtualp bnm))
 	 (draftp (mew-folder-draftp bnm))
-	 vec rttl)
+	 (directive (mew-local-get-directive pnm))
+	 vec rttl mark)
     (mew-local-debug "FILTER" string)
     (mew-filter
-     (goto-char (point-max))
      (mew-set-buffer-multibyte nil)
+     (goto-char (point-max))
      (mew-elet (insert string))
-     (goto-char (point-min))
-     (unless first
-       (when (looking-at "NumOfMsg: \\([0-9]+\\)")
-	 (setq rttl (string-to-int (match-string 1)))
-	 (mew-local-set-rttl pnm rttl)
-	 (forward-line)
-	 (delete-region (point-min) (point)))
-       (mew-local-set-first pnm t))
-     (while (and (re-search-forward mew-eoh nil t) (not (eobp)))
+     (goto-char (point-max))
+     (forward-line -1)
+     (when (looking-at "^\\.$")
+       (mew-dot-delete)
+       (goto-char (point-min))
+       (unless first
+	 (when (looking-at "NumOfMsg: \\([0-9]+\\)")
+	   (setq rttl (string-to-int (match-string 1)))
+	   (mew-local-set-rttl pnm rttl)
+	   (forward-line)
+	   (delete-region (point-min) (point)))
+	 (mew-local-set-first pnm t))
        (mew-net-status3 bnm (mew-local-get-rttl pnm) (mew-local-get-rcnt pnm))
        (mew-local-set-rcnt pnm (1+ (mew-local-get-rcnt pnm)))
        (mew-set-buffer-multibyte t)
        (setq vec (mew-scan-header draftp))
-       (mew-set-buffer-multibyte nil)
-       (mew-scan-insert-line bnm vec width nil virtualp)
-       (forward-line)
-       (delete-region (point-min) (point))))))
+       (if (eq directive 'inc) (setq mark (mew-scan-inbox-action vec)))
+       (mew-scan-body vec draftp)
+       (mew-scan-insert-line bnm vec width nil mark)
+       (delete-region (point-min) (point-max))
+       (sit-for 0) ;; for XEmacs
+       (process-send-string process "\n")))))
 
 (defun mew-local-sentinel (process event)
   (let* ((pnm (process-name process))
@@ -357,22 +370,23 @@ Binary search is used for speed reasons."
        (setq mdb (cdr mdb))
        (setq msg (car ent))
        (setq mrk (cdr ent))
-       (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	 (mew-summary-mark-as mrk 'force)
+       (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	 (mew-mark-put mrk)
 	 (forward-line)))
      (goto-char opos)
      (if (and virtualp (mew-vinfo-get-func))
 	 (funcall (mew-vinfo-get-func)))
      (mew-vinfo-set-func nil)
      (mew-info-clean-up pnm)
-     (unless virtualp
+     (if virtualp
+	 (mew-summary-set-count-line)
        (mew-summary-folder-cache-save))
      (set-buffer-modified-p nil)
      (mew-summary-unlock)
      (if (and (mew-folder-imapp bnm)
 	      (not (mew-folder-imap-queuep)))
 	 (if (mew-local-get-rttl pnm)
-	     (message (substitute-command-keys "\\<mew-summary-mode-map>Type '\\[mew-summary-ls]' to override invalid messages"))
+	     (message "%s" (substitute-command-keys "\\<mew-summary-mode-map>Type '\\[mew-summary-ls]' to override invalid messages"))
 	   ;; This folder has been just created.
 	   ;; Ideally scan should not be called. But there is no way
 	   ;; to avoid the call.
@@ -390,7 +404,7 @@ Binary search is used for speed reasons."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2001-2002 Mew developing team.
+;; Copyright (C) 2001-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
