@@ -14,10 +14,9 @@
 
 (defun mew-mark-unrefile (fld msg)
   "Delete refile state and delete the mark."
-  (when (get-buffer fld)
-    (save-excursion
-      (set-buffer fld)
-      (mew-refile-reset msg))))
+  (mew-summary-refile-remove-body)
+  (unless (mew-virtual-p)
+    (mew-refile-reset msg)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -26,9 +25,9 @@
 
 (defsubst mew-mark-unmark ()
   (save-excursion
-    (beginning-of-line)
-    (if (re-search-forward mew-regex-msg nil t)
-	(mew-mark-delete-here))))
+    (mew-summary-goto-mark)
+    (when (looking-at mew-regex-mark)
+      (mew-mark-remove))))
 
 (defun mew-summary-undo (&optional count)
   "Cancel the mark in COUNT times."
@@ -49,13 +48,10 @@
 	  (or (fboundp func) (setq func nil))
 	  (setq fld (mew-summary-folder-name))
 	  (setq msg (mew-summary-message-number))
-	  (cond
-	   ((mew-virtual-p)
-	    (mew-mark-unmark)
-	    (mew-summary-unmark-in-physical fld msg func))
-	   (t ;; Summary mode
-	    (mew-mark-unmark)
-	    (if func (funcall func fld msg)))))))))
+	  (mew-mark-unmark)
+	  (if func (funcall func fld msg))
+	  (when (mew-virtual-p)
+	    (mew-summary-unmark-in-physical fld msg func)))))))
 
 ;;
 
@@ -81,11 +77,11 @@ If optional argument NO-MSG is non-nil, no message is displayed."
       (while (re-search-forward regex nil t)
 	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number))
-	(mew-mark-delete-here t)
+	(mew-mark-remove)
 	(if (mew-virtual-p)
 	    (mew-mark-alist-set alist fld msg)
 	  (if func (funcall func fld msg))) ;; in physical only
-	(forward-line))  ;; make search faster
+	(forward-line))
       (set-buffer-modified-p nil)
       (mew-summary-unmark-in-physical-alist alist func)))
   (or no-msg (message "Unmarking...done")))
@@ -110,34 +106,32 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 (defun mew-summary-get-mark ()
   "Get a mark on the current message."
   (save-excursion
-    (beginning-of-line)
-    (if (looking-at mew-regex-msg-mark)
-	(string-to-char (mew-match-string 2))
-      nil)))
+    (mew-summary-goto-mark)
+    (when (looking-at mew-regex-mark)
+      (mew-sumsyn-mark)))) ;; return char
 
 (defalias 'mew-summary-marked-p 'mew-summary-get-mark)
 
-(defsubst mew-summary-mark-as (mark &optional force)
+(defsubst mew-summary-mark-as (mark)
   "Mark this message with MARK if possible."
-   (when (or force (not (mew-summary-marked-p)))
-     (save-excursion
-       (beginning-of-line)
-       (when (re-search-forward mew-regex-msg nil t)
-	 (mew-mark-put-here mark)))))
+  (unless (mew-summary-marked-p)
+    (mew-mark-put mark)))
 
-(defsubst mew-mark-put-here (mark &optional after-mark)
-  (mew-elet
-   (if after-mark (forward-char -1))
-   (delete-char 1)
-   (insert-and-inherit (char-to-string mark)) ;; inherit highlight
-   (mew-highlight-mark-line mark)))
+(defsubst mew-mark-put (mark)
+  (save-excursion
+    (mew-summary-goto-mark)
+    (mew-elet
+     (delete-char 1)
+     (insert-and-inherit (char-to-string mark)) ;; inherit highlight
+     (mew-highlight-mark-line mark))))
 
-(defsubst mew-mark-delete-here (&optional after-mark)
-  (mew-elet
-   (if after-mark (forward-char -1))
-   (delete-char 1)
-   (insert-and-inherit " ") ;; inherit highlight
-   (mew-highlight-unmark-line)))
+(defsubst mew-mark-remove ()
+  (save-excursion
+    (mew-summary-goto-mark)
+    (mew-elet
+     (delete-char 1)
+     (insert-and-inherit " ") ;; inherit highlight
+     (mew-highlight-unmark-line))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -182,9 +176,9 @@ cons pairs of folder name and message number."
   (save-excursion
     (let (ret mrk msg)
       (goto-char (point-min))
-      (while (re-search-forward mew-regex-msg-mark nil t)
-	(setq msg (mew-match-string 1))
-	(setq mrk (string-to-char (mew-match-string 2)))
+      (while (re-search-forward mew-regex-mark nil t)
+	(setq mrk (mew-sumsyn-mark))
+	(setq msg (mew-summary-message-number))
 	(setq ret (cons (cons msg mrk) ret))
 	(forward-line))
       (nreverse ret))))
@@ -228,15 +222,15 @@ If NO-MSG is non-nil, no message is displayed.
 NO-MSG also means that this function is being called in loop."
   (mew-summary-msg-or-part
    (let (oldmark oldlevel oldname newlevel newname case zmacs-regions
-	 msg fld marked invalidp)
+	 msg fld marked validp)
      (save-excursion
        (mew-summary-goto-message)
-       (setq invalidp (and valid-only (mew-summary-msg-invalidp)))
-       (if invalidp
+       (when (mew-sumsyn-match mew-regex-sumsyn-short)
+	 (setq msg (mew-sumsyn-message-number))
+	 (setq fld (mew-sumsyn-folder-name))
+	 (setq validp (or (not valid-only) (mew-msg-validp msg))))
+       (if (not validp)
 	   (unless no-msg (message "Cannot mark this invalid message with '%c'" newmark))
-	 (when (mew-virtual-p)
-	   (setq msg (mew-summary-message-number))
-	   (setq fld (mew-summary-folder-name)))
 	 (setq oldmark (mew-summary-get-mark))
 	 (setq oldlevel (mew-markdb-level oldmark))
 	 (setq oldname (mew-markdb-name oldmark))
@@ -245,7 +239,7 @@ NO-MSG also means that this function is being called in loop."
 	 (cond
 	  ((null oldmark);; no mark
 	   (setq case 1)
-	   (mew-summary-mark-as newmark)
+	   (mew-mark-put newmark)
 	   (setq marked t))
 	  ((eq oldmark newmark)
 	   (setq case 2)
@@ -255,7 +249,7 @@ NO-MSG also means that this function is being called in loop."
 	  ((< oldlevel newlevel)
 	   (setq case 3)
 	   (mew-summary-undo-one no-msg)
-	   (mew-summary-mark-as newmark)
+	   (mew-mark-put newmark)
 	   (setq marked t))
 	  ((= oldlevel newlevel)
 	   (cond
@@ -266,20 +260,20 @@ NO-MSG also means that this function is being called in loop."
 		 (progn
 		   (setq case 4)
 		   (mew-summary-undo-one no-msg)
-		   (mew-summary-mark-as newmark)
+		   (mew-mark-put newmark)
 		   (setq marked t))
 	       (setq case 5)))
 	    (t
 	     (setq case 6)
 	     (mew-summary-undo-one no-msg)
-	     (mew-summary-mark-as newmark)
+	     (mew-mark-put newmark)
 	     (setq marked t))))
 	  (t ;; > oldlevel newlevel
 	   (setq case 7)
 	   (message "Cannot mark here because '%s' is stronger than '%s'"
 		    oldname newname)))))
-     (unless invalidp
-       (if (and msg fld marked)
+     (when validp
+       (if (and marked (mew-virtual-p))
 	   (mew-summary-mark-in-physical fld msg newmark))
        (or no-msg (mew-mark-afterstep newmark case))
        (set-buffer-modified-p nil)))))
@@ -324,8 +318,8 @@ original position."
     (set-buffer fld)
     (save-excursion
       (goto-char (point-min))
-      (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	(mew-mark-put-here new-mark t)
+      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(mew-mark-put new-mark)
 	(set-buffer-modified-p nil)))))
 
 (defun mew-summary-unmark-in-physical (fld msg &optional func)
@@ -333,8 +327,8 @@ original position."
     (set-buffer fld)
     (save-excursion
       (goto-char (point-min))
-      (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	(mew-mark-delete-here t)
+      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(mew-mark-remove)
 	(if func (funcall func fld msg))
 	(set-buffer-modified-p nil)))))
 
@@ -345,7 +339,7 @@ original position."
 	 (nconc fld-msgs (list imsg))
        (setq ,alist (cons (list ,fld imsg) ,alist)))))
 
-(defun mew-summary-mark-in-physical-alist (alist mark)
+(defun mew-summary-mark-in-physical-alist (alist mark &optional func)
   (let (ent fld msg msgs)
     (while alist
       (setq ent (car alist))
@@ -359,8 +353,9 @@ original position."
 	  (while msgs
 	    (setq msg (int-to-string (car msgs)))
 	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	      (mew-mark-put-here mark t)
+	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	      (if func (funcall func))
+	      (mew-mark-put mark)
 	      (forward-line)))
 	  (set-buffer-modified-p nil))))))
 
@@ -378,8 +373,8 @@ original position."
 	  (while msgs
 	    (setq msg (int-to-string (car msgs)))
 	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	      (mew-mark-delete-here t)
+	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	      (mew-mark-remove)
 	      (if func (funcall func fld msg))
 	      (forward-line)))
 	  (set-buffer-modified-p nil))))))
@@ -409,7 +404,7 @@ See also '\\[mew-summary-mark-refile]', '\\[mew-summary-mark-delete]', '\\[mew-s
 (defun mew-summary-multi (&optional count)
   "\\<mew-summary-mode-map>
 Put the multi mark (default is '@') in COUNT times
-for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'. "
+for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'."
   (interactive "P")
   (mew-mark-put-mark-loop 'mew-summary-multi-one count nil))
 
@@ -476,7 +471,7 @@ BEG and END."
 	  (setq fld (mew-summary-folder-name))
 	  (setq msg (mew-summary-message-number))
 	  (mew-mark-alist-set alist fld msg))
-	(forward-line)) ;; make search faster
+	(forward-line))
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)))
 
@@ -498,14 +493,14 @@ BEG and END."
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
-	(when (looking-at "[^\n]*\r") ;; before thread info
-	  (mew-summary-mark-as mark)
-          (setq n (1+ n))
+	(when (mew-sumsyn-match mew-regex-sumsyn-short)
 	  (when (mew-virtual-p)
-	    (setq fld (mew-summary-folder-name))
-	    (setq msg (mew-summary-message-number))
-	    (mew-mark-alist-set alist fld msg)))
-	(forward-line)) ;; make search faster
+	    (setq fld (mew-sumsyn-folder-name))
+	    (setq msg (mew-sumsyn-message-number))
+	    (mew-mark-alist-set alist fld msg))
+	  (mew-summary-mark-as mark)
+          (setq n (1+ n)))
+	(forward-line))
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)
     (cond
@@ -522,25 +517,25 @@ BEG and END."
 ;;;
 
 (defun mew-summary-exchange-mark (oldmark newmark &optional valid-only)
-  (let ((regex (if valid-only 
-		   (mew-mark-valid-regex oldmark)
-		 (mew-mark-regex oldmark)))
+  (let ((regex (mew-mark-regex oldmark))
 	(case-fold-search nil)
-	fld msg alist)
+	(i 0) fld msg alist)
     (save-excursion
       (goto-char (point-min))
-      (if (not (re-search-forward regex nil t))
-	  (message "No marked messages")
-	(beginning-of-line)
-	(while (re-search-forward regex nil t)
-	  (mew-mark-put-here newmark t)
-	  (when (mew-virtual-p)
-	    (setq fld (mew-summary-folder-name))
-	    (setq msg (mew-summary-message-number))
-	    (mew-mark-alist-set alist fld msg))
-	  (forward-line)) ;; make search faster
-	(set-buffer-modified-p nil)
-	(mew-summary-mark-in-physical-alist alist newmark)))))
+      (while (re-search-forward regex nil t)
+	(when (mew-sumsyn-match mew-regex-sumsyn-short)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq msg (mew-sumsyn-message-number))
+	  (when (or (not valid-only) (mew-msg-validp msg))
+	    (mew-mark-put newmark)
+	    (when (mew-virtual-p)
+	      (mew-mark-alist-set alist fld msg))))
+	(forward-line)
+	(setq i (1+ i))))
+    (if (= i 0)
+	(message "No marked messages")
+      (set-buffer-modified-p nil)
+      (mew-summary-mark-in-physical-alist alist newmark))))
    
 (defun mew-summary-mark-delete ()	;; * -> D
   "Put the delete mark onto all messages marked with '*'."
@@ -585,7 +580,7 @@ region."
     (if (and (or (eq mew-mark-duplicated mew-mark-delete)
 		 (eq mew-mark-duplicated mew-mark-unlink))
 	     (eq mew-summary-mark-duplicated-skip nil))
-	(message "Cannot mark because messages may lost by this setting.")
+	(message "Cannot mark because messages may lost by this setting")
       (let ((reversep (eq mew-summary-mark-duplicated-skip 'last))
 	    (count 0) my-id dup-id size ids beg end region)
 	(cond
@@ -599,7 +594,7 @@ region."
 	(message "Marking duplications...")
 	(save-excursion
 	  ;; from mew-summary-thread-region in mew-threaed.el
-	  (setq size (mew-count-lines beg end))
+	  (setq size (count-lines beg end))
 	  (cond
 	   ((<= size 211)
 	    (setq size 211))
@@ -627,9 +622,8 @@ region."
 			;; second time (first duplication)
 			(unless mew-summary-mark-duplicated-skip
 			  (save-excursion
-			    (when (search-backward
-				   (format "\r <%s> " my-id) nil t)
-			      (beginning-of-line)
+			    (when (re-search-backward
+				   (mew-regex-sumsyn-my-id my-id) nil t)
 			      (mew-mark-put-mark mew-mark-duplicated 'no-msg)
 			      (setq count (1+ count))))
 			  (set (intern my-id ids) nil)))
@@ -752,7 +746,7 @@ Typically called by kill-emacs-hook."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
