@@ -266,6 +266,9 @@
 	  (unless esqp (setq arg (cons file arg)))
 	  (setq arg (nreverse arg))
 	  (setq wcs (mew-text/html-detect-cs begin end params))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param params "charset"))))
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
@@ -366,6 +369,9 @@
 	  (unless esqp (setq arg (cons file arg)))
 	  (setq arg (nreverse arg))
 	  (setq wcs (mew-text/xml-detect-cs begin end params))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param params "charset"))))
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
