@@ -65,7 +65,8 @@
 (defun mew-summary-unlock ()
   (setq mew-summary-buffer-process nil)
   (setq mew-summary-buffer-process-status nil)
-  (setq mew-summary-buffer-secure-process nil))
+  (setq mew-summary-buffer-secure-process nil)
+  (force-mode-line-update))
 
 (defun mew-net-status-clear (buf)
   (when mew-use-net-status
@@ -214,7 +215,7 @@
     (if ent
 	(setcdr ent uidl)
       (setq mew-net-uidl-db (cons (cons tag uidl) mew-net-uidl-db)))
-    (mew-lisp-save mew-net-uidl-file mew-net-uidl-db)))
+    (mew-lisp-save mew-net-uidl-file mew-net-uidl-db nil 'unlimit)))
 
 (defun mew-net-setup ()
   (setq mew-net-uidl-db (mew-lisp-load mew-net-uidl-file))
@@ -222,7 +223,7 @@
 
 (defun mew-net-clean-up ()
   (remove-hook 'kill-emacs-hook 'mew-net-clean-up)
-  (mew-lisp-save mew-net-uidl-file mew-net-uidl-db))
+  (mew-lisp-save mew-net-uidl-file mew-net-uidl-db nil 'unlimit))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -379,6 +380,7 @@
 	 (msgs (mew-dir-messages dir mew-regex-message-files3)))
     (mew-summary-unlink-msgs bnm msgs)
     (mew-erase-buffer)
+    (setq mew-summary-buffer-raw nil)
     (mew-summary-folder-cache-save)))
 
 (defun mew-net-invalid-cache-clean ()
@@ -388,7 +390,7 @@
 	 beg)
     (save-excursion
       (goto-char (point-min))
-      (when (re-search-forward mew-regex-msg-invalid nil t)
+      (when (re-search-forward mew-regex-sumsyn-invalid nil t)
 	(beginning-of-line)
 	(setq beg (point))
 	(goto-char (point-max))
@@ -427,11 +429,7 @@ If called with '\\[universal-argument]', +queue is not flushed."
     (mew-summary-switch-to-folder inbox)
     ;; in the inbox
     (when (mew-summary-exclusive-p)
-      (mew-window-configure 'summary)
-      ;;
-      (mew-current-set nil nil nil)
-      (mew-decode-syntax-delete)
-      (mew-unhighlight-cursor-line)
+      (mew-summary-reset)
       ;;
       (mew-summary-retrieve-gap inbox)
       ;;
@@ -459,11 +457,7 @@ If called with '\\[universal-argument]', +queue is not flushed."
     ;; for mew-summary-exchange-point.
     (set-marker mew-summary-inbox-position (point) (current-buffer))
     (when (mew-summary-exclusive-p)
-      (mew-window-configure 'summary)
-      ;;
-      (mew-current-set nil nil nil)
-      (mew-decode-syntax-delete)
-      (mew-unhighlight-cursor-line)
+      (mew-summary-reset)
       ;;
       (if (mew-folder-localp inbox) (mew-summary-retrieve-gap case:inbox))
       ;;
@@ -516,14 +510,17 @@ the mailbox. When executed with '\\[universal-argument]',
 In a REMOTE folder: case and protocol are determined by the folder.
 The message in the server side is always retained."
   (interactive "P")
-  (mew-summary-msg
-   (mew-summary-only
-    (unless (mew-summary-msg-invalidp)
-      (when (mew-summary-exclusive-p)
-	(let* ((bnm (mew-summary-folder-name 'ext))
+  (mew-summary-only
+   (mew-summary-msg
+    (when (mew-summary-exclusive-p)
+      (when (mew-sumsyn-match mew-regex-sumsyn-long)
+	(let* ((msg (mew-sumsyn-message-number))
+	       (uid (mew-sumsyn-message-uid))
+	       (siz (mew-sumsyn-message-size))
+	       (bnm (mew-summary-folder-name 'ext))
 	       (folder (mew-sinfo-get-folder))
 	       (case (mew-sinfo-get-case))
-	       msg uid siz del rtr rtrs mailbox)
+	       del rtr rtrs mailbox)
 	  (when (mew-folder-localp folder)
 	    (setq case mew-case-input)
 	    (setq mailbox (mew-mailbox-type case))
@@ -533,27 +530,25 @@ The message in the server side is always retained."
 	     ((eq mailbox 'imap)
 	      (setq del (eq (mew-imap-delete case) t))));; delete may be number
 	    (if rev-del (setq del (not del))))
-	  (setq uid (mew-summary-message-uid))
-	  (when uid
-	    (setq msg (mew-summary-message-number))
-	    (setq siz (mew-summary-message-size))
-	    (setq rtr (list uid siz del (list bnm msg)))
-	    (setq rtrs (list rtr)))
-	  (if (not rtrs)
-	      (message "No message to be retrieved")
+	  (setq rtr (list uid siz del (list bnm msg)))
+	  (setq rtrs (list rtr))
+	  (cond
+	   ((not rtrs)
+	    (message "No message to be retrieved"))
+	   ((not (mew-msg-validp msg))
+	    (message "No valid message to be retrieved"))
+	   ((mew-folder-localp folder)
 	    (cond
-	     ((mew-folder-localp folder)
-	      (cond
-	       ((eq mailbox 'pop)
-		(mew-pop-retrieve case 'get bnm rtrs))
-	       ((eq mailbox 'imap)
-		(mew-imap-retrieve case 'get bnm rtrs))))
-	     ((mew-folder-popp folder)
+	     ((eq mailbox 'pop)
 	      (mew-pop-retrieve case 'get bnm rtrs))
-	     ((mew-folder-imapp folder)
-	      (mew-imap-retrieve case 'get bnm rtrs))
-	     ((mew-folder-nntpp folder)
-	      (mew-nntp-retrieve case 'get bnm rtrs))))))))))
+	     ((eq mailbox 'imap)
+	      (mew-imap-retrieve case 'get bnm rtrs))))
+	   ((mew-folder-popp folder)
+	    (mew-pop-retrieve case 'get bnm rtrs))
+	   ((mew-folder-imapp folder)
+	    (mew-imap-retrieve case 'get bnm rtrs))
+	   ((mew-folder-nntpp folder)
+	    (mew-nntp-retrieve case 'get bnm rtrs)))))))))
 
 (defun mew-summary-mark-retrieve-message (&optional rev-del)
   "Retrieve the rest of truncated('T') messages marked with '*'.
@@ -583,13 +578,14 @@ The messages in the server side is always retained."
        (if rev-del (setq del (not del))))
      (save-excursion
        (goto-char (point-min))
-       (while (re-search-forward mew-regex-msg-valid-review nil t)
-	 (setq uid (mew-summary-message-uid))
-	 (when uid
-	   (setq msg (mew-summary-message-number))
-	   (setq siz (mew-summary-message-size))
-	   (setq rtr (list uid siz del (list bnm msg)))
-	   (setq rtrs (cons rtr rtrs)))
+       (while (re-search-forward mew-regex-msg-review nil t)
+	 (when (mew-sumsyn-match mew-regex-sumsyn-long)
+	   (setq uid (mew-sumsyn-message-uid))
+	   (setq msg (mew-sumsyn-message-number))
+	   (setq siz (mew-sumsyn-message-size))
+	   (when (mew-msg-validp msg)
+	     (setq rtr (list uid siz del (list bnm msg)))
+	     (setq rtrs (cons rtr rtrs))))
 	 (forward-line)))
      (if (not rtrs)
 	 (message "No message to be retrieved")
