@@ -121,7 +121,7 @@
 	 uid uidl siz rtr rtrs rttl hlds)
     (goto-char (point-min))
     ;; num subj from date msg-id ref siz lines
-    (while (re-search-forward "^[0-9]+\t[^\t]*\t[^\t]*\t[^\t]*\t<\\([^\t]+\\)>\t[^\t]*\t\\([0-9]+\\)" nil t)
+    (while (re-search-forward "^\\([0-9]+\\)\t[^\t\n]*\t[^\t\n]*\t[^\t\n]*\t<[^>\t\n]+>\t[^\t\n]*\t\\([0-9]*\\)" nil t)
       (setq uid (mew-match-string 1))
       (setq siz (mew-match-string 2))
       (if (string= uid "") (setq uid nil))
@@ -188,19 +188,20 @@
       (mew-nntp-command-quit pro pnm))
      ((eq directive 'get)
       (mew-nntp-set-truncated pnm nil)
-      (mew-nntp-process-send-string pro "ARTICLE <%s>" uid))
+      (mew-nntp-process-send-string pro "ARTICLE %s" uid))
      ((and (eq directive 'scan) (not get-body))
       (mew-nntp-set-truncated pnm t)
-      (mew-nntp-process-send-string pro "HEAD <%s>" uid))
+      (mew-nntp-process-send-string pro "HEAD %s" uid))
      ((or (= lim 0) (<= (string-to-int siz) lim))
       (mew-nntp-set-truncated pnm nil)
-      (mew-nntp-process-send-string pro "ARTICLE <%s>" uid))
+      (mew-nntp-process-send-string pro "ARTICLE %s" uid))
      (t
       (mew-nntp-set-truncated pnm t)
-      (mew-nntp-process-send-string pro "HEAD <%s>" uid)))))
+      (mew-nntp-process-send-string pro "HEAD %s" uid)))))
 
 (defun mew-nntp-command-post-article (pro pnm)
-  (let* ((width (1- (mew-scan-width)))
+  (let* ((directive (mew-nntp-get-directive pnm))
+	 (width (1- (mew-scan-width)))
 	 (rtrs (mew-nntp-get-rtrs pnm))
 	 (rtr (car rtrs))
 	 (uid (nth 0 rtr))
@@ -223,9 +224,14 @@
     ;; line delimiters
     (mew-eol-fix-for-read)
     (mew-dot-delete)
-    (setq msg-file (mew-net-get-new-message
-		    pnm fld msg 'mew-nntp-get-msgdb 'mew-nntp-set-msgdb))
-    (setq msg (car msg-file) file (cdr msg-file))
+    (cond
+     ((eq directive 'scan)
+      (setq msg uid) 
+      (setq file (mew-expand-folder fld msg)))
+     (t
+      (setq msg-file (mew-net-get-new-message
+		      pnm fld msg 'mew-nntp-get-msgdb 'mew-nntp-set-msgdb))
+      (setq msg (car msg-file) file (cdr msg-file))))
     (goto-char (point-min))
     (if truncated
 	(mew-header-insert-xmu uid siz t)
@@ -240,13 +246,13 @@
 	 (mew-nntp-command-quit pro pnm)
 	 (throw 'write-error nil)))
       (mew-set-file-modes file)
-      ;; xxx links?
       (mew-set-buffer-multibyte t)
-      (setq vec (mew-nntp-scan-header))
+      (setq vec (mew-scan-header))
       (mew-scan-set-folder vec fld)
       (mew-scan-set-message vec msg)
+      (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg nil)
+      (mew-scan-insert-line fld vec width lmsg)
       (mew-nntp-command-next-article pro pnm))))
 
 (defun mew-nntp-command-next-article (pro pnm)
@@ -274,7 +280,7 @@
     (mew-eol-fix-for-read)
     (mew-dot-delete)
     (while (not (eobp))
-      (when (looking-at "\\([a-z][^ \t]+\\)")
+      (when (looking-at "\\([a-z][^ \t\n]+\\)")
 	(setq group2 (mew-match-string 1))
 	(setq group (concat mew-folder-nntp group2)) 
 	(setq groups (cons (mew-folder-func group) groups))
@@ -323,6 +329,7 @@
 
 (defun mew-nntp-process-send-string (pro &rest args)
   (let ((str (apply 'format args)))
+    (mew-nntp-debug "=SEND=" str)
     (process-send-string pro (concat str mew-cs-eol))))
 
 (defun mew-nntp-passtag (pnm)
@@ -331,12 +338,6 @@
 	(user (mew-nntp-get-user pnm)))
     (concat user "@" server ":" port)))
 
-(defun mew-nntp-scan-header ()
-  (goto-char (point-min))
-  (unless (re-search-forward mew-eoh nil t)
-    (goto-char (point-max)))
-  (mew-scan-header))
-
 (defsubst mew-nntp-message (pnm &rest args)
   (or (mew-nntp-get-no-msg pnm) (apply 'message args)))
 
@@ -413,7 +414,7 @@
 	(setq process (mew-nntp-open pnm server port no-msg))))
       (when process
 	(mew-summary-lock process "NNTPing" (or sshpro sslpro))
-	(mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-nntp-set-no-msg pnm no-msg) ;; must come here
@@ -542,11 +543,11 @@
 	   (mew-nntp-message pnm "No messages scanned"))
 	  ((= rttl 1)
 	   (mew-nntp-message pnm "1 message scanned")
-	   (mew-lisp-save file msgid)
+	   (mew-lisp-save file msgid nil 'unlimit)
 	   (mew-summary-folder-cache-save))
 	  (t
 	   (mew-nntp-message pnm "%d messages scanned" rttl)
-	   (mew-lisp-save file msgid)
+	   (mew-lisp-save file msgid nil 'unlimit)
 	   (mew-summary-folder-cache-save))))))
      ;;
      (mew-net-status-clear (mew-nntp-get-bnm pnm))
