@@ -82,7 +82,7 @@
 	  auth func)
       (save-excursion
 	(goto-char (point-min))
-	(if (re-search-forward "SASL \\([^\n\r]+\\)" nil t)
+	(if (re-search-forward "SASL \\([^\r\n]+\\)" nil t)
 	    (setq auth (mew-auth-select (mew-match-string 1) auth-list))))
       (if (and auth
 	       (setq func (mew-pop-auth-get-func auth))
@@ -400,7 +400,7 @@
 	 (del  (nth 3 rtr))
 	 (fld-msg (nth 4 rtr))
 	 (truncated (mew-pop-get-truncated pnm))
-	 fld msg vec file msg-file lmsg)
+	 fld msg vec file msg-file lmsg mark)
     (cond
      ((null fld-msg)
       (setq fld (mew-pop-get-bnm pnm)))
@@ -441,11 +441,13 @@
       (mew-pop-set-rcnt pnm (1+ (mew-pop-get-rcnt pnm)))
       ;;
       (mew-set-buffer-multibyte t)
-      (setq vec (mew-pop-scan-header))
+      (setq vec (mew-scan-header))
       (mew-scan-set-folder vec fld)
       (mew-scan-set-message vec msg)
+      (if (eq directive 'inc) (setq mark (mew-scan-inbox-action vec)))
+      (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
-      (mew-scan-insert-line fld vec width lmsg nil)
+      (mew-scan-insert-line fld vec width lmsg mark)
       (mew-pop-set-rtrs pnm (cdr rtrs))
       (if (and del (not truncated))
 	  (progn
@@ -525,14 +527,9 @@
 
 (defun mew-pop-process-send-string (pro &rest args)
   (let ((str (apply 'format args)))
+    (mew-pop-debug "=SEND=" str)
     (process-send-string pro (concat str mew-cs-eol))))
 
-(defun mew-pop-scan-header ()
-  (goto-char (point-min))
-  (unless (re-search-forward mew-eoh nil t)
-    (goto-char (point-max)))
-  (mew-scan-header))
-
 (defsubst mew-pop-message (pnm &rest args)
   (or (mew-pop-get-no-msg pnm) (apply 'message args)))
 
@@ -635,7 +632,7 @@
 	  (if (eq directive 'exec)
 	      (mew-summary-visible-buffer bnm))
 	(mew-summary-lock process "POPing" (or sshpro sslpro))
-	(mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
+	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-pop-set-no-msg pnm no-msg) ;; must come here
@@ -784,11 +781,11 @@
 	   (mew-pop-message pnm "No messages scanned"))
 	  ((= rttl 1)
 	   (mew-pop-message pnm "1 message scanned")
-	   (mew-lisp-save file uidl)
+	   (mew-lisp-save file uidl nil 'unlimit)
 	   (mew-summary-folder-cache-save))
 	  (t
 	   (mew-pop-message pnm "%d messages scanned" rttl)
-	   (mew-lisp-save file uidl)
+	   (mew-lisp-save file uidl nil 'unlimit)
 	   (mew-summary-folder-cache-save))))
 	((eq directive 'exec)
 	 (when kils
