@@ -34,6 +34,7 @@
   ;; load from alist
   (or mew-refile-from-alist
       (setq mew-refile-from-alist (mew-lisp-load mew-refile-from-file)))
+  (mew-assoc-folder-setup)
   (add-hook 'kill-emacs-hook 'mew-refile-clean-up))
 
 (defun mew-refile-clean-up ()
@@ -125,6 +126,49 @@ mew-refile-guess-from-me-is-special."
 ;;; Guess by folder
 ;;;
 
+(defvar mew-assoc-folder nil)
+
+(defun mew-folder-func (full &optional name)
+  (cons full name))
+
+(defun mew-assoc-folder-setup ()
+  (if mew-use-fast-refile
+      (setq mew-assoc-folder 'mew-assoc-folder-fast)
+    (setq mew-assoc-folder 'mew-assoc-folder-slow)))
+
+(defun mew-assoc-folder-slow (key alist localp)
+  ;; Case-sensitive so you can use capital letters for your folders.
+  ;; But this is slow.
+  (let ((skey (downcase key))
+ 	(regex (concat mew-path-separator "$"))
+	ent name ret)
+    (catch 'loop
+      (while alist
+ 	(setq ent (car alist))
+ 	(setq name (cdr ent))
+ 	(when (and (stringp name) (string= (downcase name) skey))
+ 	  (setq ret ent)
+ 	  (unless (and localp (string-match regex (car ent)))
+	    (throw 'loop nil)))
+ 	(setq alist (cdr alist))))
+    ret))
+
+(defun mew-assoc-folder-fast (key alist localp)
+  ;; Case-insensitive so you cannot use capital letters for your folders.
+  ;; But this is fast.
+  (let ((skey (downcase key)) ret ret2 regex)
+    (if (not localp)
+	(rassoc skey alist)
+      ;; ("+foo/" . "foo")
+      ;; ("+foo/foo" . "foo")
+      (setq regex (concat mew-path-separator "$"))
+      (setq ret (rassoc skey alist))
+      (setq ret2 ret)
+      (while (and ret (string-match regex (car ret)))
+	(setq alist (cdr (member ret alist)))
+	(setq ret (rassoc skey alist)))
+      (or ret ret2))))
+
 (defun mew-refile-guess-by-folder ()
   ;; Guess folders by the To:/Cc: field with folder alist.
   ;; Mainly used for mailing-list.
@@ -138,8 +182,8 @@ mew-refile-guess-from-me-is-special."
       (setq ml-addr (mew-addrstr-parse-address (or (car to-cc) "")))
       (when ml-addr
 	(setq ml-name (mew-addrstr-extract-user ml-addr))
-	(setq ent (or (mew-assoc-folder ml-addr alist localp)
-		      (mew-assoc-folder ml-name alist localp)))
+	(setq ent (or (funcall mew-assoc-folder ml-addr alist localp)
+		      (funcall mew-assoc-folder ml-name alist localp)))
 	(mew-addq ret (nth 0 ent)))
       (setq to-cc (cdr to-cc)))
     (nreverse ret)))
@@ -164,7 +208,7 @@ RULE is as follows:
 There are two special <key>s: @samp{nil} and @samp{t}. @samp{nil} is
 used to specify <folder> to be returned when nothing is
 guessed. @samp{t} can specify <folder> to be returned in addition to
-guessed values. "
+guessed values."
   (let (key alist val f-or-r header ent ret)
     (while rule
       (setq key (car (car rule)))
@@ -221,12 +265,11 @@ guessed values. "
 (defun mew-refile-guess-by-thread ()
   ;; Guess folders by thread with alist created by
   ;; mew-refile-guess-by-thread-learn.
-  (let ((msgid (or (mew-header-get-value mew-references:)
-		   (mew-header-get-value mew-in-reply-to:))))
-    ;; search for msgid
-    (if (and msgid 
-	     (string-match "\\(<[^ \t\n]*>\\)[^>]*\0" (concat msgid "\0")))
-	(nth 1 (assoc (match-string 1 msgid) mew-refile-msgid-alist)))))
+  (let ((msgid (or (mew-idstr-get-last-id
+		    (mew-header-get-value mew-references:))
+		   (mew-idstr-get-first-id
+		    (mew-header-get-value mew-in-reply-to:)))))
+    (if msgid (nth 1 (assoc msgid mew-refile-msgid-alist)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -283,7 +326,7 @@ guessed values. "
     (while newsgroups
       (setq newsgroup (car newsgroups))
       (setq newsgroups (cdr newsgroups))
-      (setq ent (mew-assoc-case-equal newsgroup alist 1))
+      (setq ent (funcall mew-assoc-folder newsgroup alist nil))
       (mew-addq ret (nth 0 ent)))
     (nreverse ret)))
 
@@ -334,13 +377,12 @@ guessed values. "
 
 (defun mew-refile-guess-by-thread-learn (chosen info)
   ;; Create mew-refile-msgid-alist for mew-refile-guess-by-thread.
-  (let* ((msgid (mew-header-get-value mew-message-id:)) 
+  (let* ((msgid (mew-idstr-get-first-id
+		 (mew-header-get-value mew-message-id:)))
 	 (folder (car chosen))
 	 ;; ohter people's honest opinion and my honest opinion.
 	 (oho info)
 	 (mho (cdr (assoc 'mew-refile-guess-by-thread info))))
-    (if (and msgid (string-match "<[^ \n>]*>" msgid))
-	(setq msgid (match-string 0 msgid)))
     (when (and msgid chosen)
       ;; if my opninion was right, I learn it.
       ;; or a folder was not in other people's opinion,
@@ -481,15 +523,14 @@ Return t if exists or created. Otherwise, return nil."
       (setq cands (delete nil cands))
       (when exfld
 	;; copying, two folders are necessary
+	(setq singlep nil)
 	(if (or (null cands) (equal (list exfld) cands))
 	    (setq cands (list exfld proto))
 	  (setq cands (cons exfld cands))))
       (setq cands (mew-uniq-list cands)) ;; unavoidable
-      (if (and singlep (= (length cands) 1))
-	  (setq cands (car cands)))
       (unless cands
 	(setq cands (list proto)))
-      (setq folders (mew-input-folders cands proto)))
+      (setq folders (mew-input-refile-folders cands singlep case proto)))
     ;; check folder existence.
     (setq lst-lst (mapcar (lambda (x) (mew-split x ?,)) folders))
     (while lst-lst
@@ -522,6 +563,9 @@ the current folder as a candidate in addition to guessed folders."
 ;;; Refile (aka move)
 ;;;
 
+(defsubst mew-refile-log-buffer (fld)
+  (concat mew-buffer-prefix "refile " fld))
+
 (defun mew-summary-refile (&optional report)
  "Put the refile mark(default is 'o') on this message. If already
 marked with 'o', it prints where this message will be refiled. This
@@ -534,24 +578,62 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
    (mew-summary-local-or-imap
     (if report (mew-summary-refile-report) (mew-summary-refile-body))))))
 
+(defvar mew-override-body-open "<")
+(defvar mew-override-body-close "> ")
+(defconst mew-regex-override-body
+  (concat (regexp-quote mew-override-body-open)
+	  "[^>\r\n]*"
+	  (regexp-quote mew-override-body-close)))
+
+(defun mew-summary-refile-override-body (folders-str)
+  (save-excursion
+    (when (mew-summary-goto-body 'after)
+      (when (looking-at mew-regex-override-body)
+	(mew-elet
+	 (delete-region (point) (match-end 0))))
+      (mew-elet
+       (insert-and-inherit mew-override-body-open folders-str mew-override-body-close)))))
+
+(defun mew-summary-refile-remove-body ()
+  (save-excursion
+    (mew-summary-goto-body 'after)
+    (when (looking-at mew-regex-override-body)
+      (mew-elet
+       (delete-region (point) (match-end 0))))))
+
+(defun mew-summary-refile-log (fld folders-str)
+  (save-excursion
+    (let ((cbuf (current-buffer))
+	  (msg-id (mew-summary-my-id))
+	  (beg (point)) end)
+      (forward-line)
+      (when (search-backward "\r" nil t)
+	(setq end (point))
+	(set-buffer (get-buffer-create (mew-refile-log-buffer fld)))
+	(goto-char (point-max))
+	(mew-insert-buffer-substring cbuf beg end)
+	(insert "== src=" fld " dst=" folders-str " id=<" msg-id ">")
+	(insert " date=" (mew-time-ctz-to-logtime (current-time)) "\n")))))
+
 (defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark exfld)
-  (let (msg folder folders cur-folders mark
-	    buf learn-info fld tmp delbuf invalidp)
-    (save-excursion
-      ;; save the cursor position anyway
-      (mew-summary-goto-message)
-      (setq invalidp (mew-summary-msg-invalidp))
-      ;; on the message
-      (setq fld (mew-summary-folder-name))
-      (setq msg (mew-summary-message-number)) ;; msg is never nil
-      (setq mark (mew-summary-get-mark))) ;; any mark
+  (let ((pos (point))
+	msg folder folders cur-folders mark
+	buf learn-info fld tmp delbuf invalidp folders-str)
+    (mew-summary-goto-message)
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (setq fld (mew-sumsyn-folder-name))
+      (setq msg (mew-sumsyn-message-number)) ;; msg is never nil
+      (setq invalidp (not (mew-msg-validp msg))))
+    (setq mark (mew-summary-get-mark)) ;; any mark
     (cond
      ((and mark (> (mew-markdb-level mark) (mew-markdb-level mew-mark-refile)))
       (or no-msg
 	  (message "Cannot mark here because '%s' is stronger than '%s'"
-		   (mew-markdb-name mark) (mew-markdb-name mew-mark-refile))))
+		   (mew-markdb-name mark) (mew-markdb-name mew-mark-refile)))
+      nil)
      (invalidp
-      (or no-msg (message "Cannot mark this invalid message")))
+      (or no-msg (message "Cannot refile this invalid message"))
+      nil)
      (t
       (if exp-flds
 	  (setq folders exp-flds)
@@ -594,23 +676,33 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
       ;; mark refile
       (unless no-mark
 	(when folders
-	  (save-excursion
-	    (mew-summary-goto-message)
-	    (or exp-flds auto (mew-refile-guess-learn buf learn-info))
-	    (when (get-buffer fld)
-	      (save-excursion
-		(set-buffer fld)
+	  (or exp-flds auto (mew-refile-guess-learn buf learn-info))
+	  (setq folders-str (mew-join "," folders))
+	  (cond
+	   ((mew-virtual-p)
+	    (save-excursion
+	      (set-buffer fld)
+	      (goto-char (point-min))
+	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 		(mew-refile-reset msg)
-		(mew-refile-set msg folders)))
-	    (mew-mark-unmark))
-	  (mew-mark-put-mark mew-mark-refile no-msg) ;; cursor may move
-	  (if (mew-virtual-p)
-	      (mew-summary-mark-in-physical fld msg mew-mark-refile))))
+		(mew-refile-set msg folders)
+		(mew-summary-refile-log fld folders-str)
+		(mew-summary-refile-override-body folders-str)
+		(mew-mark-put mew-mark-refile)
+		(set-buffer-modified-p nil))))
+	   (t
+	    (mew-refile-reset msg)
+	    (mew-refile-set msg folders)
+	    (mew-summary-refile-log fld folders-str)))
+	  (mew-summary-refile-override-body folders-str)
+	  (mew-mark-put-mark mew-mark-refile no-msg)
+	  (setq pos (point))))
       (if delbuf (mew-remove-buffer buf))
       ;; memorize last-folder
       (setq mew-refile-last-folder folders)
-      (set-buffer-modified-p nil)
-      folders)))) ;; return value
+      (set-buffer-modified-p nil)))
+    (goto-char pos)
+    folders)) ;; return value
 
 (defun mew-summary-refile-report ()
   (let ((proto (mew-proto-to-refile (mew-summary-folder-name 'ext)))
@@ -618,21 +710,17 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	(fld (mew-summary-folder-name))
 	(msg (mew-summary-message-number2))
 	(win (selected-window))
-	(tmpbuf (generate-new-buffer mew-buffer-prefix))
 	(customize-var '(mew-refile-ctrl-multi
 			 mew-refile-guess-key-list
 			 mew-refile-guess-strip-domainpart
 			 mew-refile-guess-from-me-is-special))
 	guess)
-    (save-excursion
-      (set-buffer tmpbuf)
-      (mew-erase-buffer)
+    (with-temp-buffer
       (mew-insert-message fld msg mew-cs-autoconv
 			  mew-header-reasonable-size)
       (mew-sinfo-set-case case)
       (mew-sinfo-set-proto proto)
       (setq guess (mew-refile-guess nil t)))
-    (mew-remove-buffer tmpbuf)
     (mew-window-configure 'message)
     ;; message buffer
     (mew-elet
@@ -669,15 +757,12 @@ refile folder."
   (interactive)
   (mew-summary-not-in-draft
    (mew-summary-local-or-imap
-    (let (validp)
-      (save-excursion
-	(mew-summary-goto-message)
-	(setq validp (not (mew-summary-msg-invalidp))))
-      (when validp
-	(message "%s to %s"
-		 (mew-summary-message-number)
-		 (mew-join ", " mew-refile-last-folder))
-	(mew-summary-refile-body mew-refile-last-folder))))))
+    (save-excursion
+      (mew-summary-goto-message)
+      (let ((msg (mew-summary-message-number)))
+	(when (mew-msg-validp msg)
+	  (message "%s to %s" msg (mew-join ", " mew-refile-last-folder)))))
+    (mew-summary-refile-body mew-refile-last-folder))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -696,7 +781,7 @@ If '\\[universal-argument]' is specified, only messages marked with
     (mew-summary-local-or-imap
      (mew-decode-syntax-delete)
      (let ((mew-use-highlight-x-face nil)
-	   (lines (mew-count-lines (point-min) (point-max)))
+	   (lines (mew-sinfo-get-ttl-line))
 	   (line 1) (mark nil) msg)
        (cond
 	(mew-mark-review-only
@@ -711,23 +796,22 @@ If '\\[universal-argument]' is specified, only messages marked with
 	 (message "Auto refiling...")
 	 (save-window-excursion
 	   (goto-char (point-min))
-	   (while (not (eobp))
-	     (when (looking-at mew-regex-msg-valid)
-	       (setq mark (mew-summary-get-mark))
-	       (if mew-mark-review-only
-		   (and mark
-			(char-equal mark mew-mark-review)
-			(mew-summary-refile-body nil t 'no-msg))
-		 (or (and mark
-			  (or mew-refile-auto-refile-skip-any-mark
-			      (>= (mew-markdb-level mark)
-				  (mew-markdb-level mew-mark-refile))))
-		     (mew-summary-refile-body nil t 'no-msg)))
-	       (if (= (% (/ (* 100 line) lines) 10) 0)
-		   (message "Auto refiling...%s%%" (/ (* 100 line) lines)))
-	       (setq line (1+ line)))
-	     (forward-line))
-	   (message "Auto refiling...done"))))))))
+	   (while (re-search-forward mew-regex-sumsyn-valid nil t)
+	     (setq mark (mew-summary-get-mark))
+	     (if mew-mark-review-only
+		 (and mark
+		      (char-equal mark mew-mark-review)
+		      (mew-summary-refile-body nil t 'no-msg))
+	       (or (and mark
+			(or mew-refile-auto-refile-skip-any-mark
+			    (>= (mew-markdb-level mark)
+				(mew-markdb-level mew-mark-refile))))
+		   (mew-summary-refile-body nil t 'no-msg)))
+	     (if (= (% (/ (* 100 line) lines) 10) 0)
+		 (message "Auto refiling...%s%%" (/ (* 100 line) lines)))
+	     (setq line (1+ line))
+	     (forward-line)))
+	 (message "Auto refiling...done")))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -748,9 +832,9 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (save-excursion
 	(save-window-excursion
 	  (goto-char (point-min))
-	  (while (re-search-forward mew-regex-msg-valid-review nil t)
+	  (while (re-search-forward mew-regex-msg-review nil t)
 	    (setq last (mew-summary-refile-body
-			last nil 'no-msg nil (mew-summary-folder-name)))
+			last nil 'no-msg nil (mew-summary-folder-name 'ext)))
 	    (forward-line))
 	  (message "Mark copying...done")))))))
 
@@ -767,7 +851,7 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (save-excursion
 	(save-window-excursion
 	  (goto-char (point-min))
-	  (while (re-search-forward mew-regex-msg-valid-review nil t)
+	  (while (re-search-forward mew-regex-msg-review nil t)
 	    (setq last (mew-summary-refile-body last nil 'no-msg))
 	    (forward-line))
 	  (message "Mark refiling...done")))))))
@@ -776,7 +860,7 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2002 Mew developing team.
+;; Copyright (C) 1994-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
