@@ -78,7 +78,7 @@
    ((mew-smtp-get-user pnm)
     (let ((auth-list (mew-smtp-get-auth-list pnm))
 	  (str (mew-smtp-get-string pnm)) auth func)
-      (if (and (string-match "AUTH \\([^\n\r]+\\)\r?\n" str)
+      (if (and (string-match "AUTH \\([^\r\n]+\\)\r?\n" str)
 	       (setq auth (mew-auth-select (match-string 1 str) auth-list))
 	       (setq func (mew-smtp-auth-get-func auth))
 	       (fboundp func))
@@ -316,12 +316,12 @@
        (setq errmsg "Cannot connect to the SMTP server")
        (mew-smtp-set-error pnm errmsg)
        (setq pro nil)
-       (message errmsg))
+       (message "%s" errmsg))
       (error
        (setq errmsg (format "%s, %s" (nth 1 emsg) (nth 2 emsg)))
        (mew-smtp-set-error pnm errmsg)
        (setq pro nil)
-       (message errmsg)))
+       (message "%s" errmsg)))
     (if tm (cancel-timer tm))
     pro))
 
@@ -484,13 +484,13 @@
     (if (mew-draft-p)
 	(mew-smtp-set-case pnm (mew-tinfo-get-case)))
     ;;
+    (mew-smtp-set-recipients pnm (mew-smtp-get-orig-recipients pnm))
     (let* ((n mew-smtp-info-list-save-length)
 	   (data (make-vector n nil))
 	   (i 0))
       (while (< i n)
 	(aset data i (aref (mew-info pnm) i))
 	(setq i (1+ i)))
-      (mew-smtp-set-recipients pnm (mew-smtp-get-orig-recipients pnm))
       (mew-lisp-save info data))
     ;;
     (mew-remove-buffer (current-buffer))
@@ -523,7 +523,7 @@
      (mew-draft-header-insert-alist (mew-header-alist case))
      ;; X-Mailer: must be the last
      (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
-     (mew-header-set (concat mew-header-separator "\n"))
+     (mew-header-set "\n")
      (insert mew-bcc-body)
      (goto-char (mew-header-end))
      (forward-line)
