@@ -78,18 +78,14 @@
 
 (defun mew-sort-index (x) (car x))
 
-(defun mew-sort-adjust-number (form num line)
-  (if (string-match mew-regex-msg line)
-      (let ((n (match-end 0))
-	    (props (text-properties-at 0 line)))
-	(setq num (format form num))
-	(when props
-	  (add-text-properties 0 (length num) props num))
-	(concat num (substring line n)))
-    line)) ;; error
+(defun mew-sort-insert (line msg)
+  (insert line)
+  (save-excursion
+    (forward-line -1)
+    (mew-syntax-change-message-number msg)))
 
 ;; If not found, returns nil.
-(defun mew-summary-sort-rename (src dst pos form &optional lastp)
+(defun mew-summary-sort-rename (src dst pos &optional lastp)
   (if (mew-debug 'sort)
       (mew-elet
        (insert (format "move %s to %s\n" src dst)))
@@ -101,43 +97,52 @@
 	(lastp
 	 (when pos
 	   (goto-char pos)
-	   (insert (mew-sort-adjust-number form dst mew-sort-line))))
+	   (mew-sort-insert mew-sort-line dst)
+	   nil))
 	((null pos) ;; first
 	 (goto-char (point-min))
-	 (when (re-search-forward (mew-regex-jmp-msg src) nil t)
+	 (when (re-search-forward (mew-regex-sumsyn-msg src) nil t)
 	   (beginning-of-line)
 	   (setq beg (point))
 	   (forward-line)
 	   (setq end (point))
+	   ;; We need to keep properties in Summary mode.
+	   ;; This must be "buffer-substring".
 	   (setq mew-sort-line (buffer-substring beg end))
 	   (delete-region beg end)
 	   beg))
 	(t
 	 (goto-char (point-min))
-	 (when (re-search-forward (mew-regex-jmp-msg src) nil t)
+	 (when (re-search-forward (mew-regex-sumsyn-msg src) nil t)
 	   (beginning-of-line)
 	   (setq beg (point))
 	   (forward-line)
 	   (setq end (point))
 	   (cond
 	    ((< pos beg)
+	     ;; We need to keep properties in Summary mode.
+	     ;; This must be "buffer-substring".
 	     (setq line (buffer-substring beg end))
 	     (goto-char end)
 	     (delete-region beg end)
 	     (save-excursion
 	       (goto-char pos)
-	       (insert (mew-sort-adjust-number form dst line)))
+	       (mew-sort-insert line dst))
 	     (point))
 	    ((= pos beg)
+	     ;; We need to keep properties in Summary mode.
+	     ;; This must be "buffer-substring".
 	     (setq line (buffer-substring beg end))
 	     (delete-region beg end)
 	     (goto-char pos)                    
-	     (insert (mew-sort-adjust-number form dst line))
+	     (mew-sort-insert line dst)
 	     (point))
 	    (t
+	     ;; We need to keep properties in Summary mode.
+	     ;; This must be "buffer-substring".
 	     (setq line (buffer-substring beg end))
 	     (goto-char pos)
-	     (insert (mew-sort-adjust-number form dst line))
+	     (mew-sort-insert line dst)
 	     (delete-region beg end)
 	     beg)))))))))
 
@@ -153,20 +158,20 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
   (mew-summary-only
    (mew-summary-not-in-queue
     (mew-summary-not-in-draft
-     (mew-summary-with-mewls
+     (mew-summary-with-mewl
       (when (mew-summary-exclusive-p)
-	(mew-summary-sort-body arg)))))))
+	(let ((folder (mew-summary-folder-name)))
+	  (if (null folder)
+	      (message "No message")
+	    (mew-summary-sort-body folder arg)))))))))
 
-(defun mew-summary-sort-body (arg)
-  (let ((folder (mew-summary-folder-name))
-	(win (selected-window))
+(defun mew-summary-sort-body (folder arg)
+  (let ((win (selected-window))
 	key type idx files range beg end)
     ;;
     ;; Summary cache updates
     ;;
-    (mew-current-set nil nil nil)
-    (mew-decode-syntax-delete)
-    (mew-unhighlight-cursor-line)
+    (mew-summary-reset)
     (mew-summary-retrieve-gap folder)
     ;;
     ;; Determining range
@@ -203,19 +208,16 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
     (condition-case nil
 	(progn
 	  ;;
-	  ;; Calling mewls
+	  ;; Calling mewl
 	  ;;
-	  (let* ((tmpbuf (generate-new-buffer mew-buffer-prefix))
-		 (funcs (assoc type mew-sort-switch))
+	  (let* ((funcs (assoc type mew-sort-switch))
 		 (func1 (nth 1 funcs))
 		 (func2 (nth 2 funcs))
 		 (i 0)
 		 (fld (mew-expand-folder2 folder))
 		 num med value ent)
-	    (save-excursion
-	      (set-buffer tmpbuf)
-	      (mew-erase-buffer)
-	      (call-process mew-prog-mewls nil t nil
+	    (with-temp-buffer
+	      (call-process mew-prog-mewl nil t nil
 			    "-b" mew-mail-path "-l" "0"
 			    "-d" key "-s" (concat fld " " range))
 	      (goto-char (point-min))
@@ -231,20 +233,16 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 		  (setq files (cons num files))
 		  (setq ent (cons (cons i (funcall func1 value folder num)) ent))
 		  (setq i (1+ i)))))
-	    (mew-remove-buffer tmpbuf)
 	    (setq files (vconcat (nreverse files)))
 	    (setq ent (sort ent func2))
 	    (setq idx (vconcat (mapcar 'mew-sort-index ent))))
 	  ;;
-	  (cond
-	   ((mew-debug 'sort)
+	  (when (mew-debug 'sort)
 	    (mew-window-configure 'message)
 	    ;; message buffer
 	    (mew-elet
 	     (mew-erase-buffer)
 	     (insert "Sort as follows:\n")))
-	   (t
-	    (mew-window-configure 'summary)))
 	  ;; for C-xC-x
 	  (when (and (not (mew-debug 'sort)) (not (eobp)))
 	    (beginning-of-line)
@@ -275,7 +273,6 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 	  (let* ((dir (mew-expand-folder folder))
 		 (default-directory dir)
 		 (len (length idx));; not (length files)
-		 (form (format "%%%ds" (mew-summary-scan-form-num folder)))
 		 (tmp (mew-folder-new-message folder 'num-only))
 		 (i 0)
 		 src dst pos)
@@ -284,17 +281,15 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 	      (unless (= i (aref idx i))
 		(setq dst len)
 		(setq src i)
-		(setq pos (mew-summary-sort-rename
-			   (aref files src) tmp nil form))
+		(setq pos (mew-summary-sort-rename (aref files src) tmp nil))
 		(catch 'loop
 		  (while t
 		    (setq dst src)
 		    (setq src (aref idx dst))
 		    (if (= src i) (throw 'loop nil))
-		    (setq pos (mew-summary-sort-rename
-			       (aref files src) (aref files dst) pos form))
+		    (setq pos (mew-summary-sort-rename (aref files src) (aref files dst) pos))
 		    (aset idx dst dst)))
-		(mew-summary-sort-rename tmp (aref files dst) pos form 'last)
+		(mew-summary-sort-rename tmp (aref files dst) pos 'last)
 		(aset idx dst dst))
 	      (setq i (1+ i))))
 	  ;; The cursor moves onto point-min of the region.
@@ -344,21 +339,13 @@ sorted. But if you do so, \"\\<mew-summary-mode-map>\\[mew-summary-ls]\" + 'sync
 ;;; Packing
 ;;;
 
-(defun mew-summary-pack-rename (src dst form)
-  (if (mew-debug 'pack)
-      (mew-elet
-       (insert (format "move %s to %s\n" src dst)))
-    (mew-elet
-     (let (beg end line)
-       (rename-file src dst)
-       (mew-refile-change src dst)
-       (when (re-search-forward (mew-regex-jmp-msg src) nil t)
-	 (setq beg (match-beginning 0))
-	 (setq end (match-end 0))
-	 (setq line (buffer-substring beg end))
-	 (delete-region beg end)
-	 (insert (mew-sort-adjust-number form dst line))
-	 (forward-line))))))
+(defun mew-summary-pack-rename (src dst)
+  (mew-elet
+   (rename-file src dst)
+   (mew-refile-change src dst)
+   (when (re-search-forward (mew-regex-sumsyn-msg src) nil t)
+     (mew-syntax-change-message-number2 dst)
+     (forward-line))))
 
 (defun mew-summary-pack ()
   "Pack messages and list them up again.
@@ -367,25 +354,24 @@ If this command is used in a remote folder,
 local cache messages are packed."
   (interactive)
   (mew-summary-only
-   (mew-summary-not-in-queue
-    (mew-summary-not-in-draft
-     (when (mew-summary-exclusive-p)
-       (if (or (not mew-ask-pack)
-	       (y-or-n-p (format "Pack %s? " (mew-summary-folder-name))))
-	   (mew-summary-pack-body)))))))
+   (mew-summary-local-only
+    (mew-summary-not-in-queue
+     (mew-summary-not-in-draft
+      (when (mew-summary-exclusive-p)
+	(let ((folder (mew-summary-folder-name)))
+	  (cond
+	   ((null folder)
+	    (message "No message"))
+	   ((or (not mew-ask-pack) (y-or-n-p (format "Pack %s? " folder)))
+	    (mew-summary-pack-body folder))))))))))
 
-(defun mew-summary-pack-body ()
-  (let* ((win (selected-window))
-	 (folder (mew-summary-folder-name))
-	 (dir (mew-expand-folder folder))
+(defun mew-summary-pack-body (folder)
+  (let* ((dir (mew-expand-folder folder))
 	 (default-directory dir)
-	 (form (format "%%%ds" (mew-summary-scan-form-num folder)))
 	 (n 1)
 	 msgs msg src dst)
     ;;
-    (mew-current-set nil nil nil)
-    (mew-decode-syntax-delete)
-    (mew-unhighlight-cursor-line)
+    (mew-summary-reset)
     (mew-summary-retrieve-gap folder)
     ;;
     (message "Packing %s..." folder)
@@ -395,18 +381,9 @@ local cache messages are packed."
 	  (setq msgs (mew-dir-messages "."))
 	  (setq msgs (mapcar 'string-to-int msgs))
 	  (setq msgs (sort msgs '<)) ;; sort is inevitable
-	  (cond
-	   ((mew-debug 'pack)
-	    (mew-window-configure 'message)
-	    ;; message buffer
-	    (mew-elet
-	     (mew-erase-buffer)
-	     (insert "Pack as follows:\n")))
-	   (t
-	    (mew-window-configure 'summary)))
 	  ;; the cursor stays the current position.
 	  (save-excursion
-	    (unless (mew-debug 'pack) (goto-char (point-min)))
+	    (goto-char (point-min))
 	    (while msgs
 	      (setq msg (car msgs))
 	      (setq msgs (cdr msgs))
@@ -421,23 +398,14 @@ local cache messages are packed."
 		(while (file-exists-p dst)
 		  (setq n (1+ n))
 		  (setq dst (int-to-string n)))
-		(mew-summary-pack-rename src dst form)
+		(mew-summary-pack-rename src dst)
 		(setq n (1+ n))))))
-	  (cond
-	   ((mew-debug 'pack)
-	    (mew-message-clear-end-of)
-	    (set-buffer-modified-p nil)
-	    (goto-char (point-min))
-	    (select-window win))
-	   (t
-	    (mew-summary-folder-cache-save)
-	    (set-buffer-modified-p nil)))
+	  (mew-summary-folder-cache-save)
+	  (set-buffer-modified-p nil)
 	  (mew-summary-unlock)
-	  (unless (mew-debug 'pack)
-	    (run-hooks 'mew-pack-hook))
-	  (message "Packing %s...done" folder))
+	  (run-hooks 'mew-pack-hook)
+	  (message "Packing %s...done (the last is %d)" folder (1- n)))
       (quit
-       (select-window win)
        (set-buffer-modified-p nil)
        (mew-summary-unlock)))))
 
@@ -445,7 +413,7 @@ local cache messages are packed."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
