@@ -65,23 +65,6 @@
   (if (string-match (format "^%s:[^:]+:" mew-ssh-info-prefix) name)
       (match-string 0 name)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Port DB
-;;;
-
-(defvar mew-port-db
-  '(("smtp" .  "25")
-    ("pop3" . "110")
-    ("nntp" . "119")
-    ("imap" . "143")))
-
-(defun mew-serv-to-port (serv)
-  (if (string-match "^[0-9]+$" serv)
-      serv
-    (or (cdr (assoc serv mew-port-db)) "0")))
-
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Executing SSH
@@ -89,9 +72,9 @@
 
 (defun mew-open-ssh-stream (server serv sshserver)
   "Open SSH stream for SERVER's SERV via SSHSERVER.
-This function returns a process when a SSH connection is created
+This function returns a process when an SSH connection is created
 successfully. A local port number can be obtained the process name
-after ':'. "
+after ':'."
   (cond
    ((not (and (stringp server) (stringp serv) (stringp sshserver)))
     ())
@@ -104,7 +87,7 @@ after ':'. "
 	   (i 0) (N mew-ssh-process-exec-cnt)
 	   (pros (process-list))
 	   (regex (mew-ssh-info-name-regex sshserver server remoteport))
-	   name pnm pro status)
+	   buf name pnm pro status)
       (catch 'find
 	(while pros
 	  (when (string-match regex (process-name (car pros)))
@@ -117,12 +100,13 @@ after ':'. "
 	  pro
 	(message "Connecting to the SSH server...")
 	(setq pro nil)
+	(setq buf (generate-new-buffer (concat mew-buffer-prefix "ssh")))
 	(catch 'loop
 	  (while (< i N)
 	    (setq name (mew-ssh-info-name sshserver server remoteport localport))
 	    (setq pro
 		  (apply 'start-process
-			 name nil
+			 name buf
 			 mew-prog-ssh "-x" "-v"
 			 "-o" (concat 
 			       (if (= mew-ssh-ver 2)
@@ -208,8 +192,9 @@ after ':'. "
 
 (defun mew-ssh-sentinel (process event)
   (let ((pnm (process-name process)))
-    (if (null (mew-ssh-get-status pnm))
-	(mew-ssh-set-status pnm 'exit))))
+    (unless (mew-ssh-get-status pnm)
+      (mew-ssh-set-status pnm 'exit))
+    (mew-remove-buffer (process-buffer process))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -219,30 +204,26 @@ after ':'. "
 (defun mew-ssh-setup ()
   (if (not (mew-which-exec mew-prog-ssh))
       (setq mew-ssh-ver nil)
-    (let ((buf (generate-new-buffer mew-buffer-prefix)))
-      (save-excursion
-	(set-buffer buf)
-	(mew-erase-buffer)
-	(call-process mew-prog-ssh nil t nil "-V")
-	(goto-char (point-min))
-	(cond
-	 ((looking-at "SSH Version 1")
-	  (setq mew-ssh-ver 0))
-	 ((looking-at (concat mew-prog-ssh ": SSH Version 2"))
-	  (setq mew-ssh-ver 1))
-	 ((looking-at (concat mew-prog-ssh ": SSH Secure Shell [23]"))
-	  (setq mew-ssh-ver 1))
-	 ((looking-at "\\(SSH Version \\)?\\(OpenSSH\\|NetBSD\\|Sun_SSH\\)")
-	  (setq mew-ssh-ver 2))
-	 (t
-	  (setq mew-ssh-ver nil))))
-      (mew-remove-buffer buf))))
+    (with-temp-buffer
+      (call-process mew-prog-ssh nil t nil "-V")
+      (goto-char (point-min))
+      (cond
+       ((looking-at "SSH Version 1")
+	(setq mew-ssh-ver 0))
+       ((looking-at (concat mew-prog-ssh ": SSH Version 2"))
+	(setq mew-ssh-ver 1))
+       ((looking-at (concat mew-prog-ssh ": SSH Secure Shell [23]"))
+	(setq mew-ssh-ver 1))
+       ((looking-at "\\(SSH Version \\)?\\(OpenSSH\\|NetBSD\\|Sun_SSH\\)")
+	(setq mew-ssh-ver 2))
+       (t
+	(setq mew-ssh-ver nil))))))
 
 (provide 'mew-ssh)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2002 Mew developing team.
+;; Copyright (C) 1999-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
