@@ -14,8 +14,8 @@
 
 (defvar mew-sinfo-list
   '("scan-id" "find-key" "cursor-line" "direction" "start-point"
-    "cache-time" "scan-form" "refile" "refile-back"
-    "disp-msg" "case" "folder" "proto"))
+    "cache-time" "summary-form" "refile" "refile-back"
+    "disp-msg" "case" "folder" "proto" "mid-marker" "mid-line" "ttl-line"))
 
 (mew-blinfo-defun 'mew-sinfo mew-sinfo-list)
 
@@ -73,11 +73,6 @@ files to FILES."
 ;;; Subfunctions
 ;;;
 
-(defsubst mew-summary-msg-invalidp ()
-  (save-excursion
-    (beginning-of-line)
-    (looking-at mew-regex-msg-invalid)))
-
 (defsubst mew-summary-local-p ()
   (mew-folder-localp (mew-summary-folder-name 'ext)))
 
@@ -177,10 +172,10 @@ to the exclusion of Thread folder."
      (message "This command cannot be used in %s" (mew-summary-folder-name)))
     (t ,@body)))
 
-(defmacro mew-summary-with-mewls (&rest body)
+(defmacro mew-summary-with-mewl (&rest body)
   `(cond
-    ((not (mew-which-exec mew-prog-mewls))
-     (message "'%s' not found!" mew-prog-mewls))
+    ((not (mew-which-exec mew-prog-mewl))
+     (message "'%s' not found!" mew-prog-mewl))
     (t ,@body)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -218,7 +213,7 @@ and retern (beg . end)."
 
 (defvar mew-mode-line-target "%p")
 (defvar mew-mode-line-format
-  `("[" mew-summary-buffer-left-msgs " more]"
+  `("" mew-summary-buffer-left-msgs
     (mew-summary-buffer-raw "*")
     (mew-summary-buffer-secure-process ,mew-secure-format2)))
 
@@ -241,12 +236,15 @@ and retern (beg . end)."
     (when target
       (setq pos (- (length prev) (length (member target prev))))
       (setcar (nthcdr pos prev) mew-mode-line-format))
+    (when (boundp 'line-number-mode)
+      (make-local-variable 'line-number-mode)
+      (setq line-number-mode nil))
     (or (assq 'mew-summary-buffer-process mode-line-process)
 	(setq mode-line-process
 	      (append mew-mode-line-process mode-line-process)))))
 
 (defun mew-summary-reset-mode-line ()
-  (setq mew-summary-buffer-left-msgs  "-")) ;; local variable
+  (setq mew-summary-buffer-left-msgs nil))
 
 (defun mew-summary-mode-name (name)
   (let ((in (if (mew-case-default-p mew-case-input)
@@ -263,41 +261,26 @@ and retern (beg . end)."
 
 (defun mew-summary-mode-line ()
   (unless mew-summary-buffer-process
-    (let (regex left)
-      (if (and (mew-thread-p) mew-use-thread-separator)
-	  (progn
-	    (setq regex (concat "^" (regexp-quote mew-thread-separator)))
-	    (setq left (1- (mew-count-lines (point) (point-max) regex))))
-	(setq left (1- (mew-count-lines (point) (point-max)))))
-      (if (and (mew-decode-syntax-p)
-	       (equal (mew-decode-syntax-buffer) (current-buffer)))
-	  (setq left (- left (mew-count-lines (mew-decode-syntax-begin)
-					      (mew-decode-syntax-end)))))
-      (if (numberp left)
-	  (if (= left 0)
-	      (setq mew-summary-buffer-left-msgs  "-") ;; local variable
-	    (setq mew-summary-buffer-left-msgs (int-to-string left))))))
-  (force-mode-line-update))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Folder name
-;;;
-
-(defun mew-summary-folder-name (&optional ext)
-  (cond
-   (ext (buffer-name))
-   ((mew-summary-p)
-    (buffer-name))
-   ((mew-virtual-p)
-    (save-excursion
-      (unless (mew-summary-message-number)
-	(mew-summary-goto-message))
-      (beginning-of-line)
-      (if (looking-at mew-regex-virtual)
-	  (mew-match-string 1)
-	nil)))
-   (t nil)))
+    (let ((pos (point))
+	  (mid-point (marker-position (mew-sinfo-get-mid-marker)))
+	  (mid-line (mew-sinfo-get-mid-line))
+	  (ttl (mew-sinfo-get-ttl-line))
+	  cur left)
+      (if (< pos mid-point)
+	  (if (< pos (/ mid-point 2))
+	      (setq cur (mew-count-lines (point-min) pos))
+	    (setq cur (- mid-line (mew-count-lines pos mid-point))))
+	(if (< pos (+ mid-point (/ mid-point 2)))
+	    (setq cur (+ mid-line (mew-count-lines mid-point pos)))
+	  (setq cur (- ttl (mew-count-lines pos (point-max))))))
+      (unless (and (mew-decode-syntax-p)
+		   (equal (mew-decode-syntax-buffer) (current-buffer))
+		   (>= pos (mew-decode-syntax-begin))
+		   (<= pos (mew-decode-syntax-end)))
+	(setq cur (1+ cur)))
+      (setq left (- ttl cur))
+      (setq mew-summary-buffer-left-msgs (format "[%d/%d{%d}]" cur ttl left)))
+    (force-mode-line-update)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -311,7 +294,8 @@ and retern (beg . end)."
     nil) ;; not exclusive
    ((processp mew-summary-buffer-process)
     (or no-msg
-	(message (substitute-command-keys
+	(message "%s"
+		 (substitute-command-keys
 		  "\\<mew-summary-mode-map>Another process is running. Try later or type \\[mew-summary-kill-subprocess] to kill it")))
     nil) ;; not exclusive
    (t t))) ;; exclusive
@@ -385,7 +369,7 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	messages. When called with '\\[universal-argument]', parts are
 	skipped.
 
-\\[mew-summary-jump-message]	Jump to a message according to the number
+\\[mew-summary-goto-line]	Jump to a line according to the number
 	which you input.
 \\[mew-summary-jump-top]	Go to the beginning of this Summary mode.
 \\[mew-summary-jump-bottom]	Go to the end of this Summary mode.
@@ -452,15 +436,12 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	'\\[mew-summary-mark-delete]', '\\[mew-summary-mark-regexp]',
 	and '\\[mew-summary-mark-all]'.
 
-\\[mew-summary-display-review-down]	Jump to the message marked with
-	'*' below.
-\\[mew-summary-display-review-up]	Jump to the message marked with '*'
-	above.
+\\[mew-summary-display-review-down]	Jump to the message marked with	'*' below.
+\\[mew-summary-display-review-up]	Jump to the message marked with '*' above.
 
 \\[mew-summary-multi]	Put the multi the '@' mark on this message for
-	'\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]',
-	'\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'. It
-	can overlay the '*' mark.  The cursor stays always.
+	'\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'. 
+	It can overlay the '*' mark.  The cursor stays always.
 
 \\[mew-summary-unshar]	Apply 'unshar' on messages marked with '@'.
 \\[mew-summary-uudecode]	Apply 'uudecode' on messages marked with '@'.
@@ -582,6 +563,10 @@ The keys that are defined for both Summary mode and Virtual mode are:
 \\[mew-summary-cmd-msgs]	Executing an external command specifying messages
 	marked with '@' as arguments.
 
+\\[mew-summary-info]	Display the folder name of this message and the message number.
+	If this is a cache message, both the unique id and the size are 
+	also shown.
+
 \\[mew-summary-send-message]	If in +queue, send the messages in +queue.
 	If in +postq, post the messages in +postq.
 	If in %queue, process the jobs in %queue.
@@ -621,6 +606,8 @@ Virtual mode.
 
 \\[mew-summary-exec-delete]	Process messages marked with 'D'.
 \\[mew-summary-exec-refile]	Process messages marked with 'o'.
+	If called with '\\[universal-argument]', only messages whose destination is 
+	the same as that of the current message are processed.
 
 \\[mew-summary-mark-refile]	Put the refile mark onto all messages
 	marked with '*'.  This is very convenient to refile all
@@ -652,7 +639,7 @@ folder.
 	which you input, then put the '*' mark onto them. If called
 	with '\\[universal-argument]', execute
 	'mew-summary-pick-with-external'. Otherwise,
-	'mew-summary-pick-with-mewls' is called.  which you input, then
+	'mew-summary-pick-with-mewl' is called.  which you input, then
 	put the '*' mark onto them.
 
 \\[mew-summary-find-keyword-down]	Display a message marked with '*'
@@ -681,10 +668,6 @@ folder.
 	or the region. If this command is used in a remote folder,
 	local cache messages are sorted.
 
-\\[mew-summary-pack]	Pack messages and list them up again.  After
-	packing, the cursor stays in the current message.  If this command is
-	used in a remote folder,
-
 \\[mew-summary-make-thread]	This command makes threads for the
 	Summary mode as Virtual mode, then the cursor jump onto the current
 	message in the Virtual mode. If a corresponding Virtual mode exists,
@@ -771,6 +754,13 @@ is equivalent to
     (mew-remove-buffer (mew-buffer-message))
     (delete-frame)))
 
+(defun mew-summary-reset ()
+  (mew-unhighlight-cursor-line)
+  (mew-window-configure 'summary)
+  (mew-current-set nil nil nil)
+  (mew-summary-reset-mode-line)
+  (mew-decode-syntax-delete))
+
 (provide 'mew-summary)
 
 ;;; Copyright Notice:
