@@ -49,16 +49,16 @@
       (mew-summary-goto-message)
       (cond
        ((eq (mew-sinfo-get-direction) 'up)
-	(when (re-search-backward mew-regex-msg-or-part nil t)
-	  (setq fld (mew-summary-folder-name))
-	  (setq next (mew-summary-message-number))))
+	(when (re-search-backward mew-regex-sumsyn-short nil t)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq next (mew-sumsyn-message-number))))
        ((eq (mew-sinfo-get-direction) 'down)
 	(if (mew-decode-syntax-end)
 	    (goto-char (mew-decode-syntax-end))
 	  (forward-line))
-	(when (re-search-forward mew-regex-msg-or-part nil t)
-	  (setq fld (mew-summary-folder-name))
-	  (setq next (mew-summary-message-number))))))
+	(when (re-search-forward mew-regex-sumsyn-short nil t)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq next (mew-sumsyn-message-number))))))
     ;; should get the cursor back for display
     (save-excursion
       (if (and fld next
@@ -220,6 +220,7 @@ function displays it again getting gack to the beginning."
 		   (mew-summary-display-raw fld msg mew-file-max-size)
 		   (setq prefetch t)
 		   (message
+		    "%s"
 		    (substitute-command-keys
 		     "Too large, truncated. To see the entire message, type '\\<mew-summary-mode-map>\\[mew-summary-analyze-again]'"))))
 		(t
@@ -251,7 +252,8 @@ function displays it again getting gack to the beginning."
 	 (if (mew-xinfo-get-decode-err)
 	     (message "MIME decoding error: %s" (mew-xinfo-get-decode-err)))
 	 (if (mew-xinfo-get-action)
-	     (message (mew-xinfo-get-action)))
+	     (message "%s" (mew-xinfo-get-action)))
+	 (mew-message-mode-line fld msg)
 	 (select-window win)
 	 ;; summary buffer
 	 (mew-summary-cursor-postscript)
@@ -420,6 +422,7 @@ with 'mew-decode-broken' reversed."
 	       (mew-summary-display-part cache part)))
 	 (if (mew-xinfo-get-decode-err)
 	     (message "MIME decoding error: %s" (mew-xinfo-get-decode-err)))
+	 (mew-message-mode-line fld msg)
 	 (select-window win)
 	 ;; summary buffer
 	 (if lines (forward-line lines))
@@ -462,6 +465,9 @@ with 'mew-decode-broken' reversed."
 	   ;; message buffer
 	   (mew-current-set fld msg nums)
 	   (mew-minfo-set-summary vfld)
+	   ;; The following codes are not necessary. 
+	   ;; (mew-decode-syntax-clear)
+	   ;; (mew-decode-syntax-delete)
 	   (mew-elet
 	    (mew-summary-display-preamble)
 	    (mew-decode-syntax-copy buf)
@@ -484,7 +490,10 @@ with 'mew-decode-broken' reversed."
 	      (goto-char (point-min)))))
 	   (mew-summary-display-postscript 'no-hook))
        (mew-remove-buffer buf)
-       (select-window win)))))
+       (mew-message-mode-line fld msg)
+       (select-window win)
+       ;; summary buffer
+       (mew-summary-cursor-postscript)))))
 
 (defun mew-summary-find-file (&optional arg)
   "Open this message and makes it read-only.
@@ -520,7 +529,7 @@ ad-hoc solution for putting the review mark on a thread.")
   (funcall mew-summary-down-function)
   (cond 
    ((re-search-forward mew-regex-msg-or-part nil t)
-    (mew-summary-thread-move-cursor)
+    (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'down)
     t)
    (t
@@ -529,9 +538,8 @@ ad-hoc solution for putting the review mark on a thread.")
     (unless (pos-visible-in-window-p)
       ;; This ensures that some messages are displayed in Summary.
       (recenter))
-    (mew-summary-thread-move-cursor)
-    (mew-window-configure 'summary)
-    (mew-current-set nil nil nil)
+    (mew-thread-move-cursor)
+    (mew-summary-reset)
     (message "No more messages")
     nil)))
 
@@ -539,14 +547,13 @@ ad-hoc solution for putting the review mark on a thread.")
   (beginning-of-line)
   (cond 
    ((re-search-backward mew-regex-msg-or-part nil t)
-    (mew-summary-thread-move-cursor)
+    (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'up)
     t)
    (t 
-    (mew-summary-thread-move-cursor)
+    (mew-thread-move-cursor)
     (mew-decode-syntax-delete)
-    (mew-window-configure 'summary)
-    (mew-current-set nil nil nil)
+    (mew-summary-reset)
     (message "No more messages")
     nil)))
 
@@ -636,6 +643,31 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 	   (select-window win))
        (call-interactively 'mew-summary-display)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Cursor motion
+;;;
+
+(defun mew-summary-next-line ()
+  (interactive)
+  (beginning-of-line)
+  (forward-line)
+  (while (or (looking-at mew-regex-thread-separator)
+	     (eq (get-text-property (point) 'invisible) t))
+    (forward-line))
+  (if (and (mew-thread-p) mew-use-thread-cursor)
+      (mew-thread-move-cursor)))
+
+(defun mew-summary-previous-line ()
+  (interactive)
+  (beginning-of-line)
+  (forward-line -1)
+  (while (or (looking-at mew-regex-thread-separator)
+	     (eq (get-text-property (point) 'invisible) t))
+    (forward-line -1))
+  (if (and (mew-thread-p) mew-use-thread-cursor)
+      (mew-thread-move-cursor)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Mouse
@@ -652,7 +684,7 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
