@@ -26,7 +26,7 @@
 		      (specifier-instance default-toolbar-visible-p))
 	     (set-specifier default-toolbar-visible-p nil)
 	     (set-specifier default-toolbar-visible-p t)))
-       (save-buffer)) ;; to make sure not to use this draft again
+       (mew-draft-save-buffer)) ;; to make sure not to use this draft again
      (mew-set-file-modes (buffer-file-name))
      (mew-touch-folder mew-draft-folder)
      (message "Draft is prepared")))
@@ -36,7 +36,11 @@
   (unless mew-use-other-frame-for-draft
     (if (get-buffer (mew-buffer-message))
 	(delete-windows-on (mew-buffer-message)))
-    (if (or mew-use-full-window (< (window-height) 25)) (delete-other-windows))
+    (cond
+     ((< (window-height) 25)
+      (delete-other-windows))
+     (mew-use-full-window
+      (mew-delete-other-window)))
     (let ((split-window-keep-point t))
       (split-window-vertically))))
 
@@ -55,16 +59,31 @@
 ;;; Sending
 ;;;
 
-(defun mew-summary-send (&optional to cc subject)
+(defun mew-summary-postp ()
+  (cond
+   ((mew-summary-p)
+    (mew-folder-nntpp (mew-summary-folder-name 'ext)))
+   ((mew-virtual-p)
+    (mew-folder-nntpp (mew-virtual-folder-name)))
+   (t
+    (mew-folder-nntpp (mew-proto mew-case-output)))))
+
+(defun mew-summary-send (&optional to cc subject newsgroups)
   "Write a message. A new draft is prepared in Draft mode."
   (interactive)
   (let ((draft (mew-folder-new-message mew-draft-folder)))
+    (if (mew-summary-postp)
+	(if (null newsgroups) (setq newsgroups ""))
+      (when (and mew-ask-to (null to))
+        (setq to (mew-input-address (concat mew-to: " "))))
+      (when (and mew-ask-cc (null cc))
+        (setq cc (mew-input-address (concat mew-cc: " ")))))
     (mew-current-set-window-config)
     (mew-window-configure 'draft)
     (mew-summary-prepare-draft
      (mew-draft-find-and-switch draft)
      (mew-delete-directory-recursively (mew-attachdir draft))
-     (mew-draft-header subject nil to cc)
+     (mew-draft-header subject nil to cc newsgroups)
      (mew-draft-mode)
      (run-hooks 'mew-draft-mode-newdraft-hook))))
 
@@ -87,33 +106,164 @@
 	  (setq str (replace-match (if replace (eval replace) "") nil t str))))
     str))
 
-(defun mew-summary-reply (&optional onlytofrom)
-  "Answer to this message. A new draft is prepared in Draft mode. 
-Mew automatically decides To: and Cc:. Addresses on To: and Cc:
-are decided as follows:
+(defun mew-to-cc-newsgroups (replysender)
+  (let (alist ent key tcn-flds tcn flds to cc newsgroups fromme func)
+    (cond 
+     (replysender
+      (setq alist mew-reply-sender-alist)
+      (setq func 'mew-header-parse-address-list2))
+     ((mew-is-my-address mew-regex-my-address-list
+			 (mew-header-parse-address mew-from:))
+      (setq fromme t)
+      (setq alist mew-reply-fromme-alist)
+      ;; If from me, let's leave the "anonymous:;" keyword.
+      (setq func 'mew-header-parse-address-list))
+     (t ;; reply all
+      (setq alist mew-reply-all-alist)
+      (setq func 'mew-header-parse-address-list2)))
+    (catch 'loop
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq key (car ent))
+	(setq ent (cdr ent))
+	(when (or (eq key t)
+		  (and (stringp key) (mew-header-get-value key))
+		  (and (listp key)
+		       (string= (mew-header-get-value (nth 0 key))
+				(nth 1 key))))
+	  (while ent
+	    (setq tcn-flds (car ent))
+	    (setq ent (cdr ent))
+	    (setq tcn (car tcn-flds))
+	    (setq flds (cdr tcn-flds))
+	    (cond
+	     ((mew-case-equal tcn mew-to:)
+	      (setq to (funcall func flds)))
+	     ((mew-case-equal tcn mew-cc:)
+	      (setq cc (funcall func flds)))
+	     ((mew-case-equal tcn mew-newsgroups:)
+	      (setq newsgroups (funcall func flds)))))
+	  (throw 'loop nil))))
+    (list to cc newsgroups fromme)))
 
-    If From: of the message to be replied is not from me:
-        Reply-To: does not exist in the message to be replied
-            Copy From: of the message to be replied to To: (1)
-            Copy To: and Cc: of the message to be replied to Cc: (2)
-        Reply-To: exists in the message to be replied
-            Copy From: and Reply-To: of the message to be replied to To: (3)
-            Copy To: and Cc: of the message to be replied to Cc: (4)
-    If From: of a message to be replied is from me:
-        Copy To: of the message to be replied to To: (5)
-        Copy Cc: of the message to be replied to Cc: (6)
+(defun mew-in-reply-to-references ()
+  (let ((old-message-id  (mew-idstr-get-first-id
+			  (mew-header-get-value mew-message-id:)))
+	(old-in-reply-to (mew-idstr-get-first-id
+			  (mew-header-get-value mew-in-reply-to:)))
+	(old-references  (mew-idstr-to-id-list
+			  (mew-header-get-value mew-references:) nil 'rev))
+	id-list in-reply-to references)
+    ;; Assuming that In-Reply-To: contains one ID.
+    (when old-message-id
+      ;; when old-message-id is nil, 
+      ;; we do not care even if old-references exist.
+      (setq in-reply-to old-message-id)
+      (cond
+       (old-references
+	(setq id-list old-references)
+	(mew-addq id-list old-in-reply-to)
+	(mew-addq id-list old-message-id)
+	(setq id-list (nreverse id-list)))
+       (t
+	(setq id-list (list old-message-id))
+	(mew-addq id-list old-in-reply-to)))
+      (setq references (mew-id-list-to-idstr id-list)))
+    (list in-reply-to references)))
 
-You can customize which fields are copied in the case (1)-(6) with the
-following variables:
+(defun mew-summary-reply (&optional replysender)
+  "Reply to this message. A new draft is prepared in Draft mode. 
+Values of To:, Cc:, and Newsgroups: are prepared accoring to
+three alists.
 
-    (1) mew-noreplyto-to-list
-    (2) mew-noreplyto-cc-list
-    (3) mew-replyto-to-list
-    (4) mew-replyto-cc-list
-    (5) mew-fromme-to-list
-    (6) mew-fromme-cc-list
+(1) If called with '\\[universal-argument]', replying to the
+    sender/poster only. In this case, 'mew-reply-sender-alist' is used.
 
-If executed with '\\[universal-argument]', only From: of the message is copied to To:.
+(2) If this message is sent by ME, you probably intend to reply with
+    the original header. In this case, 'mew-reply-fromme-alist' is used.
+
+(3) Otherwise, replying to all people listed. In this case, 
+    'mew-reply-all-alist' is used.
+
+The default value of 'mew-reply-sender-alist' is as follows:
+
+	'((\"Reply-To:\"
+	   (\"To:\" \"Reply-To:\" \"From:\"))
+	  (t
+	   (\"To:\" \"From:\")))
+
+This is read as follows:
+
+	(1.1) If Reply-To: exists, copy the values of Reply-To:
+              and From: to new To:.
+	(1.2) Otherwise, copy the value of From: to To:.
+
+If you would like to reply only to the address on Reply-To: (if any),
+set 'mew-reply-sender-alist' to:
+
+	'((\"Reply-To:\"
+	   (\"To:\" \"Reply-To:\"))
+	  (t
+	   (\"To:\" \"From:\")))
+
+The default value of 'mew-reply-fromme-alist' is as follows:
+
+	'((t
+	   (\"To:\" \"To:\")
+	   (\"Cc:\" \"Cc:\")
+	   (\"Newsgroups:\" \"Newsgroups:\"))))
+
+This is read as follows:
+
+	(2.1) Copying the value of To: to new To: and 
+              copying the value of Cc: to new Cc: and
+              copying the value of Newsgroups: to new Newsgroups:.
+
+The default value of 'mew-reply-all-alist' is as follows:
+
+	'(((\"Followup-To:\" \"poster\")
+	   (\"To:\" \"From:\"))
+	  (\"Followup-To:\"
+	   (\"Newsgroups:\" \"Followup-To:\" \"Newsgroups:\"))
+	  (\"Newsgroups:\"
+	   (\"Newsgroups:\" \"Newsgroups:\"))
+	  (\"Reply-To:\"
+	   (\"To:\" \"Reply-To:\" \"From:\")
+	   (\"Cc:\" \"To:\" \"Cc:\" \"Apparently-To:\"))
+	  (t
+	   (\"To:\" \"From:\")
+	   (\"Cc:\" \"To:\" \"Cc:\" \"Apparently-To:\")))
+
+This is read as follows:
+
+	(3.1) If the value of Followup-To: is \"poster\", copying the
+              value of From: to new To:.
+	(3.2) If Followup-To: exists, copying the values of
+              Followup-To: and Newsgroups: to new Newsgroups:.
+	(3.3) If Newsgroups: exists, copying the value of Newsgroups:
+              to Newsgroups:.
+	(3.4) If Reply-To: exists, copying the values of Reply-To: and
+              From: to new To:. And copying the values of To:, Cc: and
+              Apparently-To: to new Cc:.
+
+	(3.5) Otherwise, copying the value of From: to new To:. And
+              copying the values of To:, Cc: and Apparently-To: to
+              new Cc:.
+
+You may want to set 'mew-reply-all-alist' to:
+
+	'(((\"Followup-To:\" \"poster\")
+	   (\"To:\" \"From:\"))
+	  (\"Followup-To:\"
+	   (\"Newsgroups:\" \"Followup-To:\"))
+	  (\"Newsgroups:\"
+	   (\"Newsgroups:\" \"Newsgroups:\"))
+	  (\"Reply-To:\"
+	   (\"To:\" \"Reply-To:\"))
+	  (t
+	   (\"To:\" \"From:\")
+	   (\"Cc:\" \"To:\" \"Cc:\" \"Apparently-To:\")))
 "
   (interactive "P")
   (mew-summary-msg-or-part
@@ -124,8 +274,8 @@ If executed with '\\[universal-argument]', only From: of the message is copied t
 	  (fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
 	  cwin cbuf draft case
-	  from reply-to to cc newsgroups subject in-reply-to references
-	  encrypted fromme ret-buf)
+	  subject to cc newsgroups in-reply-to references
+	  encrypted fromme ret)
       (if (string= (mew-summary-folder-name) mew-draft-folder)
 	  (message "Cannot reply to draft message")
 	(setq draft (mew-folder-new-message mew-draft-folder))
@@ -140,12 +290,11 @@ If executed with '\\[universal-argument]', only From: of the message is copied t
 	 (let ((mew-use-full-window nil))
 	   (mew-summary-display nil))
 	 ;; see also mew-draft-cite
-	 (setq ret-buf (or (save-excursion
-			     (set-buffer (mew-buffer-message))
-			     (if (mew-header-p) (current-buffer)))
-			   ;; header exists only in cache if multipart
-			   (mew-cache-hit fld msg)))
-	 (set-buffer ret-buf)
+	 (set-buffer (or (save-excursion
+			   (set-buffer (mew-buffer-message))
+			   (if (mew-msghder-p) (current-buffer)))
+			 ;; header exists only in cache if multipart
+			 (mew-cache-hit fld msg)))
 	 (when mew-case-guess-when-replied
 	   (setq case (mew-draft-get-case-by-guess
 		       mew-case-guess-when-replied-alist)))
@@ -155,61 +304,24 @@ If executed with '\\[universal-argument]', only From: of the message is copied t
 	   ;; mew-header-* cannot widen essentially. So widen here.
 	   (widen)
 	   ;; now cache buffer
-	   (setq from (mew-header-parse-address mew-from:))
-	   (setq reply-to (mew-header-parse-address mew-reply-to:))
-	   (cond 
-	    (onlytofrom (setq to from))
-	    ((mew-is-my-address mew-regex-my-address-list from)
-	     ;; This message was sent by me. So, maintain To: and Cc:.
-	     (setq fromme t)
-	     (setq to (mew-header-parse-address-list2 mew-fromme-to-list))
-	     (setq cc (mew-header-parse-address-list2 mew-fromme-cc-list))
-	     (unless to (setq to (or reply-to from)))) ;; do not use list
-	    (t
-	     (cond 
-	      (reply-to
-	       (setq to (mew-header-parse-address-list2 mew-replyto-to-list))
-	       (setq cc (mew-header-parse-address-list2 mew-replyto-cc-list)))
-	      (t
-	       (setq to (mew-header-parse-address-list2 mew-noreplyto-to-list))
-	       (setq cc (mew-header-parse-address-list2 mew-noreplyto-cc-list))))))
-	   (setq newsgroups (or (mew-header-get-value mew-followup-to:)
-				(mew-header-get-value mew-newsgroups:)))
-	   (if (and newsgroups (mew-case-equal newsgroups "poster"))
-	       (setq newsgroups nil))
+	   ;; 
+	   ;; Subject:
 	   (setq subject (mew-header-get-value mew-subj:))
-	   (if subject
-	       (setq subject (mew-subject-simplify (concat mew-reply-string subject))))
-	   (let ((old-message-id  (mew-header-get-value mew-message-id:))
-		 (old-in-reply-to (mew-header-get-value mew-in-reply-to:))
-		 (old-references  (mew-header-get-value mew-references:))
-		 (regex "<[^>]+>")
-		 (start 0) tmp-ref skip)
-	     (if (and old-message-id (string-match regex old-message-id))
-		 (setq old-message-id (match-string 0 old-message-id))
-	       (setq old-message-id nil))
-	     ;; Assuming that In-Reply-To: contains one ID.
-	     (if (and old-in-reply-to (string-match regex old-in-reply-to))
-		 (setq old-in-reply-to (match-string 0 old-in-reply-to))
-	       (setq old-in-reply-to nil))
-	     (if (null old-message-id)
-		 () ;; we do not care even if old-references exist.
-	       (setq in-reply-to old-message-id)
-	       (if (null old-references)
-		   (setq tmp-ref (if old-in-reply-to 
-				     (list old-in-reply-to old-message-id)
-				   (list old-message-id)))
-		 (while (string-match "<[^>]+>" old-references start)
-		   (setq start (match-end 0))
-		   (setq tmp-ref (cons (match-string 0 old-references) tmp-ref)))
-		 ;; described in old drums but not in RFC2822
-		 (mew-addq tmp-ref old-in-reply-to)
-		 (setq tmp-ref (nreverse (cons old-message-id tmp-ref))))
-	       (if (integerp mew-references-max-count)
-		   (setq skip (- (length tmp-ref) mew-references-max-count)))
-	       (if (and (numberp skip) (> skip 0))
-		   (setq tmp-ref (nthcdr skip tmp-ref)))
-	       (setq references (mew-join "\n\t" tmp-ref)))))
+	   (when subject
+	     (setq subject (concat mew-reply-string subject))
+	     (setq subject (mew-subject-simplify subject)))
+	   ;;
+	   ;; To:, Cc:, Newsgroups:
+	   (setq ret (mew-to-cc-newsgroups replysender))
+	   (setq to (nth 0 ret))
+	   (setq cc (nth 1 ret))
+	   (setq newsgroups (nth 2 ret))
+	   (setq fromme (nth 3 ret))
+	   ;;
+	   ;; In-Reply-To:, References:
+	   (setq ret (mew-in-reply-to-references))
+	   (setq in-reply-to (nth 0 ret))
+	   (setq references (nth 1 ret)))
 	 ;;
 	 (if (window-live-p cwin)
 	     (select-window cwin) ;; draft
@@ -224,36 +336,34 @@ If executed with '\\[universal-argument]', only From: of the message is copied t
 	   (goto-char (mew-header-end))
 	   (forward-line))
 	 (mew-draft-mode encrypted)
-	 (run-hooks 'mew-draft-mode-newdraft-hook))
-	ret-buf)))))
+	 (run-hooks 'mew-draft-mode-newdraft-hook)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Replaying with citation
 ;;;
 
-(defun mew-summary-reply-with-citation (&optional onlytofrom)
+(defun mew-summary-reply-with-citation (&optional replysender)
   "Answer to this message. A new draft is prepared in Draft mode. 
 And this message is automatically cited. See also 'mew-summary-reply'."
   (interactive "P")
   (mew-summary-msg-or-part
    (mew-summary-not-in-draft
-    (let (mew-message-citation-buffer)
-      (let ((mew-summary-reply-position nil))
-	;; for mew-use-other-frame-for-draft
-	(setq mew-message-citation-buffer (mew-summary-reply onlytofrom)))
+    (let ((mew-summary-reply-position nil)
+	  (mew-message-citation-frame-id (mew-frame-id)))
+      (mew-summary-reply replysender)
       ;; mew-draft-mode-hook may insert text.
       (save-excursion
 	(goto-char (point-max))
 	(run-hooks 'mew-before-cite-hook)
-	(mew-draft-cite))
-      ;; the cursor is after To:
-      (cond
-       ((eq mew-summary-reply-with-citation-position 'body)
-	(goto-char (mew-header-end))
-	(forward-line))
-       ((eq mew-summary-reply-with-citation-position 'end)
-	(goto-char (point-max))))))))
+	(mew-draft-cite)))
+    ;; the cursor is after To:
+    (cond
+     ((eq mew-summary-reply-with-citation-position 'body)
+      (goto-char (mew-header-end))
+      (forward-line))
+     ((eq mew-summary-reply-with-citation-position 'end)
+      (goto-char (point-max)))))))
   
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -273,6 +383,8 @@ Draft mode and this message is automatically attached."
 	   (file (mew-expand-folder fld msg))
 	   (draft (mew-folder-new-message mew-draft-folder))
 	   (draftdir (file-name-nondirectory draft))
+	   (to (and mew-ask-to (mew-input-address (concat mew-to: " "))))
+	   (cc (and mew-ask-cc (mew-input-address (concat mew-cc: " "))))
 	   subject fwsubject cwin)
       (mew-summary-prepare-draft
        (mew-summary-prepare-three-windows)
@@ -286,7 +398,7 @@ Draft mode and this message is automatically attached."
        ;;
        (set-buffer (or (save-excursion
 			 (set-buffer (mew-buffer-message))
-			 (if (mew-header-p) (current-buffer)))
+			 (if (mew-msghder-p) (current-buffer)))
 		       ;; header exists only in cache if multipart
 		       (mew-cache-hit fld msg)))
        (setq subject (mew-header-get-value mew-subj:))
@@ -294,7 +406,7 @@ Draft mode and this message is automatically attached."
 	   (setq fwsubject (mew-subject-simplify (concat mew-forward-string subject))))
        (select-window cwin) ;; draft
        ;;
-       (mew-draft-header fwsubject 'nl)
+       (mew-draft-header fwsubject 'nl to cc)
        (mew-draft-mode)
        (run-hooks 'mew-draft-mode-newdraft-hook)
        (mew-draft-multi-copy draft (list file))
@@ -316,12 +428,14 @@ attached."
    (mew-summary-not-in-draft
     (mew-current-set-window-config)
     (let* ((draft (mew-folder-new-message mew-draft-folder))
-	   (draftdir (file-name-nondirectory draft)))
+	   (draftdir (file-name-nondirectory draft))
+	   (to (and mew-ask-to (mew-input-address (concat mew-to: " "))))
+	   (cc (and mew-ask-cc (mew-input-address (concat mew-cc: " ")))))
       (mew-summary-prepare-draft
        (mew-summary-prepare-three-windows)
        (mew-draft-find-and-switch draft t)
        (mew-delete-directory-recursively (mew-attachdir draft))
-       (mew-draft-header nil 'nl)
+       (mew-draft-header nil 'nl to cc)
        (mew-draft-mode)
        (run-hooks 'mew-draft-mode-newdraft-hook)
        (mew-draft-multi-copy draft FILES)
@@ -334,7 +448,7 @@ attached."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
