@@ -90,7 +90,7 @@ Summary mode."
 	  (if (mew-virtual-thread-p folder)
 	      (if (mew-thread-cache-valid-p folder)
 		  (mew-summary-switch-to-folder folder)
-		(message "%s is old. " folder))
+		(message "%s is old" folder))
 	    (mew-summary-switch-to-folder folder))
         (message "No such virtual folder: %s" folder)))
      (t ;; local/remote folders
@@ -108,36 +108,30 @@ Summary mode."
 	(when scanp
 	  (if no-ls
 	      (progn
-		(mew-summary-switch-to-folder folder) ;; Virtual mode
-		(if goend (goto-char (point-max))))
+		(mew-summary-switch-to-folder folder)
+		(if goend (goto-char (point-max)))
+		(if (pos-visible-in-window-p (point-min))
+		    (mew-summary-cook-window)))
 	    (setq new-folder (mew-summary-switch-to-folder folder))
-	    (mew-summary-ls nil (or goend new-folder)))))))))
+	    (if (and (mew-summary-ls nil (or goend new-folder))
+		     (pos-visible-in-window-p (point-min)))
+		(mew-summary-cook-window)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Moving to message
+;;; Going to a line
 ;;;
 
-(defun mew-summary-jump-message (&optional msg)
-  "Jump to a message according to the number which you input.
-If 'mew-summary-jump-message-then-display' is non-nil,
+(defun mew-summary-goto-line ()
+  "Jump to a line according to the number which you input.
+If 'mew-summary-goto-line-then-display' is non-nil,
 the message is then displayed."
-  (interactive "P")
-  (if (integerp msg) (setq msg (int-to-string msg))
-    (if (not (stringp msg)) (setq msg nil)))
-  (let ((here (point)))
-    (unless msg (setq msg (read-string "Message No.: " "")))
-    (cond 
-     ((string= msg "") ())
-     ((eq msg t) ;; xxx
-      (goto-char (point-max))) ;; (forward-line -1)
-     (t 
-      (goto-char (point-min))
-      (if (not (re-search-forward (mew-regex-jmp-msg msg) nil t))
-	  (goto-char here)
-	(beginning-of-line)
-	(if mew-summary-jump-message-then-display
-	    (mew-summary-display nil)))))))
+  (interactive)
+  (let ((line (string-to-int (read-string "Line No.: " ""))))
+    (when (/= line 0)
+      (goto-line line)
+      (if mew-summary-goto-line-then-display
+	  (mew-summary-display nil)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -233,7 +227,7 @@ the top message is then displayed."
 	 (if (mew-virtual-thread-p folder)
 	     (if (mew-thread-cache-valid-p folder)
 		 (setq getback t)
-	       (message "%s is old. " folder))
+	       (message "%s is old" folder))
 	   (setq getback t)))
 	(t
 	 (setq getback t)))
@@ -320,7 +314,7 @@ If called with '\\[universal-argument]', you can specify a printer name."
 	(set-buffer (mew-buffer-message))
 	(save-restriction
 	  (widen)
-	  (setq have-header (mew-header-p))
+	  (setq have-header (mew-msghder-p))
 	  (setq str (mew-buffer-substring (point-min) (point-max)))))
       (with-temp-buffer
 	(insert str)
@@ -367,9 +361,9 @@ argument."
 	  (command (read-string "Command: ")))
      (while (not (mew-which-exec command))
        (setq command (read-string "Command: ")))
-     (message (format "Executing %s for %s..." command msg))
+     (message "Executing %s for %s..." command msg)
      (call-process command nil nil nil file)
-     (message (format "Executing %s for %s...done" command msg)))))
+     (message "Executing %s for %s...done" command msg))))
 
 (defun mew-summary-cmd-msgs ()
   "Executing an external command specifying messages
@@ -379,9 +373,9 @@ marked with '@' as arguments."
    (let ((command (read-string "Command: ")))
      (while (not (mew-which-exec command))
        (setq command (read-string "Command: ")))
-     (message (format "Executing %s ..." command))
+     (message "Executing %s ..." command)
      (apply 'call-process command nil nil nil FILES)
-     (message (format "Executing %s ...done" command)))))
+     (message "Executing %s ...done" command))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -477,7 +471,7 @@ If called with '\\[universal-argument]', options are asked."
 	      (if (or (mew-ct-imagep ct) (mew-ct-modelp ct))
 		  (funcall (nth 1 program) cache begin end params fname)
 		(funcall (nth 1 program) cache begin end params)))
-	  (message (substitute-command-keys "Use '\\<mew-summary-mode-map>\\[mew-summary-execute-command]' or \\[universal-argument] \\<mew-summary-mode-map>\\[mew-summary-execute-external]'"))))
+	  (message "%s" (substitute-command-keys "Use '\\<mew-summary-mode-map>\\[mew-summary-execute-command]' or \\[universal-argument] \\<mew-summary-mode-map>\\[mew-summary-execute-external]'"))))
        (t
 	(cond
 	 ((listp program)
@@ -492,29 +486,39 @@ If called with '\\[universal-argument]', options are asked."
 	  (if (not (mew-which-exec program))
 	      (message "Program %s is not found" program)
 	    (let ((file (mew-make-temp-name fname))
-		  rcs)
+		  wcs)
 	      (save-excursion
 		(set-buffer cache)
 		;; NEVER use call-process-region for privacy reasons
-		(if (mew-ct-linebasep ct)
-		    (if (mew-ct-textp ct)
-			(cond
-			 ((string= mew-ct-htm ct)
-			  (setq rcs (mew-text/html-detect-cs begin end ctl)))
-			 ((string= mew-ct-xml ct)
-			  (setq rcs (mew-text/xml-detect-cs begin end ctl)))
-			 (t
-			  (setq rcs (or (mew-charset-to-cs
-					 (mew-syntax-get-param ctl "charset"))
-					(if mew-decode-broken
-					    (mew-charset-to-cs
-					     (mew-charset-guess-region
-					      begin end))
-					  mew-cs-text-for-write)))))
-		      (setq rcs mew-cs-text-for-write)) ;; xxx
-		  (setq rcs mew-cs-binary))
+		(cond
+		 ((not (mew-ct-linebasep ct))
+		  (setq wcs mew-cs-binary))
+		 ((not (mew-ct-textp ct))
+		  (setq wcs mew-cs-text-for-write))
+		 (t
+		  (cond
+		   ((string= mew-ct-htm ct)
+		    (setq wcs (mew-text/html-detect-cs begin end ctl))
+		    (unless (mew-coding-system-p wcs)
+		      (setq wcs (mew-charset-to-cs
+				 (mew-syntax-get-param ctl "charset")))))
+		   ((string= mew-ct-xml ct)
+		    (setq wcs (mew-text/xml-detect-cs begin end ctl))
+		    (unless (mew-coding-system-p wcs)
+		      (setq wcs (mew-charset-to-cs
+				 (mew-syntax-get-param ctl "charset")))))
+
+		   (t
+		    (setq wcs (mew-charset-to-cs
+			       (mew-syntax-get-param ctl "charset")))))
+		  (unless (mew-coding-system-p wcs)
+		    (setq wcs (if mew-decode-broken
+				  (mew-charset-to-cs
+				   (mew-charset-guess-region
+				    begin end))
+				mew-cs-text-for-write)))))
 		(mew-frwlet
-		 mew-cs-dummy rcs
+		 mew-cs-dummy wcs
 		 (write-region begin end file nil 'no-msg))
 		(if async
 		    (mew-mime-start-process program options file)
@@ -685,7 +689,7 @@ the other network processes are also hung up."
   "Concat Message/Partial fragments marked with '@' to an original
 message."   (interactive)
   (mew-summary-only
-   (mew-summary-with-mewls
+   (mew-summary-with-mewl
     (mew-summary-multi-msgs
      (let ((cfld (mew-summary-folder-name))
 	   (folder (mew-input-burst-folder))
@@ -705,7 +709,7 @@ message."   (interactive)
 	  mew-cs-text-for-read mew-cs-text-for-write
 	  (write-region (point-min) (point-max) tfile nil 'no-msg))
 	 (mew-erase-buffer)
-	 (call-process mew-prog-mewls nil t nil
+	 (call-process mew-prog-mewl nil t nil
 		       "-b" mew-mail-path "-l" "0"
 		       "-i" tfile "-d" "content-type")
 	 (mew-delete-file tfile)
@@ -936,11 +940,8 @@ you choose 'Summary mode only', you can quickly put the delete
     (if (mew-sinfo-get-disp-msg)
 	(mew-summary-display 'redisplay)
       (mew-summary-goto-message)
-      (mew-decode-syntax-delete)
-      (mew-window-configure 'summary)
-      (mew-current-set nil nil nil)
-      (mew-summary-cook-window)
-      (mew-summary-reset-mode-line))))
+      (mew-summary-reset)
+      (mew-summary-cook-window))))
   (run-hooks 'mew-summary-toggle-disp-msg-hook))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
