@@ -156,13 +156,14 @@
 (defsubst mew-syntax-cat (syntax part)
   (vconcat syntax (vector part)))
 
-(defun mew-syntax-cdp-format (ct file &optional cdp)
+(defun mew-syntax-cdp-format (ct file &optional cdp force-file)
+  ;; If cdp is a string, it is used as a file.
   (let* ((cdpdb (mew-cdpdb-by-ct ct))
 	 (val (mew-cdpdb-val cdpdb))
 	 (filep (mew-cdpdb-file cdpdb)))
     (if (stringp cdp) (setq file cdp))
     (when val
-      (if (and filep file)
+      (if (and (or filep force-file) file)
 	  (list val (list "filename" (file-name-nondirectory (mew-header-sanity-check-string file))))
 	(list val)))))
 
@@ -291,21 +292,20 @@
 ;;
 
 (defun mew-summary-goto-part (msg part)
-  (goto-char (point-min))
-  (cond
-   ((mew-virtual-p)
-    (if (re-search-forward (format "\r.*%s" msg) nil t)
-	(re-search-forward (format "^.....%s" part) nil t)))
-   ((mew-summary-p)
-    (if (re-search-forward (format "^ *%s" msg) nil t)
-	(re-search-forward (format "^.....%s" part) nil t))))
-  (beginning-of-line))
+  (let (here)
+    (save-excursion
+      (goto-char (point-min))
+      (and (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	   (re-search-forward (mew-regex-jmp-part part) nil t)
+	   (setq here (point))))
+    (and here (goto-char here))))
 
 (defun mew-summary-goto-message ()
   (if (eobp) (forward-line -1))
   (when (mew-in-decode-syntax-p)
     (goto-char (mew-decode-syntax-begin))
-    (forward-line -1)))
+    (forward-line -1))
+  (beginning-of-line))
 
 (defun mew-syntax-number ()
   (let ((event last-command-event)
@@ -319,14 +319,13 @@
       (setq str (toolbar-button-help-string (event-toolbar-button event)))
       ;; last-pressed-toolbar-button cannot be used.
       (if (string-match mew-regex-part-icon str)
-	  (setq ret (mew-match-string 1 str))))
+	  (setq ret (mew-sumsyn-part))))
     (or ret
 	(if (or (mew-in-attach-p)
 		(mew-in-decode-syntax-p))
 	    (save-excursion
 	      (beginning-of-line)
-	      (if (looking-at mew-regex-part)
-		  (mew-match-string 1)))))))
+	      (if (looking-at mew-regex-part) (mew-sumsyn-part)))))))
 
 (defsubst mew-syntax-number-to-nums (strnum)
   (if strnum
@@ -336,12 +335,69 @@
 (defsubst mew-syntax-nums ()
   (mew-syntax-number-to-nums (mew-syntax-number)))
 
+(defsubst mew-summary-goto-mark ()
+  (beginning-of-line))
+
+(defsubst mew-summary-goto-body (&optional after)
+  (when mew-summary-form-body-starter
+    (beginning-of-line)
+    (let ((beg (point)))
+      (forward-line)
+      (if (search-backward "\r" nil t)
+	  (if (search-backward mew-summary-form-body-starter beg t)
+	      (progn
+		(if after (goto-char (match-end 0)))
+		t)
+	    nil)
+	(forward-line -1) ;; just in case
+	nil))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; get number
+;;; Summary mode syntax
 ;;;
-    
-(defun mew-summary-message-number ()
+
+(defun mew-summary-info (arg)
+  "Display the folder name of this message and the message number.
+If this is a cache message, both the unique id and the size are also shown."
+  (interactive "P")
+  (when (mew-sumsyn-match mew-regex-sumsyn-long)
+    (if arg
+	(message "my-id=%s par-id=%s"
+		 (mew-sumsyn-my-id)
+		 (mew-sumsyn-parent-id))
+      (message "%s/%s size=%s uid=%s"
+	       (mew-sumsyn-folder-name)
+	       (mew-sumsyn-message-number)
+	       (mew-sumsyn-message-size)
+	       (mew-sumsyn-message-uid)))))
+
+(defsubst mew-sumsyn-match (regex)
+  (save-excursion
+    (beginning-of-line)
+    (let ((start (point)))
+      (end-of-line)
+      (when (search-backward "\r" start t)
+	(looking-at regex)))))
+
+(defsubst mew-sumsyn-folder-name    () (mew-match-string 1))
+(defsubst mew-sumsyn-message-number () (mew-match-string 2))
+(defsubst mew-sumsyn-my-id          () (mew-match-string 3))
+(defsubst mew-sumsyn-parent-id      () (mew-match-string 4))
+(defsubst mew-sumsyn-message-uid    () (mew-match-string 5))
+(defsubst mew-sumsyn-message-size   () (mew-match-string 6))
+
+(defsubst mew-sumsyn-part () (mew-match-string 1))
+
+(defsubst mew-summary-folder-name (&optional ext)
+  (if ext
+      (buffer-name)
+    (save-excursion
+      (mew-summary-goto-message)
+      (when (mew-sumsyn-match mew-regex-sumsyn-short)
+	(mew-sumsyn-folder-name)))))
+
+(defsubst mew-summary-message-number ()
   (let ((event last-command-event)
 	ret str)
     (if (and mew-xemacs-p
@@ -362,39 +418,48 @@
     (if ret
 	ret
       (unless (mew-in-decode-syntax-p)
-	(save-excursion
-	  (beginning-of-line)
-	  (cond 
-	   ((mew-summary-p)
-	    (if (looking-at mew-regex-msg)
-		(mew-match-string 1)
-	      nil))
-	   ((mew-virtual-p)
-	    (if (looking-at mew-regex-virtual)
-		(mew-match-string 2)
-	      nil))
-	   (t nil)))))))
+	(when (mew-sumsyn-match mew-regex-sumsyn-short)
+	  (mew-sumsyn-message-number))))))
 
-(defun mew-summary-message-number2 ()
+(defsubst mew-summary-message-number2 ()
   (save-excursion
     (mew-summary-goto-message)
-    (mew-summary-message-number)))
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (mew-sumsyn-message-number))))
 
-(defun mew-summary-message-uid ()
-  (if (looking-at mew-regex-summary2)
-      (mew-match-string 3)))
+(defsubst mew-summary-my-id ()
+  (when (mew-sumsyn-match mew-regex-sumsyn-long)
+    (mew-sumsyn-my-id)))
 
-(defun mew-summary-message-size ()
-  (if (looking-at mew-regex-summary2)
-      (mew-match-string 4)))
+(defsubst mew-summary-parent-id ()
+  (when (mew-sumsyn-match mew-regex-sumsyn-long)
+    (mew-sumsyn-parent-id)))
 
-(defun mew-summary-my-id ()
-  (if (looking-at mew-regex-summary)
-      (mew-match-string 1)))
+(defsubst mew-summary-message-uid ()
+  (when (mew-sumsyn-match mew-regex-sumsyn-long)
+    (mew-sumsyn-message-uid)))
 
-(defun mew-summary-parent-id ()
-  (if (looking-at mew-regex-summary)
-      (mew-match-string 2)))
+(defsubst mew-summary-message-size ()
+  (when (mew-sumsyn-match mew-regex-sumsyn-long)
+    (mew-sumsyn-message-size)))
+
+(defsubst mew-syntax-change-message-number (msg)
+  (save-excursion
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (mew-syntax-change-message-number2 msg))))
+
+;; see also mew-sumsyn-message-number
+(defsubst mew-syntax-change-message-number2 (msg)
+  (delete-region (match-beginning 2) (match-end 2))
+  (goto-char (match-beginning 2))
+  (insert-and-inherit msg))
+
+(defsubst mew-syntax-change-parent-id (id)
+  (save-excursion
+    (when (mew-sumsyn-match mew-regex-sumsyn-long)
+      (delete-region (match-beginning 4) (match-end 4))
+      (goto-char (match-beginning 4))
+      (insert-and-inherit id))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -404,8 +469,8 @@
 (defun mew-encode-syntax-single (file &optional ctl cte cd cid cdp 
 				      privacy decrypters)
   ;; cid is just for beauty
-  ;; if cdp is t, set cdp from file.
-  ;; if cdp is a string, set cdp from cdp.
+  ;; if cdp is non-nil, set cdp from file.
+  ;; if cdp is a string, set cdp from cdp as a file.
   (let* (ct ctdb)
     (cond
      (ctl
@@ -440,7 +505,7 @@
   (let ((i 1) (ret))
     (while (<= i n)
       (setq ret (vconcat ret (list (mew-encode-syntax-single
-				    (int-to-string i) nil nil nil nil t))))
+				    (int-to-string i) nil nil nil nil 'cdp))))
       (setq i (1+ i)))
     (vconcat (mew-encode-syntax-multi dir mew-type-mlm)
 	     (list (mew-encode-syntax-single mew-draft-coverpage 
@@ -873,7 +938,7 @@
   (and (marker-position mew-marker-decode-syntax-begin)
        (marker-position mew-marker-decode-syntax-end)))
 
-(defsubst mew-header-p ()
+(defsubst mew-msghder-p ()
   (save-restriction
     (widen)
     (next-single-property-change (point-min) 'read-only)))
@@ -894,7 +959,7 @@
   (marker-position mew-marker-decode-syntax-end))
 
 (defsubst mew-header-end ()
-  (mew-header-p))
+  (mew-msghder-p))
 
 (defsubst mew-attach-begin ()
   (if (mew-attach-p)
@@ -928,18 +993,20 @@
 
 ;;
 
-(defsubst mew-header-set (sep &optional header)
+(defsubst mew-header-set (sep)
   (mew-elet
-   (let ((end (point)))
+   (let ((pos (point)))
      (if sep
-	 (insert sep)
+	 (progn
+	   (insert-before-markers sep) ;; for mew-summary-reply
+	   (unless (bolp) (forward-line)))
        (forward-line))
-     (put-text-property end (point) 'read-only t)
-     (mew-front-nonsticky end (1+ end))
-     (if header
+     (put-text-property pos (point) 'read-only t)
+     (mew-front-nonsticky pos (1+ pos))
+     (if (mew-header-p)
 	 (mew-rear-sticky (1- (point)) (point))
        (mew-rear-nonsticky (1- (point)) (point)))
-     end)))
+     pos)))
 
 (defsubst mew-header-clear ()
   ;; the cursor moves to the end of the header (with some exceptions)
@@ -947,7 +1014,6 @@
    (mew-header-goto-end) ;; do not use mew-header-end
    (let ((pos (point)))
      (forward-line)
-;;     (put-text-property pos (point) 'read-only nil)
      ;; If the body contains the read-only property, mew-header-p
      ;; makes a mistake. So, remove the read-only property from
      ;; the entire buffer.
@@ -980,10 +1046,10 @@
       (mew-overlay-delete (mew-tinfo-get-attach-keymap))
       (mew-tinfo-set-attach-keymap nil))))
 
-(defsubst mew-header-prepared (&optional header)
-  (mew-header-set (concat mew-header-separator "\n") header)
+(defsubst mew-header-prepared ()
+  (mew-header-set (concat mew-header-separator "\n"))
   (mew-highlight-header)
-  (unless header (mew-draft-header-keymap)))
+  (unless (mew-header-p) (mew-draft-header-keymap)))
 
 (defsubst mew-draft-header-keymap ()
   (save-excursion
