@@ -48,30 +48,15 @@ should understand this risk."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-thread-indent-string "  "
-  "*A string to be inserted for indentation of thread."
-  :group 'mew-summary
-  :type 'string)
-
-(defvar mew-thread-indent-array nil)
-(defvar mew-thread-indent-array-size 33) ;; divide by 32
-
-(defcustom mew-use-fancy-thread nil
-  "*If non-nil, use fancy style thread visualization.
-See also mew-fancy-thread-indent-strings."
-  :group 'mew-summary
-  :type 'boolean)
-
-
-(defcustom mew-fancy-thread-indent-strings [" +" " +" " |" "  "]
-  "*Vector of strings to be used for indentation of fancy style thread.
-Valid only if mew-use-fancy-thread is non-nil.
+(defcustom mew-thread-indent-strings [" +" " +" " |" "  "]
+  "*Vector of strings to be used for indentation of thread.
 
 This consists of four members; 1st member for prefixing to a child
 message that is not the last one, 2nd member is for prefixing to the 
 last child, 3rd and 4th members are for prefixing to grand-child thread trees, 
 4th member is for the child tree of the last child message.
-For example, [ \" +\" \" +\" \" |\" \"  \" ] makes thread view below.
+
+Example1: [\" +\" \" +\" \" |\" \"  \"] makes thread view below.
 
     Message 1
      +Message 2
@@ -79,6 +64,14 @@ For example, [ \" +\" \" +\" \" |\" \"  \" ] makes thread view below.
      +Message 4
        +Message 5
 
+Example2: [\"  \" \"  \" \"  \" \"  \"] makes thread view below.
+
+    Message 1
+      Message 2
+        Message 3
+      Message 4
+        Message 5
+
 All members must have the same length."
   :group 'mew-summary
   :type 'sexp)
@@ -88,10 +81,6 @@ All members must have the same length."
   :group 'mew-summary
   :type 'boolean)
 
-
-(defvar mew-thread-indent-length nil)
-(defvar mew-thread-indent-list nil)
-
 (defvar mew-use-thread-separator nil
   "*If non-nil, the specified string is inserted between threads.")
 (defvar mew-thread-separator "--")
@@ -141,33 +130,24 @@ All members must have the same length."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;;
+;;; Thread setup
 ;;;
 
+(defvar mew-thread-indent-length nil)
+(defvar mew-thread-indent-width nil)
+
 (defun mew-thread-setup ()
-  (cond
-   (mew-use-fancy-thread
-    (let ((idt1 (aref mew-fancy-thread-indent-strings 0))
-	  (idt2 (aref mew-fancy-thread-indent-strings 1))
-	  (idt3 (aref mew-fancy-thread-indent-strings 2))
-	  (idt4 (aref mew-fancy-thread-indent-strings 3)))
-      (unless (and (= (length idt1) (length idt2))
-		   (= (length idt2) (length idt3))
-		   (= (length idt3) (length idt4)))
-	(error
-	 "All members of mew-fancy-thread-indent-strings must have the same length."))
-      (setq mew-thread-indent-length (length idt1))))
-   (t
-    (let* ((i 0)
-	   (size mew-thread-indent-array-size)
-	   (array (make-vector size nil))
-	   (indent ""))
-      (while (< i size)
-	(aset array i indent)
-	(setq i (1+ i))
-	(setq indent (concat indent mew-thread-indent-string)))
-      (setq mew-thread-indent-array array))
-    (setq mew-thread-indent-length (length mew-thread-indent-string)))))
+  (let ((idt1 (aref mew-thread-indent-strings 0))
+	(idt2 (aref mew-thread-indent-strings 1))
+	(idt3 (aref mew-thread-indent-strings 2))
+	(idt4 (aref mew-thread-indent-strings 3)))
+    (unless (and (= (string-width idt1) (string-width idt2))
+		 (= (string-width idt2) (string-width idt3))
+		 (= (string-width idt3) (string-width idt4)))
+      (error
+       "All members of mew-thread-indent-strings must have the same length"))
+    (setq mew-thread-indent-width  (string-width idt1))
+    (setq mew-thread-indent-length (length idt1))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -177,9 +157,7 @@ All members must have the same length."
 (defun mew-summary-mark-make-thread ()
   "Make threads for messages marked with '*'."
   (interactive)
-  (mew-summary-thread-region (point-min) (point-max) 'mark
-                             ;; (mew-summary-message-number) xxx
-			     ))
+  (mew-summary-thread-region (point-min) (point-max) 'mark))
 
 (defsubst mew-thread-cache-valid-p (vfolder)
   (let ((ofld (mew-thread-to-folder vfolder)))
@@ -200,8 +178,7 @@ threads for messages in the region.
 If called in Virtual mode, switch back to the corresponding Summary
 mode and move to the current message."
   (interactive "P")
-  (when (mew-syntax-number)
-    (mew-summary-goto-message))
+  (mew-summary-goto-message)
   (mew-decode-syntax-delete)
   (let* ((msg (mew-summary-message-number))
 	 (disp (mew-sinfo-get-disp-msg))
@@ -218,8 +195,7 @@ mode and move to the current message."
 	  (mew-summary-toggle-disp-msg (if disp 'on 'off))
 	  (if (not msg)
 	      (goto-char (point-max))
-	    (mew-summary-jump-message msg)
-	    (mew-summary-display nil)))))
+	    (mew-summary-move-and-display msg)))))
      ((and (setq vfolder (mew-folder-to-thread folder))
 	   (get-buffer vfolder)
 	   (mew-virtual-thread-p vfolder)
@@ -227,9 +203,8 @@ mode and move to the current message."
       (mew-summary-visit-folder vfolder)
       (mew-summary-toggle-disp-msg (if disp 'on 'off))
       (when msg
-	(mew-summary-jump-message msg)
-	(mew-summary-thread-move-cursor)
-	(mew-summary-display nil)))
+	(mew-summary-move-and-display msg)
+	(mew-thread-move-cursor)))
      (arg
       (let ((begend (mew-summary-get-region)))
 	(mew-summary-thread-region (car begend) (cdr begend))))
@@ -247,15 +222,13 @@ threads are created, see 'mew-use-complete-thread'."
 	    (disp (mew-sinfo-get-disp-msg))
 	    (ctime (mew-sinfo-get-cache-time))
 	    (case (mew-sinfo-get-case))
-	    (column (mew-summary-scan-form folder 'column))
-	    (numlen (mew-summary-scan-form-num folder))
- 	    overnum spacer indentlst
+	    (column (mew-get-summary-form folder 'column))
 	    db size top start me prnt prnt-cld my-id prnt-id func msg line
 	    tm1 tm2 tm3 tm4 tm5 tm6)
        (message "Making thread (first pass)...")
        (save-restriction
 	 (narrow-to-region beg end)
-	 (setq size (mew-count-lines beg end))
+	 (setq size (count-lines beg end))
 	 (cond
 	  ((<= size 211)
 	   (setq size 211))
@@ -267,19 +240,6 @@ threads are created, see 'mew-use-complete-thread'."
 	   (setq size 18211)))
 	 (setq db (make-vector size 0)) ;; hash
 	 (save-excursion
-	   ;; Calculate message number width
- 	   (goto-char (point-max))
- 	   (mew-summary-goto-message)
- 	   (when (looking-at mew-regex-msg)
- 	     (setq overnum (length (mew-match-string 1)))
- 	     (if (<= overnum numlen)
-		 (setq overnum nil)
-	       (setq column (+ column (- overnum numlen)))
-	       (setq spacer (make-vector overnum nil))
-	       (setq size 0)
-	       (while (< size overnum)
-		 (aset spacer size (make-string size ? ))
-		 (setq size (1+ size)))))
 	   (goto-char (point-min))
 	   (if mark
 	       (setq func (lambda ()
@@ -289,23 +249,18 @@ threads are created, see 'mew-use-complete-thread'."
 	   (while (funcall func)
 	     (beginning-of-line)
 	     (setq start (point))
-	     (if (not (looking-at mew-regex-summary))
+	     (if (not (mew-sumsyn-match mew-regex-sumsyn-long))
 		 (forward-line)
-	       (setq my-id   (mew-match-string 1))
-	       (setq prnt-id (mew-match-string 2))
-	       (setq msg     (mew-summary-message-number))
+	       (setq msg     (mew-sumsyn-message-number))
+	       (setq my-id   (mew-sumsyn-my-id))
+	       (setq prnt-id (mew-sumsyn-parent-id))
 	       (forward-line)
 	       ;; Throw away properties here and give properties later.
 	       ;; This is faster than inheriting properties.
 	       (setq line (mew-buffer-substring start (point)))
 	       (setq me (mew-thread-make-entry))
 	       (mew-thread-set-msg me msg)
-	       (if (and overnum (> overnum (length msg)))
-		   (mew-thread-set-line
-		    me
-		    (concat (aref spacer (- overnum (max (length msg) numlen)))
-			    line))
- 		 (mew-thread-set-line me line))
+	       (mew-thread-set-line me line)
 	       (if (string= my-id "")
 		   (setq top (cons me top))
 		 (mew-thread-set-myid me my-id)
@@ -334,6 +289,7 @@ threads are created, see 'mew-use-complete-thread'."
 	 (mew-vinfo-set-top nil)
 	 (mew-vinfo-set-db db)
  	 (mew-vinfo-set-column column)
+	 (mew-vinfo-set-flds (list folder))
 	 ;;
 	 (message "Making thread (second pass)...")
 	 (setq tm3 (current-time))
@@ -358,31 +314,20 @@ threads are created, see 'mew-use-complete-thread'."
 	 (setq tm4 (current-time))
 	 (message "Displaying thread...")
 	 (setq tm5 (current-time))
- 	 (setq mew-thread-indent-list nil)
-	 (cond
-	  (mew-use-fancy-thread
-	   (mew-summary-fancy-thread-print-top
- 	    (mew-vinfo-get-top) folder column))
-	  (t
-	   (mew-summary-thread-print
-	    (mew-vinfo-get-top) folder column)))
-	 ;; Indent property insert
- 	 (mew-elet
- 	  (while (setq indentlst (car mew-thread-indent-list))
- 	    (put-text-property (nth 0 indentlst) (nth 1 indentlst)
- 			       'mew-thread-indent (nth 2 indentlst))
- 	    (setq mew-thread-indent-list (cdr mew-thread-indent-list))))
+	 (mew-summary-thread-print-top
+	  (mew-vinfo-get-top) folder column)
 	 ;;
+	 ;; Unmarking in both Summary and Thread
 	 (when mark (mew-mark-undo-mark mew-mark-review))
 	 (setq tm6 (current-time))
 	 (when mew-gemacs-p
 	   (jit-lock-register 'mew-summary-cook-region))
+	 (mew-summary-set-count-line)
 	 (set-buffer-modified-p nil)
 	 (if disp-msg
-	     (mew-summary-jump-message disp-msg)
+	     (mew-summary-move-and-display disp-msg)
 	   (goto-char (point-max)))
-	 (mew-summary-display nil)
-	 (mew-summary-thread-move-cursor)
+	 (mew-thread-move-cursor)
 	 (message "Displaying thread...done")
 	 (run-hooks 'mew-thread-display-hook)
 	 (when (mew-debug 'thread)
@@ -393,62 +338,43 @@ threads are created, see 'mew-use-complete-thread'."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Visualizing thread
+;;; Subfunctions
 ;;;
 
-(defun mew-summary-thread-print (tree folder column)
-  (let ((tree-stack nil)
-	(level 0) pos)
-    (while tree
-      (let* ((me (car tree))
-	     (cld (mew-thread-get-child me)))
-	(mew-elet
-	 (if (= level 0) (mew-thread-insert-separator))
-	 (insert (mew-thread-get-line me))
-	 (forward-line -1)
-	 (move-to-column column)
- 	 (cond
- 	  ((= level 0)
- 	   (setq mew-thread-indent-list
- 		 (cons (list (point) (1+ (point)) 0) mew-thread-indent-list)))
- 	  ((< level mew-thread-indent-array-size)
- 	   (setq pos (point))
- 	   (insert (aref mew-thread-indent-array level))
- 	   (setq mew-thread-indent-list
- 		 (cons (list pos (point) level) mew-thread-indent-list)))
- 	  (t
- 	   (setq pos (point))
-	   (let* ((i 0)
-		  (max (1- mew-thread-indent-array-size)) ;; take care
-		  (lim (/ level max))
-		  (j (% level max)))
-	     (while (< i lim)
-	       (insert (aref mew-thread-indent-array max))
-	       (setq i (1+ i)))
- 	     (insert (aref mew-thread-indent-array j))
- 	     (setq mew-thread-indent-list
- 		   (cons (list pos (point) level) mew-thread-indent-list)))))
-	 (end-of-line)
-	 (insert " \006 " folder " " (mew-thread-get-msg me))
-	 (forward-line))
-	(setq tree (cdr tree))
-	(cond
-	 (cld
-	  (setq tree-stack (cons tree tree-stack))
-	  (setq tree cld)
-	  (setq level (1+ level)))
-	 (t
-	  (while (and (null tree) tree-stack)
-	    (setq tree (car tree-stack))
-	    (setq tree-stack (cdr tree-stack))
-	    (setq level (1- level)))))))))
+(defun mew-thread-put-property (beg end level)
+  (put-text-property beg end 'mew-thread-indent level))
+
+(defun mew-thread-get-property (beg)
+  (get-text-property beg 'mew-thread-indent))
+
+(defun mew-thread-previous-property (beg)
+  (previous-single-property-change beg 'mew-thread-indent))
+
+(defun mew-thread-next-property (beg)
+  (next-single-property-change beg 'mew-thread-indent))
+
+(defun mew-thread-next-property2 (beg end level)
+  (text-property-any beg end 'mew-thread-indent level))
+
+(defun mew-thread-adjust-body (level)
+  (when (mew-summary-goto-body)
+    (mew-elet
+     (let ((end (point))
+	   (width (* level mew-thread-indent-width))
+	   (sum 0))
+       (while (< sum width)
+	 (setq sum (+ (char-width (char-before)) sum))
+	 (forward-char -1))
+       (delete-region (point) end)
+       (when (/= sum width)
+	 (insert (make-string (- sum width) 32)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Visualizing thread (fancy style)
+;;; Visualizing thread
 ;;;
 
-(defun mew-summary-fancy-thread-print-top (top folder column)
+(defun mew-summary-thread-print-top (top folder column)
   (while top
     (let* ((me (car top))
 	   (cld (mew-thread-get-child me)))
@@ -457,15 +383,12 @@ threads are created, see 'mew-use-complete-thread'."
        (insert (mew-thread-get-line me))
        (forward-line -1)
        (move-to-column column)
-       (setq mew-thread-indent-list
- 	     (cons (list (point) (1+ (point)) 0) mew-thread-indent-list))
-       (end-of-line)
-       (insert " \006 " folder " " (mew-thread-get-msg me))
+       (mew-thread-put-property (point) (1+ (point)) 0)
        (forward-line))
-      (if cld (mew-summary-fancy-thread-print-tree cld folder column))
+      (if cld (mew-summary-thread-print-tree cld folder column))
       (setq top (cdr top)))))
 
-(defun mew-summary-fancy-thread-print-tree (tree folder column)
+(defun mew-summary-thread-print-tree (tree folder column)
   (let ((tree-stack nil)
 	(prefix "")
 	(level 1) pos)
@@ -479,21 +402,20 @@ threads are created, see 'mew-use-complete-thread'."
 	 (move-to-column column)
  	 (setq pos (point))
 	 (if next
-	     (insert prefix (aref mew-fancy-thread-indent-strings 0))
-	   (insert prefix (aref mew-fancy-thread-indent-strings 1)))
- 	 (setq mew-thread-indent-list
- 	       (cons (list pos (point) level) mew-thread-indent-list))
-	 (end-of-line)
-	 (insert " \006 " folder " " (mew-thread-get-msg me))
+	     (insert prefix (aref mew-thread-indent-strings 0))
+	   (insert prefix (aref mew-thread-indent-strings 1)))
+	 (mew-thread-put-property pos (point) level)
+	 (mew-thread-adjust-body level)
 	 (forward-line))
+	;;
 	(setq tree next)
 	(cond
 	 (cld
 	  (if next
 	      (setq prefix
-		    (concat prefix (aref mew-fancy-thread-indent-strings 2)))
+		    (concat prefix (aref mew-thread-indent-strings 2)))
 	    (setq prefix
-		  (concat prefix (aref mew-fancy-thread-indent-strings 3))))
+		  (concat prefix (aref mew-thread-indent-strings 3))))
 	  (setq tree-stack (cons tree tree-stack))
 	  (setq tree cld)
 	  (setq level (1+ level)))
@@ -534,10 +456,16 @@ threads are created, see 'mew-use-complete-thread'."
   "Put the 'o' mark on all messages of the current sub-thread."
   (interactive)
   (mew-thread-only
-   (let ((folders (mew-summary-refile-body nil nil nil 'no-mark))
-	 alist)
+   (let* ((fld (mew-thread-to-folder (mew-summary-folder-name 'ext)))
+	  (folders (mew-summary-refile-body nil nil nil 'no-mark))
+	  (folders-str (mew-join "," folders))
+	  (func (lambda ()
+		  (mew-summary-refile-override-body folders-str)
+		  (unless (mew-virtual-p)
+		    (mew-summary-refile-log fld folders-str))))
+	  alist)
      (when folders
-       (setq alist (mew-thread-mark mew-mark-refile))
+       (setq alist (mew-thread-mark mew-mark-refile 'valid-only func))
        (mew-refile-set-from-alist alist folders)))))
 
 (defun mew-thread-mark-copy ()
@@ -545,11 +473,13 @@ threads are created, see 'mew-use-complete-thread'."
 with the current folder as a candidate in addition to guessed folders."
   (interactive)
   (mew-thread-only
-   (let ((folders (mew-summary-refile-body
-		   nil nil nil 'no-mark (mew-summary-folder-name)))
-	 alist)
+   (let* ((folders (mew-summary-refile-body
+		    nil nil nil 'no-mark (mew-summary-folder-name)))
+	  (folders-str (mew-join "," folders))
+	  (func (lambda () (mew-summary-refile-override-body folders-str)))
+	  alist)
      (when folders
-       (setq alist (mew-thread-mark mew-mark-refile))
+       (setq alist (mew-thread-mark mew-mark-refile 'valid-only func))
        (mew-refile-set-from-alist alist folders)))))
 
 (defun mew-refile-set-from-alist (alist folders)
@@ -568,43 +498,41 @@ with the current folder as a candidate in addition to guessed folders."
 	    (mew-refile-reset msg)
 	    (mew-refile-set msg folders)))))))
 
-(defun mew-thread-mark (mark &optional valid-only)
+(defun mew-thread-mark (mark &optional valid-only func)
   (mew-thread-only
    (mew-summary-msg-or-part
-    (let ((column (mew-vinfo-get-column))
+    (let ((regex (if valid-only mew-regex-sumsyn-valid mew-regex-sumsyn-short))
+	  (column (mew-vinfo-get-column))
 	  indent cindent fld msg alist bottom)
       (mew-summary-goto-message)
-      (mew-summary-thread-move-cursor)
       (mew-decode-syntax-delete)
       (save-excursion
 	(move-to-column column)
-	(setq indent (get-text-property (point) 'mew-thread-indent))
-	(unless (mew-summary-msg-invalidp)
-	  (mew-summary-mark-as mark 'force)
-	  (setq fld (mew-summary-folder-name))
-	  (setq msg (mew-summary-message-number))
+	(setq indent (mew-thread-get-property (point)))
+	(when (mew-sumsyn-match regex)
+	  (setq fld (mew-sumsyn-folder-name))
+	  (setq msg (mew-sumsyn-message-number))
+	  (if func (funcall func))
+	  (mew-mark-put mark)
 	  (mew-mark-alist-set alist fld msg))
 	(forward-line)
 	(catch 'loop
 	  (while (not (eobp))
 	    (move-to-column column)
-	    (unless (or (mew-syntax-number) (mew-summary-msg-invalidp))
-	      (when (setq cindent
-			  (get-text-property (point) 'mew-thread-indent))
-		(if (>= indent cindent)
-		    (throw 'loop nil)
-		  (mew-summary-mark-as mark 'force)
-		  (setq fld (mew-summary-folder-name))
-		  (setq msg (mew-summary-message-number))
+	    (when (setq cindent	(mew-thread-get-property (point)))
+	      (if (>= indent cindent)
+		  (throw 'loop nil)
+		(when (mew-sumsyn-match regex)
+		  (setq fld (mew-sumsyn-folder-name))
+		  (setq msg (mew-sumsyn-message-number))
+		  (if func (funcall func))
+		  (mew-mark-put mark)
 		  (mew-mark-alist-set alist fld msg))))
 	    (forward-line)))
 	(beginning-of-line)
 	(setq bottom (point))
-	(mew-summary-mark-in-physical-alist alist mark))
-      (mew-summary-goto-message)
-      (mew-decode-syntax-delete)
+	(mew-summary-mark-in-physical-alist alist mark func))
       ;; for C-x C-x
-      (beginning-of-line)
       (let ((zmacs-regions nil)
 	    ;; To treat this thread as if it were a single message.
 	    (mew-summary-down-function (lambda () (goto-char bottom))))
@@ -612,10 +540,10 @@ with the current folder as a candidate in addition to guessed folders."
 	(mew-summary-display-after mew-summary-mark-direction))
       alist))))
 
-(defsubst mew-thread-unmark-one ()
-  (beginning-of-line)
-  (when (re-search-forward mew-regex-msg nil t)
-    (mew-mark-delete-here)))
+(defun mew-thread-undo (fld msg)
+  (let* ((mark (mew-summary-get-mark))
+	 (func (mew-markdb-func-undo mark)))
+    (and func (fboundp func) (funcall func fld msg))))
 
 (defun mew-thread-unmark ()
   "Unmark messages under this sub-thread."
@@ -625,27 +553,28 @@ with the current folder as a candidate in addition to guessed folders."
     (let ((column (mew-vinfo-get-column))
 	  fld msg alist indent cindent)
       (mew-summary-goto-message)
-      (mew-summary-thread-move-cursor)
+      (mew-thread-move-cursor)
       (mew-decode-syntax-delete)
       (save-excursion
 	(move-to-column column)
- 	(setq indent (get-text-property (point) 'mew-thread-indent))
+ 	(setq indent (mew-thread-get-property (point)))
 	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number))
 	(mew-mark-alist-set alist fld msg)
-	(mew-thread-unmark-one)
+	(mew-thread-undo fld msg)
+	(mew-mark-unmark)
 	(forward-line)
 	(catch 'loop
  	  (while (not (eobp))
 	    (move-to-column column)
-	    (when (setq cindent (get-text-property
-				 (point) 'mew-thread-indent))
+	    (when (setq cindent (mew-thread-get-property (point)))
 	      (if (>= indent cindent)
 		  (throw 'loop nil)
 		(setq fld (mew-summary-folder-name))
 		(setq msg (mew-summary-message-number))
 		(mew-mark-alist-set alist fld msg)
-		(mew-thread-unmark-one)))
+		(mew-thread-undo fld msg)
+		(mew-mark-unmark)))
 	    (forward-line)))
 	(mew-thread-unmark-physical-from-alist alist))))))
 
@@ -663,20 +592,16 @@ with the current folder as a candidate in addition to guessed folders."
 	  (while msgs
 	    (setq msg (int-to-string (car msgs)))
 	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	      (beginning-of-line)
-	      (when (eq mew-mark-refile
-			(and (looking-at mew-regex-msg-mark)
-			     (string-to-char (mew-match-string 2))))
-		(mew-refile-reset msg))
-	      (mew-thread-unmark-one))))))))
+	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	      (mew-thread-undo fld msg)
+	      (mew-mark-unmark))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Thread utilities
 ;;;
 
-(defun mew-summary-thread-up ()
+(defun mew-thread-up ()
   "Move onto the top of the current thread. If the current message is
 a top node, move onto the top of the previous thread."
   (interactive)
@@ -689,17 +614,16 @@ a top node, move onto the top of the previous thread."
        (setq pos (point))
        (catch 'loop
  	 (while (and (not (bobp))
- 		     (setq pos (previous-single-property-change
- 				pos 'mew-thread-indent)))
- 	   (when (and pos (eq (get-text-property pos 'mew-thread-indent) 0))
+		     (setq pos (mew-thread-previous-property pos)))
+	   (when (and pos (eq (mew-thread-get-property pos) 0))
  	     (throw 'loop (setq here pos))))))
      (if (not here)
 	 (message "No more threads")
        (goto-char here)
-       (mew-summary-thread-move-cursor)
+       (mew-thread-move-cursor)
        (mew-summary-display nil)))))
 
-(defun mew-summary-thread-down ()
+(defun mew-thread-down ()
   "Move onto the top of the next thread."
   (interactive)
   (mew-thread-only
@@ -708,13 +632,12 @@ a top node, move onto the top of the previous thread."
      (save-excursion
        (mew-decode-syntax-delete)
        (forward-line)
-       (setq here (text-property-any (point) (point-max)
- 				     'mew-thread-indent 0)))
+       (setq here (mew-thread-next-property2 (point) (point-max) 0)))
      (if (not here)
 	 (message "No more threads")
        (goto-char here)
-       (if (not (mew-summary-message-number)) (forward-line))
-       (mew-summary-thread-move-cursor)
+       (unless (mew-summary-message-number) (forward-line))
+       (mew-thread-move-cursor)
        (mew-summary-display nil)))))
 
 (defun mew-summary-thread-parent ()
@@ -727,13 +650,13 @@ a top node, move onto the top of the previous thread."
 	key)
     (if (or (null par-id) (string= par-id ""))
 	(message "No required info")
-      (setq key (format "\r <%s> " par-id))
-      (if (or (search-backward key nil t)
+      (setq key (mew-regex-sumsyn-my-id par-id))
+      (if (or (re-search-backward key nil t)
 	      (progn
 		(goto-char (point-max))
-		(search-backward key nil t)))
+		(re-search-backward key nil t)))
 	  (progn
-	    (mew-summary-thread-move-cursor)
+	    (mew-thread-move-cursor)
 	    (mew-summary-display nil)
 	    (message "Parent found"))
 	(goto-char pos)
@@ -749,13 +672,13 @@ a top node, move onto the top of the previous thread."
 	key)
     (if (or (null my-id) (string= my-id ""))
 	(message "No required info")
-      (setq key (format "\r <[^>]*> <%s>" (regexp-quote my-id)))
+      (setq key (mew-regex-sumsyn-par-id my-id))
       (if (or (re-search-forward key nil t)
 	      (progn
 		(goto-char (point-min))
 		(re-search-forward key nil t)))
 	  (progn
-	    (mew-summary-thread-move-cursor)
+	    (mew-thread-move-cursor)
 	    (mew-summary-display nil)
 	    (message "Child found"))
 	(goto-char pos)
@@ -769,10 +692,10 @@ a top node, move onto the top of the previous thread."
 	key)
     (if (or (null par-id) (string= par-id ""))
 	(message "No required info")
-      (setq key (format "\r <[^>]*> <%s>" (regexp-quote par-id)))
+      (setq key (mew-regex-sumsyn-par-id par-id))
       (if (re-search-backward key nil t)
 	  (progn
-	    (mew-summary-thread-move-cursor)
+	    (mew-thread-move-cursor)
 	    (mew-summary-display nil)
 	    (message "Sibling found"))
 	(goto-char pos)
@@ -786,11 +709,11 @@ a top node, move onto the top of the previous thread."
 	key)
     (if (or (null par-id) (string= par-id ""))
 	(message "No required info")
-      (setq key (format "\r <[^>]*> <%s>" (regexp-quote par-id)))
+      (setq key (mew-regex-sumsyn-par-id par-id))
       (forward-line)
       (if (re-search-forward key nil t)
 	  (progn
-	    (mew-summary-thread-move-cursor)
+	    (mew-thread-move-cursor)
 	    (mew-summary-display nil)
 	    (message "Sibling found"))
 	(goto-char pos)
@@ -800,16 +723,17 @@ a top node, move onto the top of the previous thread."
 ;;;
 ;;; Thread sub-functions
 ;;;
-(defun mew-summary-thread-move-cursor ()
+
+(defun mew-thread-move-cursor ()
   "Move cursor after indentation of thread."
   (if (and mew-use-thread-cursor
 	   (mew-thread-p)
 	   (mew-summary-message-number))
       (let (indent)
  	(move-to-column (mew-vinfo-get-column))
- 	(if (setq indent (get-text-property (point) 'mew-thread-indent))
+ 	(if (setq indent (mew-thread-get-property (point)))
  	    (unless (= indent 0)
- 	      (goto-char (next-single-property-change (point) 'mew-thread-indent)))
+ 	      (goto-char (mew-thread-next-property (point))))
  	  (beginning-of-line)))
     (beginning-of-line)))
 
@@ -836,11 +760,230 @@ a top node, move onto the top of the previous thread."
 	(cdr (nreverse msgs))
       (nreverse msgs))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Hide/disclose children
+;;;
+
+(defvar mew-mark-thread-root ?+)
+
+(defun mew-thread-toggle ()
+  (interactive)
+  (mew-thread-only
+   (mew-summary-goto-message)
+   (mew-decode-syntax-delete)
+   (if (looking-at (concat "^." (regexp-quote (char-to-string mew-mark-thread-root))))
+       (mew-thread-graft)
+     (mew-thread-prune))
+   (mew-thread-move-cursor)
+   (set-buffer-modified-p nil)))
+
+(defun mew-thread-prune ()
+  (beginning-of-line)
+  (let ((pos (mew-thread-next-property (point))))
+    (unless (and pos (eq (mew-thread-get-property pos) 0)) ;; root
+      (catch 'loop
+	(while (setq pos (mew-thread-previous-property pos))
+	  (when (and pos (eq (mew-thread-get-property pos) 0))
+	    (throw 'loop (goto-char pos)))))))
+  (beginning-of-line)
+  (save-excursion
+    (forward-line)
+    (let ((beg (point))
+	  (next (mew-thread-next-property2 (point) (point-max) 0)))
+      (goto-char (or next (point-max)))
+      (forward-line -1)
+      (if (mew-summary-message-number) (forward-line))
+      (if (= beg (point))
+	  (message "No children to be pruned")
+	(mew-elet
+	 (put-text-property beg (point) 'invisible t)
+	 (goto-char beg)
+	 (forward-line -1)
+	 (forward-char)
+	 (put-text-property (point) (1+ (point)) 'invisible t)
+	 (insert mew-mark-thread-root))))))
+
+(defun mew-thread-graft ()
+  (save-excursion
+    (forward-line)
+    (let ((start (point))
+	  (next (mew-thread-next-property2 (point) (point-max) 0))
+	  beg end)
+      (goto-char (or next (point-max)))
+      (forward-line -1)
+      (if (mew-summary-message-number) (forward-line))
+      (setq end (point))
+      (if (= start end)
+	  (message "No children to be leaned")
+	(mew-elet
+	 (goto-char start)
+	 (setq beg start)
+	 (while (search-forward "\r" end t)
+	   (put-text-property beg (1- (point)) 'invisible nil)
+	   (forward-line)
+	   (put-text-property (1- (point)) (point) 'invisible nil)
+	   (setq beg (point)))
+	 (goto-char start)
+	 (forward-line -1)
+	 (forward-char)
+	 (delete-char 1)
+	 (put-text-property (point) (1+ (point)) 'invisible nil))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Thread editing
+;;;
+
+(defun mew-thread-glue ()
+  (interactive)
+  (mew-summary-msg
+   (let ((pos (marker-position (mark-marker))))
+     (cond
+      ((null pos)
+       (message "No marker"))
+      ((mew-thread-p)
+       (mew-thread-glue-it))
+      (t ;; summary or virtual
+       (mew-summary-glue-it))))))
+
+(defun mew-summary-glue-it ()
+  (save-excursion
+    (mew-summary-goto-message)
+    (beginning-of-line)
+    ;; parent    
+    (let ((id (mew-summary-my-id))
+	  fld msg)
+      (goto-char (marker-position (mark-marker)));; user's mark
+      ;; children
+      (when (mew-sumsyn-match mew-regex-sumsyn-long)
+	(setq fld (mew-sumsyn-folder-name))
+	(setq msg (mew-sumsyn-message-number)))
+      (when (and fld msg id)
+	(mew-thread-change-parent-id id)
+	(mew-thread-save-xref fld msg id)
+	(if (mew-virtual-p)
+	    (mew-summary-change-parent-id fld msg id)))
+      (sit-for 0.1))))
+
+(defun mew-thread-glue-it ()
+  (let ((column (mew-vinfo-get-column))
+	(m (make-marker))
+	(width 0) (wd 0) (adjust 0)
+	(prefix "")
+	fld msg id beg end tree indent idt pbeg pindent has-child)
+    (save-excursion
+      ;; parent
+      (mew-summary-goto-message)
+      (beginning-of-line)
+      (setq id (mew-summary-my-id))
+      (move-to-column column)
+      (setq beg (point))
+      (setq pindent (mew-thread-get-property (point)))
+      (goto-char (mew-thread-next-property (point)))
+      (setq end (point))
+      (unless (= pindent 0)
+	(while (< width mew-thread-indent-width)
+	  (forward-char -1)
+	  (setq width (+ width (char-width (char-after)))))
+	(setq prefix (mew-buffer-substring beg (point)))
+	(if (string= (mew-buffer-substring (point) end)
+		     (aref mew-thread-indent-strings 0))
+	    (setq prefix (concat prefix (aref mew-thread-indent-strings 2)))
+	  (setq prefix (concat prefix (aref mew-thread-indent-strings 3)))))
+      (when (mew-summary-goto-body)
+	(while (> (point) beg)
+	  (setq wd (+ wd (char-width (char-before))))
+	  (forward-char -1)))
+      (setq wd (/ wd mew-thread-indent-width))
+      (if (> pindent wd) (setq adjust (- pindent wd)))
+      (setq pindent (1+ pindent))
+      (forward-line)
+      ;; the next line of parent
+      (set-marker m (point))
+      (unless (looking-at mew-regex-thread-separator)
+	(move-to-column column)
+	(if (= (mew-thread-get-property (point)) pindent)
+	    (setq has-child t)))
+      (move-to-column column)
+      ;; children
+      (goto-char (marker-position (mark-marker)));; user's mark
+      (when (mew-sumsyn-match mew-regex-sumsyn-long)
+	(setq fld (mew-sumsyn-folder-name))
+	(setq msg (mew-sumsyn-message-number)))
+      (when (and fld msg id)
+	(mew-elet
+	 (mew-syntax-change-parent-id id)
+	 (beginning-of-line)
+	 (setq beg (point))
+	 (move-to-column column)
+	 (setq pbeg (point))
+	 (setq indent (mew-thread-get-property (point)))
+	 (insert prefix)
+	 (if has-child
+	     (insert (aref mew-thread-indent-strings 0))
+	   (insert (aref mew-thread-indent-strings 1)))
+	 (goto-char (mew-thread-next-property (point)))
+	 (mew-thread-put-property pbeg (point) (+ indent pindent))
+	 (mew-thread-adjust-body (- pindent adjust))
+	 (catch 'loop
+	   (while t
+	     (forward-line)
+	     (move-to-column column)
+	     (setq pbeg (point))
+	     (setq idt (mew-thread-get-property (point)))
+	     (if (or (null idt) (<= idt indent))
+		 (throw 'loop nil))
+	     (insert prefix)
+	     (if has-child
+		 (insert (aref mew-thread-indent-strings 2))
+	       (insert (aref mew-thread-indent-strings 3)))
+	     (goto-char (mew-thread-next-property (point)))
+	     (mew-thread-put-property pbeg (point) (+ idt pindent))
+	     (mew-thread-adjust-body (- pindent adjust))))
+	 (beginning-of-line)
+	 (setq end (point))
+	 (when (looking-at mew-regex-thread-separator)
+	   (forward-line)
+	   (delete-region end (point)))
+	 ;; This must be "buffer-substring".
+	 (setq tree (buffer-substring beg end))
+	 (delete-region beg end)
+	 ;; the next line of parent
+	 (goto-char (marker-position m))
+	 (insert tree)
+	 (set-buffer-modified-p nil))
+	(mew-summary-change-parent-id fld msg id)
+	(mew-thread-save-xref fld msg id)))))
+
+(defun mew-summary-change-parent-id (fld msg id)
+  (set-buffer fld)
+  (save-excursion
+    (goto-char (point-min))
+    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+      (mew-thread-change-parent-id id))))
+
+(defun mew-thread-change-parent-id (id)
+  (mew-elet
+   (mew-syntax-change-parent-id id))
+  (unless (mew-virtual-p)
+    (mew-summary-folder-cache-save))
+  (set-buffer-modified-p nil))
+
+(defun mew-thread-save-xref (fld msg id)
+  (with-temp-buffer
+    (let ((file (mew-expand-folder fld msg)))
+      (insert-file-contents file)
+      (mew-header-delete-lines (list mew-x-mew-ref:))
+      (goto-char (point-min))
+      (mew-header-insert mew-x-mew-ref: id)
+      (write-region (point-min) (point-max) file nil 'no-msg))))
+
 (provide 'mew-thread)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2002 Mew developing team.
+;; Copyright (C) 2000-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
