@@ -357,11 +357,6 @@ Note that this hook is called after composition of the message."
   :group 'mew-summary
   :type 'hook)
 
-(defcustom mew-pack-hook nil
-  "*Hook called when pack finished." 
-  :group 'mew-summary
-  :type 'hook)
-
 (defcustom mew-summary-toggle-disp-msg-hook nil
   "*Hook called when mew-summary-toggle-disp-msg finished."
   :group 'mew-summary
@@ -437,13 +432,55 @@ i.e. \"Friendly name <user@mail-domain>\"."
   :group 'mew-basic
   :type 'string)
 
+
+(defvar mew-reply-all-alist
+  '((("Followup-To:" "poster")
+     ("To:" "From:"))
+    ("Followup-To:"
+     ("Newsgroups:" "Followup-To:" "Newsgroups:"))
+    ("Newsgroups:"
+     ("Newsgroups:" "Newsgroups:"))
+    ("Reply-To:"
+     ("To:" "Reply-To:" "From:")
+     ("Cc:" "To:" "Cc:" "Apparently-To:"))
+    (t
+     ("To:" "From:")
+     ("Cc:" "To:" "Cc:" "Apparently-To:")))
+  "*Alist to be used to prepare To:/Cc:/Newsgroups: in a reply draft.
+For most cases, this alist is used.
+For more infomation, see the document of '\\<mew-summary-mode-map>\\[mew-summary-reply]'")
+
+(defvar mew-reply-sender-alist
+  '(("Reply-To:"
+     ("To:" "Reply-To:" "From:"))
+    (t
+     ("To:" "From:")))
+  "*Alist to be used to prepare To:/Cc:/Newsgroups: in a reply draft.
+When '\\[universal-argument]' is specified for '\\<mew-summary-mode-map>\\[mew-summary-reply]' and '\\[mew-summary-reply-with-citation]', this alist is used.
+For more infomation, see the document of '\\[mew-summary-reply]'")
+
+(defvar mew-reply-fromme-alist
+  '((t
+     ("To:" "To:")
+     ("Cc:" "Cc:")
+     ("Newsgroups:" "Newsgroups:")))
+  "*Alist to be used to prepare To:/Cc:/Newsgroups: in a reply draft.
+When the message to be replied is sent/posted by ME, this alist is used.
+For more infomation, see the document of '\\<mew-summary-mode-map>\\[mew-summary-reply]'")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; SSH
+;;; SSH/SSL
 ;;;
 
 (defcustom mew-ssh-keep-connection t
-  "*If non-nil, a SSH process for each service (POP, IMAP, NNTP, SMTP)
+  "*If non-nil, an SSH process for each service (POP, IMAP, NNTP, SMTP)
+is kept for further connections. This must be 't' for IMAP and NNTP."
+  :group 'mew-net
+  :type 'boolean)
+
+(defcustom mew-ssl-keep-connection t
+  "*If non-nil, an SSL process for each service (POP, IMAP, NNTP, SMTP)
 is kept for further connections. This must be 't' for IMAP and NNTP."
   :group 'mew-net
   :type 'boolean)
@@ -504,14 +541,14 @@ If nil, an address specified by the From: field is used."
   :type '(choice string (const nil)))
 
 (defcustom mew-smtp-msgid-user nil
-  "*If non-nil, this value is used to generate a user part
-of Message-ID:."
+  "*A user part for creation of Message-Id: for SMTP. If nil, the
+value of 'mew-user' is used."
   :group 'mew-smtp
   :type '(choice string (const nil)))
 
 (defcustom mew-smtp-msgid-domain nil
-  "*A domain name for creation of Message-Id:. If nil, the value of
-'mew-mail-domain' is used."
+  "*A domain name for creation of Message-Id: for SMTP. If nil, the
+value of 'mew-mail-domain' is used."
   :group 'mew-smtp
   :type '(choice string (const nil)))
 
@@ -533,7 +570,7 @@ of Message-ID:."
 ;;;
 
 (defcustom mew-mbox-command "incm"
-  "*A command to be execute if 'mew-mailbox-type' is 'mbox. "
+  "*A command to be execute if 'mew-mailbox-type' is 'mbox."
   :group 'mew-pop
   :type '(choice string (const nil)))
 
@@ -620,7 +657,7 @@ Otherwise they are not deleted."
 (defcustom mew-pop-size (* 54 1024) ;; 4K hdr + 50K bdy
   "*The limit size of messages to be retrieved by POP. The default is
 55296 byte. 0 means unlimited, so you can get all messages from the
-POP server. "
+POP server."
   :group 'mew-pop
   :type 'integer)
 
@@ -699,7 +736,7 @@ Otherwise they are not deleted."
 (defcustom mew-imap-size (* 54 1024) ;; 4K hdr + 50K bdy
   "*The limit size of messages to be retrieved by IMAP. The default is
 55296 byte. 0 means unlimited, so you can get all messages from the
-IMAP server. "
+IMAP server."
   :group 'mew-imap
   :type 'integer)
 
@@ -757,7 +794,7 @@ used."
 (defcustom mew-nntp-size (* 54 1024) ;; 4K hdr + 50K bdy
   "*The limit size of messages to be retrieved by NNTP. The default is
 55296 byte. 0 means unlimited, so you can get all messages from the
-NNTP server. "
+NNTP server."
   :group 'mew-nntp
   :type 'integer)
 
@@ -767,21 +804,20 @@ If non-nil, only header is retrieved."
   :group 'mew-nntp
   :type 'boolean)
 
+(defcustom mew-nntp-msgid-user nil
+  "*A user part for creation of Message-Id: for NNTP. If nil, the
+value of 'mew-user' is used."
+  :group 'mew-nntp
+  :type '(choice string (const nil)))
+
+(defcustom mew-nntp-msgid-domain nil
+  "*A domain name for creation of Message-Id: for NNTP. If nil, the
+value of 'mew-mail-domain' is used."
+  :group 'mew-nntp
+  :type '(choice string (const nil)))
+
 (defvar mew-nntp-timeout-time 4)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Folder spec
-;;;
-
-(defvar mew-folder-spec-type
-  '((regex     . string-match)
-    (recursive . mew-folder-recursive-match)
-    (string    . string=)))
-
-(defsubst mew-folder-spec-func (type)
-  (or (cdr (assq type mew-folder-spec-type)) 'string=))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; range
@@ -843,17 +879,13 @@ If this value is nil, an appropriate value is set when Mew is booting.
 ;;; Scan
 ;;;
 
-(defcustom mew-scan-form-header '((-5 num) mark)
-  "*A list to be appended to scan-form if the first element of
-scan-form is not an integer. Since Mew assumes that
-each line of Summary mode begins with the number of message followed
-by one mark, this variable MUST be consist of 'num and 'mark."
-  :group 'mew-summary
-  :type '(repeat (choice string
-                        (list integer (choice (const num) (const mark)))
-                        (choice (const num) (const mark)))))
+(defconst mew-summary-form-header '(mark)
+  "*A list to be appended to summary-form if the first element of
+summary-form is not an integer. Since Mew assumes that
+each line of Summary mode begins with one mark, this variable 
+MUST be '(mark).")
 
-(defconst mew-custom-type-of-scan-form
+(defconst mew-custom-type-of-summary-form
   '(choice
     (cons :tag "Setup message number column"
 	  (integer :tag "Message number column" )
@@ -894,20 +926,24 @@ by one mark, this variable MUST be consist of 'num and 'mark."
 			    (const size) (const from)
 			    (const subj) (symbol :tag "Other"))))))
 
-(defcustom mew-scan-form '(type (5 date) " " (14 from) " " t (0 subj))
-  "*The format in Summary mode, called scan-form. Scan-form is a list
+(defvar mew-summary-form-body-starter "|")
+
+(defcustom mew-summary-form
+  `(type (5 date) " " (14 from) " " t (30 subj)
+	 ,mew-summary-form-body-starter (0 body))
+  "*The format in Summary mode, called summary-form. summary-form is a list
 of list, symbol, and string.
 
 (a) A string is printed as it is.
 
 (b) A list consists of an integer and a symbol. The symbol specifies a
 function to be called. The name of the function is produced by
-concatenating 'mew-scan-form-func-prefix' and the symbol name.
-(e.g. 'mew-scan-form-date' for the symbol 'date'). 
+concatenating 'mew-summary-form-func-prefix' and the symbol name.
+(e.g. 'mew-summary-form-date' for the symbol 'date'). 
 
-Pre-defined symbols are 'type, 'time, 'date, 'year, 'size, 'from, and
-'subj. For more details, see the explanation of the functions called
-'mew-scan-form-<symbol>'.
+Pre-defined symbols are 'type, 'time, 'date, 'year, 'size, 'from,
+'subj and 'body. For more details, see the explanation of the
+functions called 'mew-summary-form-<symbol>'.
 
 The integer specifies the width of field which will be filled with a
 return string of the function.
@@ -922,38 +958,35 @@ ASCII string.
 
 (c) A symbol except 't' is equivalent to (1 symbol).
 
-(d) If the first element of scan-form is an integer N, this integer is
-converted into '((N num) mark). Otherwise, the value of
-mew-scan-form-header is inserted in front of scan-form value when used
-as scan format. Also note that users MUST not use the symbol 'num and
-'mark in 'mew-scan-form'. See also 'mew-scan-form-header'.
+(d) The value of mew-summary-form-header (mark) is appended to
+summary-form when used. See also 'mew-summary-form-header'.
 
-(e) 't' means the position where 'mew-thread-indent-string' is inserted.
+(e) 't' means the position where thread indentation is inserted.
 
 An example is as follows:
-	'(type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (0 subj))
+	'(type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (30 subj) \"|\" (0 body))
 
-You can also set this value in 'mew-scan-form-list'."
+You can also set this value in 'mew-summary-form-list'."
   :group 'mew-summary
-  :type mew-custom-type-of-scan-form)
+  :type mew-custom-type-of-summary-form)
 
-(defcustom mew-scan-form-list nil
-  "*A list to define 'mew-scan-form' for each folder. Each component
-is (key scan-form). 
+(defcustom mew-summary-form-list nil
+  "*A list to define 'mew-summary-form' for each folder. Each component
+is (key summary-form). 
 
-If KEY is t, all folders matches it. The corresponding SCAN-FORM is
+If KEY is t, all folders matches it. The corresponding SUMMARY-FORM is
 returned always.
 
 If KEY is a string, match-function is applied according to
-'mew-scan-form-list-string-type'. If matched, the corresponding
-SCAN-FORM is returned.
+'mew-summary-form-list-string-type'. If matched, the corresponding
+SUMMARY-FORM is returned.
 
 If KEY is a list of strings, match-function is applied according to
-'mew-scan-form-list-list-type'. If a match found out of the strings,
-the corresponding SCAN-FORM is returned.
+'mew-summary-form-list-list-type'. If a match found out of the strings,
+the corresponding SUMMARY-FORM is returned.
 
-Candidate values for 'mew-scan-form-list-string-type' and
-'mew-scan-form-list-list-type' are as follows:
+Candidate values for 'mew-summary-form-list-string-type' and
+'mew-summary-form-list-list-type' are as follows:
 
 	'regex		Regular expression.
 	'recursive	Initial substring match. That is, 
@@ -961,13 +994,13 @@ Candidate values for 'mew-scan-form-list-string-type' and
 			all sub-folders are matched.
 	'string		String comparison.
 
-SCAN-FORM is a list. See 'mew-scan-form' for more information.
+SUMMARY-FORM is a list. See 'mew-summary-form' for more information.
 
 An example is as follows:
       '(((\"+inbox\")
-	 (type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (0 subj)))
+	 (type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (30 subj) \"|\" (0 body)))
 	(t
-	 (type (5 date) \" \" (14 from) \" \" t (0 subj))))
+	 (type (5 date) \" \" (14 from) \" \" t (30 subj) \"|\" (0 body))))
 "
   :group 'mew-summary
   :type `(choice
@@ -976,29 +1009,29 @@ An example is as follows:
 	   (list (choice (string :tag "Folder")
 			 (repeat :tag "Folders" (string :tag "Folder"))
 			 (const :tag "Other" t))
-		 ,mew-custom-type-of-scan-form
+		 ,mew-custom-type-of-summary-form
 		 (choice (integer :tag "Thread indent column")
 			 (const nil))))))
 
-(defcustom mew-scan-form-list-string-type 'regex
+(defcustom mew-summary-form-list-string-type 'regex
   "*A value to specify an action if the key is a string in 
-'mew-scan-form-list'. Candidates are 'regex, 'recursive, and 'string."
+'mew-summary-form-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
-(defcustom mew-scan-form-list-list-type 'recursive
+(defcustom mew-summary-form-list-list-type 'recursive
   "*A value to specify an action if the key is a list of strings in 
-'mew-scan-form-list'. Candidates are 'regex, 'recursive, and 'string."
+'mew-summary-form-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
-(defcustom mew-scan-form-from-me-prefix "To:"
+(defcustom mew-summary-form-from-me-prefix "To:"
   "*The prefix to be prepend to an destination address
 if the message is originated by me."
   :group 'mew-summary
   :type 'string)
 
-(defcustom mew-scan-form-extract-rule '(nickname)
+(defcustom mew-summary-form-extract-rule '(nickname)
   "*A list to specify what part to extract from the From: field. Each
 element must be 'name, 'comment, 'address, 'nickname, or appropriate
 \"regex\".
@@ -1040,60 +1073,71 @@ of the From: field is used."
   :type '(repeat (choice regexp (const name) (const comment)
                          (const address) (const nickname))))
 
-(defcustom mew-scan-form-mark-delete nil
+(defcustom mew-summary-form-mark-delete nil
   "*If non-nil, the 'D' mark automatically is put onto
-duplicated messages."
+duplicated messages. If a character, use it as a mark
+instead of the 'D' mark."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-scan-form-mark-review nil
+(defcustom mew-summary-form-mark-review nil
   "*If non-nil, the '*' mark automatically is put onto
-messages destined to me."
+messages destined to me. If a character, use it as a mark
+instead of the '*' mark."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-scan-form-size-0k nil
+(defcustom mew-summary-form-size-0k nil
   "*If non-nil, the size of message is displayed as '0k'
 if the size is less than 1k byte."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-scan-form-size-huge t
+(defcustom mew-summary-form-size-huge t
   "*If non-nil, the size of message is displayed as 'HUGE'
 when the size is greater than or equal to 1000 after maximum
-quantization by `mew-scan-form-size-unit'."
+quantization by `mew-summary-form-size-unit'."
   :group 'mew-summary
   :type 'boolean)
 
+(defvar mew-spam: "X-Spam-Flag:"
+  "*The field name which your spam filter inserts. 
+E.g.
+	\"X-Spam-Flag:\" for 'spamassassin'
+	\"X-Bogosity:\" for 'bogofilter'")
+
+
 (defcustom mew-scan-fields
-  '("Folder:" "Filename:" "Subject:" "Date:" "From:" "To:"
-    "Content-Type:" "Message-Id:" "X-Mew-UIDL:"
-    "In-Reply-To:" "References:")
-  "*A list which specifies mewls's output.
+  `("Folder:" "Filename:" 
+    ,mew-subj: ,mew-date: ,mew-from: ,mew-to: ,mew-cc:
+    ,mew-ct: ,mew-cte: ,mew-x-mew-uidl:
+    ,mew-message-id: ,mew-in-reply-to: ,mew-references: ,mew-x-mew-ref:
+    ,mew-spam: "Body")
+  "*A list which specifies mewl's output.
 The first element MUST be \"Folder:\".
 The second element MUST be \"Filename:\".
-Both \"In-Reply-To:\" and \"References:\" MUST be included for thread."
+Both \"In-Reply-To:\" and \"References:\" MUST be included for thread.
+The last MUST be \"Body\"."
   :group 'mew-summary
   :type '(cons (const "Folder:")
                (cons (const "Filename:")
                      (repeat string))))
 
 (defvar mew-scan-fields-alias
-  '("FLD" "NUM" "SUBJ" "DATE" "FROM" "TO" "CT" "ID" "UID" "IRT" "REF")
+  '("FLD" "NUM" "SUBJ" "DATE" "FROM" "TO" "CC"
+    "CT" "CTE" "UID"
+    "ID" "IRT" "REF" "XREF"
+    "SPAM" "BODY")
   "*A list of aliases for 'mew-scan-fields'.
 Functions called MEW-FOO will be defined according to this variable.")
 
-(defvar mew-scan-form-func-prefix "mew-scan-form-"
-  "The prefix for symbol used in 'mew-scan-form'.")
-
-(defvar mew-scan-wait-for 10000
-  "*Micro second for mewls to sleep so that the interaction
-between Emacs and mewls smoother.")
+(defvar mew-summary-form-func-prefix "mew-summary-form-"
+  "The prefix for symbol used in 'mew-summary-form'.")
 
 (defcustom mew-thread-column 28
-  "A position where 'mew-thread-indent-string' is inserted
+  "A position where thread indentation is inserted
 for indentation of thread. This valuable is for backward compatibility
-only. Use 't' in 'mew-scan-form instead."
+only. Use 't' in 'mew-summary-form instead."
   :group 'mew-summary
   :type 'integer)
 
@@ -1176,7 +1220,12 @@ to this file."
   :type 'integer)
 
 (defcustom mew-scan-max-field-length 5
-  "*Max field length which 'mewls' treats."
+  "*Max field length which 'mewl' treats."
+  :group 'mew-env
+  :type 'integer)
+
+(defcustom mew-scan-max-body-length 20
+  "*Max body length which 'mewl' treats."
   :group 'mew-env
   :type 'integer)
 
@@ -1258,6 +1307,15 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-ask-charset nil
+  "*Mew asks a user whether or not the charset chosen on composing
+is appropriate.
+	nil:               not ask
+	A list of charset: ask if the charset is not a member of the list
+	t:                 ask if 'mew-charset-m17n' is used."
+  :group 'mew-draft
+  :type '(choice (const nil) (const t) (repeat string)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Use
@@ -1270,7 +1328,7 @@ Otherwise, windows are configured dynamically."
   :type 'boolean)
 
 (defcustom mew-use-other-frame-for-draft nil
-  "*If non-nil, a new frame is created when a draft is prepared. "
+  "*If non-nil, a new frame is created when a draft is prepared."
   :group 'mew-env
   :type 'boolean)
 
@@ -1330,7 +1388,7 @@ every 'mew-biff-interval' minutes.
 To use this feature, 'mew-use-cached-passwd' should be 't'.
 
 If you want to use the biff functionality against a local mailbox, use
-'display-time-mode' instead. "
+'display-time-mode' instead."
   :group 'mew-env
   :type 'boolean)
 
@@ -1395,6 +1453,12 @@ Typing '%' automatically inserts 'mew-case-input' before '%'."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-use-spc-padding t
+  "*If non-nil SPCs are padded from the end of lines of Summary mode 
+to the right side."
+  :group 'mew-summary
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Appearance
@@ -1477,8 +1541,8 @@ displayed."
   :group 'mew-summary
   :type '(choice integer (const nil)))
 
-(defcustom mew-summary-jump-message-then-display t
-  "*If non-nil 'mew-summary-jump-message' displays the message
+(defcustom mew-summary-goto-line-then-display t
+  "*If non-nil 'mew-summary-goto-line' displays the message
 where the cursor jumped."
   :group 'mew-summary
   :type 'boolean)
@@ -1736,6 +1800,25 @@ an appropriate charset cannot be chosen. Possible candidates are
   :group 'mew-draft
   :type '(choice (const "utf-8") (const "iso-2022-jp-2")))
 
+(defcustom mew-charset-latin "iso-8859-15"
+  "*This variable is used for selection of charset when composing.
+If characters of both Latin 1 and Lain 9 exist in a draft, Mew takes
+the following step to decide a charset.
+
+1. If 'unify-8859-on-decoding-mode' is used:
+	1.1. Use \"iso-8859-1\" if no loss.
+	1.2. Otherwise, use \"utf-8\".
+2. If 'unify-8859-on-decoding-mode' is not used:
+
+	2.1. If both ISO-8859-1 and ISO-8859-15 can be used with no
+             loss, 'mew-charset-latin' is used.
+	2.2. Use \"iso-8859-1\" if no loss.
+	2.3. Use \"iso-8859-15\" if no loss.
+	2.3. Otherwise, use \"utf-8\".
+"
+  :group 'mew-draft
+  :type '(choice (const "iso-8859-1") (const "iso-8859-15")))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Signature
@@ -1766,42 +1849,6 @@ as the final part."
   :group 'mew-draft
   :type 'string)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Temporary solution to decide To: when reply
-;;;
-
-(defcustom mew-noreplyto-to-list `(,mew-from:)
-  "*To: field list picked at mew-summary-reply if Reply-To: does not exist."
-  :group 'mew-reply
-  :type '(repeat string))
-
-(defcustom mew-noreplyto-cc-list `(,mew-to: ,mew-cc: ,mew-apparently-to:)
-  "Cc: field list picked at mew-summary-reply if Reply-To: does not exist."
-  :group 'mew-reply
-  :type '(repeat string))
-
-(defcustom mew-replyto-to-list `(,mew-reply-to: ,mew-from:)
-  "*To: field list picked at mew-summary-reply if Reply-To: exists."
-  :group 'mew-reply
-  :type '(repeat string))
-
-(defcustom mew-replyto-cc-list `(,mew-to: ,mew-cc: ,mew-apparently-to:)
-  "*Cc: field list picked at mew-summary-reply if Reply-To: exists."
-  :group 'mew-reply
-  :type '(repeat string))
-
-(defcustom mew-fromme-to-list `(,mew-to: ,mew-apparently-to:)
-  "*To: field list picked at mew-summary-reply if From: is me."
-  :group 'mew-reply
-  :type '(repeat string))
-
-(defcustom mew-fromme-cc-list `(,mew-cc:)
-  "*Cc: field list picked at mew-summary-reply if From: is me."
-  :group 'mew-reply
-  :type '(repeat string))
-
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Samba
@@ -1933,13 +1980,19 @@ circular domain completion."
 
 (defcustom mew-field-delete-for-saving ()
   "*A field list to be deleted when saving a message to a file."
-  :group 'mew-draft
+  :group 'mew-summary
   :type '(repeat string))
 
 (defcustom mew-field-delete-for-joining
   `(,mew-mv: ,mew-subj: ,mew-message-id: "Encrypted" "Content-")
   "*A field list to be deleted when joining."
-  :group 'mew-draft
+  :group 'mew-summary
+  :type '(repeat string))
+
+(defcustom mew-field-for-printing
+  '("Subject:" "From:" "To:" "Cc:" "Date:")
+  "*A field list to print"
+  :group 'mew-summary
   :type '(repeat string))
   
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1988,20 +2041,13 @@ on '\\<mew-summary-mode-map>\\[mew-summary-save]'. Otherwise overwrote."
 ;;; Folders
 ;;;
 
-(defcustom mew-touch-folder-p nil
-  "*If non-nil, 'mew-summary-touch-file' is used to store
-time-stamp of a directory"
-  :group 'mew-env
-  :type 'boolean)
-
-(defvar mew-touch-folder-check-enabled-p t
-  "*If non-nil, 'mew-touch-folder-check' is enabled.")
-
-(defvar mew-regex-folder-candidate "^[^.0-9]\\|^[0-9].*[^0-9]"
+(defcustom mew-regex-folder-candidate "^[^.0-9]\\|^[0-9].*[^0-9]"
   "*Regular expression used in 'mew-dir-list-with-link-count'
 and 'mew-dir-list-without-link-count' to list up folder
 candidates. The default value \"^[^.0-9]\\|^[0-9].*[^0-9]\" lists up 
-folders whose name is alphabet only. ")
+folders whose name is alphabet only."
+  :group 'mew-env
+  :type 'regexp)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -2161,9 +2207,11 @@ otherwise  : considered as 'totrash."
 ;;;
 
 (defcustom mew-temp-file-initial
-  (expand-file-name (user-login-name) (or (getenv "TMP")
-					  (getenv "TEMP")
-					  "/tmp"))
+  (expand-file-name (user-login-name) (if (fboundp 'temp-directory)
+					  (temp-directory)
+					(if (boundp 'temporary-file-directory)
+					    temporary-file-directory
+					  "/tmp")))
   "*Hint to make a secure directory on the local file system. On
 setup phase Mew make a secure directory from this variable and set
 mew-temp-file a file name prefix contained the directory name. The
@@ -2319,8 +2367,13 @@ in Summary/Virtual mode."
   :group 'mew-highlight
   :type 'boolean)
 
-(defcustom mew-use-highlight-x-face mew-xemacs-p
-  "*Iconify X-Face: on XEmacs in Message mode."
+(defcustom mew-use-highlight-x-face mew-icon-p
+  "*Iconify X-Face: in Message mode."
+  :group 'mew-highlight
+  :type 'boolean)
+
+(defcustom mew-use-highlight-x-face-inversion nil
+  "*Invert an X-Face: image in Message mode."
   :group 'mew-highlight
   :type 'boolean)
 
@@ -2330,7 +2383,7 @@ in Summary/Virtual mode."
 
 (defcustom mew-cursor-mark ">"
   "*The mark in the beginning of the cursor line if
-'mew-use-cursor-mark' is *non-nil*. "
+'mew-use-cursor-mark' is *non-nil*."
   :group 'mew-highlight
   :type '(choice string (const nil)))
 
@@ -2405,8 +2458,8 @@ with Addrbook. See 'mew-addrbook-switch'."
   :type 'symbol)
 
 (defcustom mew-addrbook-for-summary 'nickname
-  "*How to replace a From: address in scan form with Addrbook.
-See 'mew-addrbook-switch' and 'mew-scan-form-extract-rule'."
+  "*How to replace a From: address in summary-form with Addrbook.
+See 'mew-addrbook-switch' and 'mew-summary-form-extract-rule'."
   :group 'mew-addrbook
   :type 'symbol)
 
@@ -2416,6 +2469,8 @@ when '\\<mew-summary-mode-map>\\[mew-summary-addrbook-add]' in Summary mode."
   :group 'mew-addrbook
   :type 'boolean)
 
+(defvar mew-addrbook-alias-not-learn-list nil)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cache
@@ -2442,7 +2497,7 @@ when '\\<mew-summary-mode-map>\\[mew-summary-addrbook-add]' in Summary mode."
 Currently, 'pgp-signature, 'pgp-encryption, 'pgp-signature-encryption,
 'pgp-encryption-signature, and nil are available. Since signature
 does not require receiver's public key, signature service may be
-appropriate for this value. "
+appropriate for this value."
   :group 'mew-privacy
   :type '(choice (const pgp-signature)
                  (const pgp-encryption)
@@ -2464,7 +2519,7 @@ according to 'mew-protect-privacy-encrypted-type'."
   "*A type of privacy protection for drafts replying encrypted
 messages. Currently, 'pgp-signature, 'pgp-encryption, 
 'pgp-signature-encryption, 'pgp-encryption-signature, 
-and nil are available. "
+and nil are available."
   :group 'mew-privacy
   :type '(choice (const pgp-signature)
                  (const pgp-encryption)
@@ -2525,7 +2580,7 @@ your public key as well as receivers' one."
 ;;; Sub-programs
 ;;;
 
-(defvar mew-prog-mewls       "mewls")
+(defvar mew-prog-mewl        "mewl")
 (defvar mew-prog-mime-encode "mewencode")
 (defvar mew-prog-mime-decode "mewencode")
 (defvar mew-prog-8bit        "mewencode")
@@ -2545,6 +2600,10 @@ your public key as well as receivers' one."
 (defvar mew-prog-ssh-arg      nil)
 
 (defvar mew-prog-uncompface  "uncompface")
+(defvar mew-prog-icontopbm   "icontopbm")
+(defvar mew-prog-pbmtoxbm    "pbmtoxbm")
+(defvar mew-prog-pbminvert   "pnminvert")
+
 (defvar mew-prog-shell       "/bin/sh")
 
 (defvar mew-prog-shell-arg   "-c")
@@ -2574,9 +2633,6 @@ w3 is used."
 ;;; X face
 ;;;
 
-(defvar mew-x-face-filter `(,mew-prog-uncompface "ikon2xbm"))
-(defvar mew-x-face-prog "display")
-(defvar mew-x-face-args '("-geometry" "+0+0"))
 (defcustom mew-x-face-file "~/.xface"
   "*If *non-nil* and the file exists, X-Face: fields is inserted."
   :group 'mew-summary
@@ -2677,11 +2733,13 @@ produced by broken MUAs."
 
 (defvar mew-no-warning-params ()) ;; downcased
 
-(defvar mew-warning-field-level 1
+(defcustom mew-warning-field-level 1
   "*If warning level of a field is greater than or equal to this value,
 the warning is displayed on X-Mew:.
 Level 1: syntax is correct, but not recommenced.
-Level 2: syntax error.")
+Level 2: syntax error."
+  :group 'mew-message
+  :type 'integer)
 
 (defcustom mew-encode-singlepart-hook nil
   "*Hook called before making a single part."
@@ -2692,7 +2750,7 @@ Level 2: syntax error.")
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
