@@ -78,7 +78,7 @@
    (cons mew-was-regex nil)) ;; regexp
   "*Replacement alist to simplify Subject: field body
 Each element is cons cell whos car is REGEXP to replace,
-cdr is new text. ")
+cdr is new text.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -98,10 +98,10 @@ cdr is new text. ")
 
 ;; In each folder
 
-(defvar mew-summary-cache-file ".mew-cache"
+(defvar mew-summary-cache-file ".mew-summary"
   "Cache file for Summary mode contents.")
 
-(defvar mew-summary-touch-file ".mew-touch"
+(defvar mew-summary-touch-file ".mew-mtime"
   "Time-stamp file for message folders.")
 
 ;; Under mew-conf-path
@@ -119,6 +119,7 @@ cdr is new text. ")
 
 (defvar mew-smtp-log-file "Smtplog")
 (defvar mew-nntp-log-file "Nntplog")
+(defvar mew-refile-log-file "Refilelog")
 
 ;; xxx Home
 
@@ -357,7 +358,44 @@ in draft buffer.")
     ;; Unknown suffix matches here and return the entry specified 
     ;; by mew-content-type.
     (nil             ".*"))
-  "(content-type filename <encoding> <prog> <icon>)")
+  "Database for MIME content type.
+This is a list of entries which consist of <content-type>, 
+<regular expression of file suffix>, <content-transfer-encoding>, 
+<symbol for visualizer>, <symbol for icon>.
+
+Here is an example:
+
+	(\"image/png\" \"\\\\.png$\" mew-b64 mew-prog-image mew-icon-image)
+
+This database is used both when visualizing and composing. For
+example, when visualizing, a visualizer is chosen according to
+its <content-type>. When composing, content-type is selected
+according to the suffix of the file to be attached with <regular
+expression of file suffix>.
+
+For <content-transfer-encoding>, 'nil', 'mew-b64', or 'mew-qp'
+should be specified. 'mew-b64' and 'mew-qp' means Base64 and
+Quoted-Printable, respectively. This is used as an appropriate
+content-transfer-encoding when composing.
+
+Each symbol specified at <symbol for visualizer> should have a value
+whose format is one of followings:
+
+- symbol
+	symbol is a name of Elisp function for inline.
+- (symbol1 symbol2)
+	symbol1 is a name of Elisp function for inline.
+	symbol2 is a name of Elisp function for external.
+- (string list nil-or-t)
+	string is a name of an external program.
+	list is a list of arguments for the external program.
+	nil-or-t: 
+		If nil, the external program is called synchronously.
+		If t, the external program is called asynchronously.
+
+If you want to add an entry for new content-type, please let the author
+know. Such an entry should be shared by all users and be included in
+the next version of Mew.")
 
 (defsubst mew-ctdb-by-ct (ct)
   (mew-assoc-match2 ct mew-mime-content-type 0))
@@ -775,7 +813,7 @@ For more detail, see mew-mark-put-mark and mew-mark-afterstep.")
   (nth case (assoc mark mew-mark-afterstep-spec)))
 
 (defvar mew-mark-spec
-  `((,mew-mark-review "review" 0 nil nil nil nil nil )
+  `((,mew-mark-review "review" 0 nil nil nil nil nil)
     (,mew-mark-multi  "multi"  0 nil nil nil nil nil)
     (,mew-mark-delete "delete" 1 nil t   nil mew-mark-exec-delete nil)
     (,mew-mark-unlink "unlink" 1 nil t   nil mew-mark-exec-unlink nil)
